/*====================================================================

Objectweb Browser framework
Copyright (C) 2000-2003 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.core.api;

import org.objectweb.util.browser.api.DropAction;

/**
 * Factory for DropAction
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * 
 * @version 0.1
 */
public interface DropActionFactory {

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    /**
     * The standard name of this interface to use
     * in the context of the Fractal framework.
     */
    static final String DROP_ACTION_FACTORY = DropActionFactory.class.getName();
    

    //==================================================================
    //
    // Public methods.
    //
    //==================================================================

    /**
     * Returns a DropAction associated to the param's object.
     *
     * @param object The object you want to find the associated action
     * @return The associated action.
     */
    public DropAction newDropAction(Object object);

    /**
     * Fixes the action class to execute
     * 
     * @param theClass The class to instantiate
     */
    public void setClassName(Class theClass);

    /**
     * Returns a description of the action
     * 
     * @return A description of the action
     */
    public String getLabel(); 

    /**
     * Fixes a description of the action
     * 
     * @param label The description of the action
     */
    public void setLabel(String label);

}

