/*===========================================================================

ObjectWeb Naming Context Framework 
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jrme Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.common;

/** The Browsr API's imports */
import java.io.Serializable;

import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.api.Tree;
import org.objectweb.util.browser.core.api.ExtendedEntry;
import org.objectweb.util.browser.core.api.ExtendedTreeView;
import org.objectweb.util.browser.core.api.TreeConfiguration;

/**
 * Offers a limited view on the tree
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class DefaultTreeView 
  implements ExtendedTreeView, Serializable 
{

    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /** The associated tree */
    protected DynamicTree tree_;

    /** The selected entry and his parent */
    protected Entry entry_, parent_;

    /** The handle object */
    protected Object handleObject_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default Constuctor
     * @param tree The associated tree
     */
    public DefaultTreeView(DynamicTree tree) {
        tree_ = tree;
        entry_ = tree_.getSelectedEntry();
        parent_ = tree_.getSelectedEntryParent();
    }

    // ==================================================================
    //
    // Protected methods.
    //
    // ==================================================================

    /**
     * @param entry The entry on which you want to obtain the value
     * @return The associated value of the entry
     */
    protected Object getValue(Entry entry) {
        if (entry != null)
            return (((ExtendedEntry) entry).getWrappedObject());
        return null;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

//    /**
//     * Refresh the selected node
//     */
//    public void refresh() {
//        tree_.refreshSelectedNode();
//    }

    /**
     * Gives the way to configure the tree
     * @return The entry to configure the tree
     */
    public TreeConfiguration getTreeConfiguration() {
        return (TreeConfiguration) tree_;
    }

    /**
     * Returns the current selected Entry.
     */
    public Entry getSelectedEntry() {
        return entry_;
    }

    /**
     * Returns the parent entry
     */
    public Entry getParentEntry() {
        return parent_;
    }

    /**
     * Returns the wrapped object of value of the associated Entry
     */
    public Object getSelectedObject() {
        return getValue(entry_);
    }

    /**
     * Returns the parent object
     */
    public Object getParentObject() {
        return getValue(parent_);
    }

    /**
     * Returns an object to manipulate the Tree.
     */
    public Tree
    getTree(){
        return (Tree)tree_;        
    }
    
}
