/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2003 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.icon;

/** The Java API */
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;

/**
 * Privides the default object icon with a specific color point
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class ObjectPointIcon 
    extends javax.swing.plaf.metal.MetalIconFactory.FileIcon16 {

    // ==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    /** The point color */
    protected Color c_;

    /** If a circle must be displayed */
    protected boolean displayCircle_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default Constructor
     * @param c The color of the point
     */
    public ObjectPointIcon(boolean displayCircle, Color c){
        displayCircle_ = displayCircle;
        c_ = c;
    }

    // ==================================================================
    //
    // Internals methods.
    //
    // ==================================================================

    /**
     * Draws the point in the default object icon
     */
    private void myPaint(Component c, Graphics g) {
        java.awt.Font oldFont = g.getFont();
        g.setFont(oldFont.deriveFont(java.awt.Font.BOLD));
        g.drawString("c",5,12);
        if(displayCircle_){
            g.setColor(c_);
            g.fillOval(10,11,5,5);
        }
        g.setFont(oldFont);
    }

    // ==================================================================
    //
    // Public methods surcharging the FileIcon16 class.
    //
    // ==================================================================

    public void paintIcon(Component c, Graphics g, int x, int y) {
        super.paintIcon(c,g,x,y);
        myPaint(c,g);   
    }

}
