/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.naming;

import org.objectweb.util.browser.api.Context;
import org.objectweb.util.browser.api.Entry;
import java.util.Vector;

/**
 * Basic implementation of the Context interface. It uses the Vector class. 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * @version 0.1
 */

public class DefaultContext implements Context {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The elements. */
    protected Vector elts_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Constructs a DefaultContextContainer.
     */
    public DefaultContext() {
        // Inits internal state.
        elts_ = new Vector();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    /**
     * Creates an Entry.
     *
     * @param value The value of the entry to create.
     * @param id The id of the name of the entry to create.
     *
     * @return A new Entry.
     *
     * @postcondition
     *              return.getOWValue() == value
     * and return.getOWName().getOWID() == id
     * and        return.getOWContext() == this;
     */
    protected Entry createEntry(Object value, String id) {
        return new DefaultEntry(value, new DefaultName(id), this);
    }

    // ==================================================================
    //
    // Public methods for interface Context.
    //
    // ==================================================================

    public Entry[] getEntries() {
        return (Entry[])elts_.toArray(new Entry[0]);
    }

}
