/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.gui.lib;

/** The browser's imports */
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.api.Wrapper;
import org.objectweb.util.browser.gui.api.ValidateReport;
import org.objectweb.util.browser.core.common.DynamicTree;

/** The java API's imports */
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.AbstractAction;
import javax.swing.Box;
import javax.swing.SwingConstants;
import javax.swing.JOptionPane;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;

/**
 * This class represents the panel which allows to choose an element in the tree.
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class TreeChooserBox 
    extends AbstractElementBox {

    //==================================================================
    //
    // Internal state.
    //
    //==================================================================

    /** The value of the label */
    protected String label_;

    /** The JButton used to browse the tree */
    protected JButton browseButton_;

    /** The Tree */
    protected DynamicTree tree_;

    /** */
    protected JTextField textField_;

    /** The choosen object */
    protected Object object_;

    //==================================================================
    //
    // Constructors.
    //
    //==================================================================

    /**
     * Default constructor
     * @param label The label of the box
     * @param tree The DynamicTree to use
     */     
    public TreeChooserBox(String label, DynamicTree tree) {
        this(label,tree,true);   
    }
    
    /**
     * Constructor which specifies if the value is mandatory or not
     * @param label The label of the box
     * @param tree The DynamicTree to use
     * @param isMandatory If the value is mandatory : TRUE, else : FALSE
     */
    public TreeChooserBox(String label, DynamicTree tree, boolean isMandatory) {
        super(isMandatory);
        label_ = label;
        tree_ = tree;
        tree_.setPopupEnabled(false);
        add(Box.createHorizontalGlue());
        JLabel browserLabel = new JLabel(label + ": ", SwingConstants.RIGHT);
        browserLabel.setAlignmentX(Component.RIGHT_ALIGNMENT);
        browserLabel.setAlignmentY(Component.CENTER_ALIGNMENT);
        add(browserLabel);
        add(Box.createHorizontalStrut(5));
        textField_ = new JTextField();
        textField_.setEditable(false);
        textField_.setPreferredSize(new Dimension(200, 20));
        textField_.setMaximumSize(new Dimension(200, 20));
        add(textField_);
        add(Box.createHorizontalStrut(5));
        browseButton_ = new JButton(new BrowseAction(this));
        browseButton_.setPreferredSize(new Dimension(20, 20));
        browseButton_.setMaximumSize(new Dimension(20, 20));
        add(browseButton_);
    }

    // ==================================================================
    //
    //  Public methods for ElementBox.
    //
    // ==================================================================

    /**
     * Validates the content of the ElementBox. Indicates if the different value are acceptables or not.
     * @return The corresponding ValidateReport.
     */
    public ValidateReport validateBox(){
        if(isMandatory_){
            if(getObject()==null)
                return new DefaultValidateReport(false,"The \"" + label_ + "\" value is mandatory");
        }
        return new DefaultValidateReport();
    }

    /**
     * Invokes just before displaying.
     */
    public void preInitialize(){
        if(tree_!=null)
            tree_.setPopupEnabled(false);
    }
    
    /**
     * Invokes just before removing.
     */
    public void postInitialize(){
        if(tree_!=null)
            tree_.setPopupEnabled(true);
    }
    
    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================

    /** Returns the chosen Object */
    public Object getObject() {
        if(object_!=null && Wrapper.class.isAssignableFrom(object_.getClass()))
            return ((Wrapper)object_).getWrapped();
        return object_;
    }

    /**
     * Action which 
     */
    protected class BrowseAction extends AbstractAction {

        /** The parent of the panel */
        protected Component parent_;

        /**
         * Default constructor
         */
        protected BrowseAction(Component parent) {
            super("...", null);
            parent_ = parent;
        }

        public void actionPerformed(java.awt.event.ActionEvent ae) {
            JPanel treePanel = new JPanel();
            treePanel.setLayout(new GridLayout(1, 0));
            treePanel.setPreferredSize(new Dimension(450, 350));
            treePanel.add(new JScrollPane(tree_));
            int result = JOptionPane.showOptionDialog((Component)ae.getSource(), treePanel, "Select an object", JOptionPane.OK_CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE, null, null, null);
            if (result == 0) {
                Entry entry = tree_.getSelectedEntry();
                if (entry != null) {
                    Object o = entry.getValue();
                    if (o != null) {
                        object_ = o;
                        textField_.setText(entry.getName().toString());               
                    }
                }
            }
        }

    }

}
