/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): offroy ________________________.
Contributor(s): ______________________________________.

====================================================================*/

package org.omg.lifl.eclipse.plugin.project.OpenCCM.utils;

import java.util.Iterator;

import org.apache.tools.ant.types.FilterSet;
import org.omg.lifl.eclipse.plugin.project.OpenCCM.ProjectCreationWizardPage;

/**
 * This class ...
 *
 * @author offroy@lifl.fr
 *
 * @version 0.1
 */
public class BuildPropertySet {
    private String ORBName;
    private String importPath;
    private String orbPath;
    private String installOpenCCM;
    private String transactionPath;
    private String transactionJar;
    private String nsPath;
    private String cppPath;
    private String javaccPath;
    // TODO make a Class for CompilOption
    private String compilOptionDebug;
    private String compilOptionDeprecated;
    private String compilOptionOptimize;

    private static final String undefined = "unused";

    public BuildPropertySet(ProjectCreationWizardPage page) {
        // get all needed properties for creating build.properties OpenCCM config file
        if (page.getCvsFSchooser().getCheckGroup().isSubGroupVisible())
            importPath = page.getCvsFSchooser().get_CVSsrcFSPathValue();
        if (page.getOpenccmConfigChooser().get_ORBpreference().isEnable())
            orbPath =
                page.getOpenccmConfigChooser().get_ORBpreference().get_Name();
        ORBName = page.getOpenccmConfigChooser().get_ORBName();
        if (page.getOpenccmConfigChooser().get_InstallOpenCCM().isEnable())
            installOpenCCM =
                page.getOpenccmConfigChooser().get_InstallOpenCCM().get_Name();
        if (page
            .getOpenccmConfigChooser()
            .get_TransactionpreferenceHome()
            .isEnable())
            transactionPath =
                page
                    .getOpenccmConfigChooser()
                    .get_TransactionpreferenceHome()
                    .get_Name();
        if (page
            .getOpenccmConfigChooser()
            .get_TransactionpreferenceJar()
            .isEnable())
            transactionJar =
                page
                    .getOpenccmConfigChooser()
                    .get_TransactionpreferenceJar()
                    .get_Name();
        if (page.getOpenccmConfigChooser().get_NSDirPreference().isEnable())
            nsPath =
                page.getOpenccmConfigChooser().get_NSDirPreference().get_Name();
        if (page
            .getOpenccmConfigChooser()
            .get_PreprocessorPreference()
            .isEnable())
            cppPath =
                page
                    .getOpenccmConfigChooser()
                    .get_PreprocessorPreference()
                    .get_Name();
        if (page.getOpenccmConfigChooser().get_JavaccPreference().isEnable())
            javaccPath =
                page
                    .getOpenccmConfigChooser()
                    .get_JavaccPreference()
                    .get_Name();
        if (page
            .getOpenccmConfigChooser()
            .get_CompilationOptionDebugPreference()
            .isEnable())
            compilOptionDebug =
                page
                    .getOpenccmConfigChooser()
                    .get_CompilationOptionDebugPreference()
                    .get_Name();
        if (page
            .getOpenccmConfigChooser()
            .get_CompilationOptionDeprecationPreference()
            .isEnable())
            compilOptionDeprecated =
                page
                    .getOpenccmConfigChooser()
                    .get_CompilationOptionDeprecationPreference()
                    .get_Name();
        if (page
            .getOpenccmConfigChooser()
            .get_CompilationOptionOptimizePreference()
            .isEnable())
            compilOptionOptimize =
                page
                    .getOpenccmConfigChooser()
                    .get_CompilationOptionOptimizePreference()
                    .get_Name();
    }

    public void createFilter(
        FilterSet fs,
        String tagTab[],
        String value[]) {
        TagGroup tag = new TagGroup(tagTab, value);
        Iterator it = tag.getTheTags().keySet().iterator();
        while (it.hasNext()) {
            String key = (String) it.next();
            fs.addFilter(key, (String) tag.getTheTags().get(key));
        }
    }

    

    /**
     * @param fs
     */
    public void createOpenCCMFilter(FilterSet fs) {
        // set the structure of build.properties file for Ant copy replacement
        String tagTab[] =
            {
                "ORB.NAME",
                "INSTALL.OPENCCM.DIR",
                "ORB.HOME.DIR",
                "NS.HOME.DIR",
                "OTS.HOME.DIR",
                "OTS.JAR",
                "CPP.COMPILER",
                "JAVACC.DIR",
                "BUILD.DEBUG",
                "BUILD.DEPRECATION",
                "BUILD.OPTIMIZE" };
        String value[] =
            {
                getValue(this.getORBName()),
                getValue(this.getInstallOpenCCM()),
                getValue(this.getOrbPAth()),
                getValue(this.getNsPath()),
                getValue(this.getTransactionPath()),
                getValue(this.getTransactionJar()),
                getValue(this.getCppPath()),
                getValue(this.getJavaccPath()),
                getValue(this.getCompilOptionDebug()),
                getValue(this.getCompilOptionDeprecated()),
                getValue(this.getCompilOptionOptimize())};

        createFilter(fs,tagTab,value);
    }

    /**
     * @param fs
     */
    public void createClasspathFilter(FilterSet fs) {
        // set the structure of classpath file for Ant copy replacement
        String tagTab[] =
            {
                "ORB.HOME.DIR",
                "NS.HOME.DIR",
                "OTS.HOME.DIR",
                "OTS.JAR",
                "JAVACC.DIR"};
        String value[] =
            {
                getValue(this.getOrbPAth()),
                getValue(this.getNsPath()),
                getValue(this.getTransactionPath()),
                getValue(this.getTransactionJar()),
                getValue(this.getJavaccPath())};

        createFilter(fs,tagTab,value);
    }

    /**
     * @param show 
     * @return a string representing the BuildPropertySet state
     */
    public String showDebug(boolean show) {
        if (show) {
            return "generate build.properties with this parameter :\n"
                + "import                :"
                + getValue(this.getImportPath())
                + "\n"
                + "ORB directory         :"
                + getValue(this.getOrbPAth())
                + "\n"
                + "OpenCCM install dir   :"
                + getValue(this.getInstallOpenCCM())
                + "\n"
                + "Transaction directory :"
                + getValue(this.getTransactionPath())
                + "\n"
                + "Transaction Jar dir   :"
                + getValue(this.getTransactionJar())
                + "\n"
                + "NS home directory     :"
                + getValue(this.getNsPath())
                + "\n"
                + "CPP directory         :"
                + getValue(this.getCppPath())
                + "\n"
                + "JavaCC home dir       :"
                + getValue(this.getJavaccPath())
                + "\n"
                + "Compil Opt    Debug   :"
                + getValue(this.getCompilOptionDebug())
                + "\n"
                + "Compil Opt Deprecated :"
                + getValue(this.getCompilOptionDeprecated())
                + "\n"
                + "Compil Opt Optimize   :"
                + getValue(this.getCompilOptionOptimize());
        } else
            return "don't generate build.properties";
    }

    /**
     * @param value
     */
    private String getValue(String value) {
        return ((value != null && value.compareTo("") != 0) ? value : "unused");
    }

    /**
     * @return
     */
    public String getCompilOptionDebug() {
        return compilOptionDebug;
    }

    /**
     * @return
     */
    public String getCompilOptionDeprecated() {
        return compilOptionDeprecated;
    }

    /**
     * @return
     */
    public String getCompilOptionOptimize() {
        return compilOptionOptimize;
    }

    /**
     * @return
     */
    public String getCppPath() {
        return cppPath;
    }

    /**
     * @return
     */
    public String getImportPath() {
        return importPath;
    }

    /**
     * @return
     */
    public String getJavaccPath() {
        return javaccPath;
    }

    /**
     * @return
     */
    public String getNsPath() {
        return nsPath;
    }

    /**
     * @return
     */
    public String getOrbPAth() {
        return orbPath;
    }

    /**
     * @return
     */
    public String getTransactionJar() {
        return transactionJar;
    }

    /**
     * @return
     */
    public String getTransactionPath() {
        return transactionPath;
    }

    /**
     * @param string
     */
    public void setCompilOptionDebug(String string) {
        compilOptionDebug = string;
    }

    /**
     * @param string
     */
    public void setCompilOptionDeprecated(String string) {
        compilOptionDeprecated = string;
    }

    /**
     * @param string
     */
    public void setCompilOptionOptimize(String string) {
        compilOptionOptimize = string;
    }

    /**
     * @param string
     */
    public void setCppPath(String string) {
        cppPath = string;
    }

    /**
     * @param string
     */
    public void setImportPath(String string) {
        importPath = string;
    }

    /**
     * @param string
     */
    public void setJavaccPath(String string) {
        javaccPath = string;
    }

    /**
     * @param string
     */
    public void setNsPath(String string) {
        nsPath = string;
    }

    /**
     * @param string
     */
    public void setOrbPAth(String string) {
        orbPath = string;
    }

    /**
     * @param string
     */
    public void setTransactionJar(String string) {
        transactionJar = string;
    }

    /**
     * @param string
     */
    public void setTransactionPath(String string) {
        transactionPath = string;
    }

    /**
     * @return
     */
    public String getInstallOpenCCM() {
        return installOpenCCM;
    }

    /**
     * @param string
     */
    public void setInstallOpenCCM(String string) {
        installOpenCCM = string;
    }

    /**
     * @return
     */
    public String getORBName() {
        return ORBName;
    }

    /**
     * @param string
     */
    public void setORBName(String string) {
        ORBName = string;
    }

}
