/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): offroy ________________________.
Contributor(s): ______________________________________.

====================================================================*/
/*
 * Created on 23 mai 2003 by jerome OFFROY (offroy@lifl.fr)
 *
 */

package org.omg.lifl.eclipse.plugin.project.OpenCCM.utils;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.omg.lifl.eclipse.plugin.project.utils.AttributeManager;
import org.omg.lifl.eclipse.plugin.project.utils.SWT.CheckGroup;
import org.omg.lifl.eclipse.plugin.project.utils.SWT.DirectoryChooser;
import org.omg.lifl.eclipse.plugin.project.utils.SWT.DirectoryChooserCheckTextSetter;
import org.omg.lifl.eclipse.plugin.project.utils.SWT.SetterGroupPreference;

/**
 * This class deals with OpenCCM build.properties configuration file
 *      - store preferences
 *      - create SWT widget to change preferences
 *
 * @author LIFL
 *
 * @version 0.1
 */
public class EclipseConfigChooser {

    private String groupTitle;

    private Group group_ORB_Chooser;
    private CheckGroup checkGroup;

    private SetterGroupPreference _ImportWorkbenchPreference;

    /**
     * Construct the preference needed for configuring the 
     * OpenCCM build.properties configuration file
     * @param elem the element which contains needed configuration
     */
    public EclipseConfigChooser(IConfigurationElement elem) {
        groupTitle = AttributeManager.getAttribute(elem, "EclipseConfigGroupLabel");
        // to set Eclipse Workbench Preference
        _ImportWorkbenchPreference =
            AttributeManager.getAllAttribute(elem, "ImportPreference");

    }

    /**
     * create a SWT group which a check
     * to set OpenCCM build.properties configuration file
     * @param composite the parent Composite 
     */
    public void viewToSetEclipseConfig(Composite composite) {
        checkGroup = new CheckGroup(composite, groupTitle);

        Group parent_group = new Group(checkGroup.getGroup(), SWT.NONE);
        parent_group.setLayout(new RowLayout(SWT.VERTICAL));
        checkGroup.setSubGroup(parent_group);

        // area to choose to import or not Eclipse Workbench preferences (with Backup)
        DirectoryChooser dcImportWorkbenchPreference =
            new DirectoryChooserCheckTextSetter(parent_group, _ImportWorkbenchPreference);
    }

    /**
     * @return the main SWT CheckGroup
     */
    public CheckGroup getCheckGroup() {
        return checkGroup;
    }

    /**
     * @param group is used to set main SWT CheckGroup 
     */
    public void setCheckGroup(CheckGroup group) {
        checkGroup = group;
    }

    /**
     * @param preference SetterGroupPreference to set
     * the workbench preference file to import
     */
    public void set_ImportWorkbenchPreference(SetterGroupPreference preference) {
        _ImportWorkbenchPreference = preference;
    }

    /**
     * @return
     */
    public SetterGroupPreference get_ImportWorkbenchPreference() {
        return _ImportWorkbenchPreference;
    }

}
