/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.coach.tcl;

import tcl.lang.*;
import org.coach.idltree.*;
import java.io.*;
import java.util.*;
import org.omg.Components.*;
import org.coach.util.IorPrinter;
import org.coach.tracing.service.ThreadContext;
import org.coach.tracing.service.ThreadContext;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;

public class TclLib {
    private Interp tclInterp;
    private org.omg.CORBA.ORB orb;
    private org.omg.CosNaming.NamingContext rootContext;

    public TclLib(Interp tclInterp){
        orb = org.objectweb.ccm.CORBA.TheORB.getORB();
        try {
            org.omg.CORBA.Object obj = orb.resolve_initial_references("NameService");
            rootContext = org.omg.CosNaming.NamingContextHelper.narrow(obj);
        } catch (org.omg.CORBA.ORBPackage.InvalidName e1) {
            // NS is not started
        }

        this.tclInterp = tclInterp;

        try{
            tclInterp.setVar("tclShell", tcl.lang.ReflectObject.newInstance(tclInterp, TclLib.class, this), 0);
            tclInterp.eval("proc ccmFindObject {objectName} {global tclShell; return [$tclShell findObject $objectName]}");
            tclInterp.eval("proc ccmGetFacets {componentNode} {global tclShell; set Facets [$tclShell listFacets $componentNode]; for {set i 0} {$i < [$Facets length]} {incr i} {lappend lstr [$Facets get $i]}; return $lstr}");
            tclInterp.eval("proc ccmGetOperations {facetRef} {global tclShell; set Methods [$tclShell listMethods2 $facetRef]; for {set i 0} {$i < [$Methods length]} {incr i} {lappend lstr [$Methods get $i]}; return $lstr}");
            tclInterp.eval("proc ccmProvide {componentNode facetName} {global tclShell; return [$tclShell getFacet $componentNode $facetName]}");
            tclInterp.eval("proc ccmInvoke {componentNode ccmOperation} {global tclShell; return [$tclShell invoke $componentNode $ccmOperation]}");
            tclInterp.eval("proc ccmGetOperation {facetRef operationName} {global tclShell; return [$tclShell getXmlOperation $facetRef $operationName]} ");
            tclInterp.eval("proc ccmCreateIdlNode {rid} {global tclShell; return [$tclShell newType $rid]} ");
            tclInterp.eval("proc ccmFromXml {xmlString} {global tclShell; return [$tclShell xmlToType $xmlString]} ");
            tclInterp.eval("proc ccmToXml {idlNode} {global tclShell; return [$tclShell typeToXml $idlNode]} ");
            tclInterp.eval("proc ccmGetValue {idlNode {path \"\"}} {global tclShell; $tclShell getValue $idlNode $path} ");
            tclInterp.eval("proc ccmSetValue {idlNode value {path \"\"}} {global tclShell; $tclShell setValue $idlNode $path $value} ");
            tclInterp.eval("proc ccmSetNode  {idlNode value {path \"\"}} {global tclShell; $tclShell setNode  $idlNode $path $value} ");
            tclInterp.eval("proc ccmGetNode {idlNode path} {global tclShell; $tclShell getNode $idlNode $path} ");
            tclInterp.eval("proc ccmGetProperty {propName} {global tclShell; return [$tclShell getProperty $propName]} ");
            tclInterp.eval("proc ccmSetProperty {propName propValue} {global tclShell; return [$tclShell setProperty $propName $propValue]} ");
            tclInterp.eval("proc ccmFieldValues {node} {global tclShell; return [$tclShell printNode $node]}");
            tclInterp.eval("proc ccmFields {node} {global tclShell; return [$tclShell printFields $node]}");
            tclInterp.eval("proc ccmValues {node} {global tclShell; return [$tclShell printValues $node]}");
            tclInterp.eval("proc ccmFromIor {ior} {global tclShell; return [$tclShell fromIor $ior]} ");
            tclInterp.eval("proc ccmCreateReply {id method} {global tclShell; return [$tclShell createXmlReply $id $method]}");
            tclInterp.eval("proc ccmCreateOperation {id method} {global tclShell; return [$tclShell createXmlOperation $id $method]}");
            tclInterp.eval("proc ccmSetTrailLabel {label} {global tclShell; $tclShell setTrailLabel $label} ");
            tclInterp.eval("proc ccmGetTrailLabel {} {global tclShell; return [$tclShell getTrailLabel]} ");
            
            tclInterp.eval("proc ccmHelp {} {global tclShell; return [$tclShell getHelp]} ");

            Properties props = System.getProperties();
            for (Enumeration e = props.keys(); e.hasMoreElements();) {
                String key = (String)e.nextElement();
                if (System.getProperty(key).endsWith("\\")) {
                    tclInterp.eval("set " + key + " {" + System.getProperty(key) + " }");
                } else {
                    tclInterp.eval("set " + key + " {" + System.getProperty(key) + "}");
                }
            }             
        } catch (tcl.lang.TclException e) {
           System.out.println(tclInterp.getResult());
        }
    }
    
    public void setOrb(org.omg.CORBA.ORB orb) {
        if (this.orb == null) {
            this.orb = orb;
        }
    }

    public String getHelp() {
        String h = "Help for COACH TCL Scripting\n" +
            " Procedures:\n" +
            " ccmFindObject {objectName} \n" +            
            " ccmGetFacets {componentNode} \n" +
            " ccmGetOperations {facetNode} \n" +
            " ccmProvide {componentNode facetName} \n" +
            " ccmInvoke {componentNode ccmOperation} \n" +
            " ccmGetOperation {facetNode operationName} \n" +
            " ccmCreateOperation {facetId operation} \n" +
            " ccmCreateReply {facetId operation} \n" +
            " ccmCreateIdlNode {id} \n" +
            " ccmFromXml {xmlString} \n" +
            " ccmToXml {idlNode} \n" +
            " ccmGetValue {idlNode {path \"\"}} \n" +
            " ccmSetValue {idlNode value {path \"\"}} \n" +
            " ccmSetNode {idlNode value {path \"\"}} \n" +
            " ccmGetNode {idlNode path} \n" +
            " ccmFieldValues {node} \n" +
            " ccmFields {node} \n" +
            " ccmValues {node} \n" +            
            " ccmFromIor {ior} \n" +
            " ccmSetTrailLabel {label} \n" +
            " ccmGetTrailLabel {} \n" +
            " ccmGetProperty {propName} \n" +
            " ccmSetProperty {propName propValue} ";
        return h;
    }

    public void runScript(String src) {
        try {
            tclInterp.eval("source " + src);
        } catch (tcl.lang.TclException e) {
           System.out.println(tclInterp.getResult());
        }
    }
    
    public void setTrailLabel(String label) {
        ThreadContext tc = ThreadContext.getCurrentThreadContext();
        tc.setTrailLabel(label);
    }

    public String getTrailLabel() {
        ThreadContext tc = ThreadContext.getCurrentThreadContext();
        return tc.getTrailLabel();
    }
        
    public IdlInterface findObject(String name) {
        try {
            org.omg.CORBA.Object c = null;
            c = rootContext.resolve(splitName(name));
            String ior = orb.object_to_string(c);
            IorPrinter iorPrinter = new IorPrinter(ior);
            String id = iorPrinter.getTypeId();
            IdlInterface interfaceNode = new IdlInterface(id, false);
            interfaceNode.setValue(ior);
            return interfaceNode;
        } catch (Exception ex) {
            throw new RuntimeException("ccmFindObject: " + ex.toString());
        }
    }

    public void tclPuts(String s) {
        System.out.println(s);
    }
    
    public IdlInterface fromIor(String ior) {
        try {
            IorPrinter iorPrinter = new IorPrinter(ior);
            String id = iorPrinter.getTypeId();
            IdlInterface interfaceNode = new IdlInterface(id, false);
            interfaceNode.setValue(ior);
            return interfaceNode;
        } catch (Exception ex) {
            throw new RuntimeException("ccmFromIor: " + ex.toString());
        }
    }
    
    public IdlInterface getFacet(IdlNode componentNode, String v) {
        if (!(componentNode instanceof IdlObject)) {
            throw new RuntimeException("Illegal argument: component IdlNode expected");
        }
        try {
            CCMObject c = null;
            try {
                c = CCMObjectHelper.narrow(orb.string_to_object(componentNode.getValue()));
            } catch (Exception ex) {
                throw new RuntimeException(componentNode.getId() + " is not a component");
            }
            org.omg.CORBA.Object f = c.provide_facet(v);
            String ior = orb.object_to_string(f);
            IorPrinter iorPrinter = new IorPrinter(ior);
            String id = iorPrinter.getTypeId();
            IdlInterface interfaceNode = new IdlInterface(id, false);
            interfaceNode.setValue(ior);
            return interfaceNode;
        } catch (Exception ex) {
            throw new RuntimeException("ccmGetFacet: " + ex.toString());
        }
    }

    public String printFields(IdlNode node) {
        return node.printFields(false);            
    }

    public String printNode(IdlNode node) {
        return node.printFields(true);            
    }

    public String printValues(IdlNode node) {
        return node.printValues();     
    }
        
    public String getProperty(String propName) {
        String p = System.getProperty(propName);
        if (p == null) {
            throw new RuntimeException("No such property: " + propName);
        } else {
            return p;
        }
    }

    public void setProperty(String propName, String propValue) {
        try {
            System.setProperty(propName, propValue);
        } catch (Exception ex) {
            throw new RuntimeException("ccmSetProperty: " + ex.toString());
        }
    }
                    
    public String[] listFacets(IdlNode componentNode) {
        if (!(componentNode instanceof IdlObject)) {
            throw new RuntimeException("Illegal argument: component IdlNode expected");
        }
        CCMObject c = null;
        try {
            c = CCMObjectHelper.narrow(orb.string_to_object(componentNode.getValue()));
        } catch (Exception ex) {
            throw new RuntimeException(componentNode.getId() + " is not a component");
        }
        try {
            //ToDo:
//            FacetDescriptions facetDescriptions = c.get_all_facets();
            return new String[0];
        } catch (Exception ex) {
            throw new RuntimeException("ccmGetFacets: " + ex.toString());
        }
    }
    
    public String[] listMethods(String id) {
        try {
            IdlInterface itf = new IdlInterface(id, false);
            return itf.getOperations();
        } catch (Exception ex) {
            throw new RuntimeException("ccmGetOperations: " + ex.toString());
        }
    }

    public String[] listMethods2(IdlNode objectNode) {
        if (!(objectNode instanceof IdlObject)) {
            throw new RuntimeException("Illegal argument: object IdlNode expected");
        }
        try {
            String ior = objectNode.getValue();
            IorPrinter iorPrinter = new IorPrinter(ior);
            String id = iorPrinter.getTypeId();           
            IdlInterface itf = new IdlInterface(id, false);
            return itf.getOperations();
        } catch (Exception ex) {
            throw new RuntimeException("ccmGetOperations: " + ex.toString());
        }
    }
 
    public IdlOperation getXmlOperation(IdlNode objectNode, String method) {
        if (!(objectNode instanceof IdlObject)) {
            throw new RuntimeException("Illegal argument: object IdlNode expected");
        }
        try {
            String ior = objectNode.getValue();
            IorPrinter iorPrinter = new IorPrinter(ior);
            String id = iorPrinter.getTypeId(); 
            return new IdlOperation(id, method);
        } catch (Exception ex) {
            throw new RuntimeException("ccmGetOperation: " + ex.toString());
        }
    }

    public IdlReply createXmlReply(String id, String method) {
        return new IdlReply(id, method);
    }

    public IdlOperation createXmlOperation(String id, String method) {
        return new IdlOperation(id, method);
    }
        
    public IdlNode newType(String id) {
        try {
            return IdlNode.create(XmlNode.type(id));
        } catch (Exception ex) {
            throw new RuntimeException("ccmNewIdlNode: " + ex.toString());
        }
    }

    public IdlNode xmlToType(String xml) {
        try {
            return XmlNode.getIdlNodeXml(xml);
        } catch (Exception ex) {
            throw new RuntimeException("ccmFromXml: " + ex.toString());
        }
    }

    public String typeToXml(IdlNode node) {
        try {
          XmlWriter w = new XmlWriter();
          XmlNode.write(node, w);
          return w.toString();
        } catch (Exception ex) {
            throw new RuntimeException("ccmToXml: " + ex.toString());
        }
    }
                
    public IdlNode getNode(IdlNode node, String path) {
        return node.getNode(path);
    }
    
    public void setNode(IdlNode node, String path, IdlNode v) {
        if (path.equals("exception") && node instanceof IdlReply) {
            ((IdlReply)node).setException(v.toAny());
        } else {           
            IdlNode n = node.getNode(path);
            n.replace(v);
        }
    }
    
    public void setValue(IdlNode node, String path, String v) {
        IdlNode n = getNode(node, path);
        if (n.isMutable()) {
            n.setValue(v);
        } else {
            throw new RuntimeException("IdlNode " + node.toString() + " is not mutable.");
        }
    }

    public boolean isLeaf(IdlNode node) {       
        return node.isLeaf();
    }
    
    public String getValue(IdlNode node, String path) {
        IdlNode n = getNode(node, path);
        return n.getValue();
    }
    
    public IdlReply invoke(IdlNode targetNode, IdlOperation opr) {
        if (!(targetNode instanceof IdlObject)) {
            throw new RuntimeException("Illegal argument: object IdlNode expected");
        }
        try {
            return opr.invoke(orb, orb.string_to_object(targetNode.getValue()));
        } catch (Exception ex) {
ex.printStackTrace();
            throw new RuntimeException("ccmInvoke: " + ex.toString());
        }
    }
                
    private NameComponent[] splitName(String name) {
        Vector v = new Vector();
        StringTokenizer st = new StringTokenizer(name, "/");
        while (st.hasMoreTokens()) {
            String element = st.nextToken();
            int index = element.indexOf(".");
            if (index != -1) {
                v.addElement(new NameComponent(element.substring(0,index), element.substring(index + 1)));
            } else {
                // HJB: allow names without a kind field
                v.addElement(new NameComponent(element, ""));
            }
        }
        NameComponent[] result = new NameComponent[v.size()];
        v.copyInto(result);
        return result;
    }
}