/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Sylvain Leblanc_______________________.

====================================================================*/

package org.objectweb.ccm.CORBA;

import org.omg.CORBA.TCKind;
/**
 * This class provides CORBA::TypeCode util functions.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 *
 * @version 0.4
 */

public abstract class TypeCodeUtils
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  The singleton CORBA::TypeCode for the OMG IDL void type.
     **/
    static private org.omg.CORBA.TypeCode tc_void_ = null;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    private
    TypeCodeUtils()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     *  Obtain the CORBA::TypeCode for the OMG IDL void type.
     *
     *  @return The CORBA::TypeCode for the OMG IDL void type.
     */
    public static org.omg.CORBA.TypeCode
    getTC_void()
    {
        if(tc_void_ == null)
        {
            tc_void_ = TheORB.getORB().get_primitive_tc(
                                                        org.omg.CORBA.TCKind.tk_void);
        }
        return tc_void_;
    }

    /**
     **  Obtain the orignal CORBA::TypeCode.
     **
     **  @param tc The aliased CORBA::TypeCode.
     **
     **  @return The unaliased CORBA::TypeCode.
     **/
    public static org.omg.CORBA.TypeCode
    getOrigType(org.omg.CORBA.TypeCode tc)
    {
        try
        {
            while(tc.kind() == org.omg.CORBA.TCKind.tk_alias)
            {
                tc = tc.content_type();
            }
            return tc;
        }
        catch(org.omg.CORBA.TypeCodePackage.BadKind exc)
        {
            // Wrap the CORBA::TypeCode::BadKind exception.
            throw new UserExceptionWrapper(exc);
        }

    }

    /**
     * Returns a String representation of a TypeCode. This string
     * representation is based on OMG IDL syntax.
     *
     * @param tc The type code.
     *
     * @return A string representation of the given
     * <code>org.omg.CORBA.TypeCode</code>.
     */
    public static String
    toStringValue(org.omg.CORBA.TypeCode tc)
    {
        TCKind tk = tc.kind();

        try {
            if (tk == TCKind.tk_null)
                return "null";
            else if (tk == TCKind.tk_void)
                return "void";
            else if (tk ==  TCKind.tk_short)
                return "short";
            else if (tk == TCKind.tk_ushort)
                return "unsigned short";
            else if (tk == TCKind.tk_long)
                return "long";
            else if (tk == TCKind.tk_ulong)
                return "unsigned long";
            else if (tk == TCKind.tk_longlong)
                return "long long";
            else if (tk == TCKind.tk_ulonglong)
                return "unsigned long long";
            else if (tk == TCKind.tk_float)
                return "float";
            else if (tk == TCKind.tk_double)
                return "double";
            else if (tk == TCKind.tk_longdouble)
                return "long double";
            else if (tk == TCKind.tk_boolean)
                return "boolean";
            else if (tk == TCKind.tk_char)
                return "char";
            else if (tk == TCKind.tk_wchar)
                return "wchar";
            else if (tk == TCKind.tk_octet)
                return "octet";
            else if (tk == TCKind.tk_any)
                return "any";
            else if (tk == TCKind.tk_Principal)
                return "::CORBA::Principal";
            else if (tk == TCKind.tk_TypeCode)
                return "::CORBA::TypeCode";            
            else if (tk == TCKind.tk_objref)
                return "Object";
            
            // tk_local_interface not managed 
            // cf. ORBacus 4.1
            else if ( (tk == TCKind.tk_struct) ||
                      (tk == TCKind.tk_union) ||
                      (tk == TCKind.tk_enum) ||
                      (tk == TCKind.tk_except) ||
                      (tk == TCKind.tk_alias) ||
                      (tk == TCKind.tk_value_box) ||
                      (tk == TCKind.tk_value) ||
                      (tk == TCKind.tk_native) ||
                      (tk == TCKind.tk_abstract_interface) )
            {
                return tc.name();
            }
            else if (tk == TCKind.tk_string)
            {
                int l = tc.length();
                if (l==0)
                    return "string";
                return "string<"+l+">";
            }
            else if (tk == TCKind.tk_wstring)
            {
                int l = tc.length();
                if (l==0)
                    return "wstring";
                return "wstring<"+l+">";
            }
            else if (tk == TCKind.tk_sequence)
            {
                int l = tc.length();
                String str = toStringValue(tc.content_type());
                if (l==0)
                    return "sequence<"+str+">";
                return "sequence<"+str+","+l+">";
            }
            else if (tk == TCKind.tk_array)
            {
                String str = toStringValue(tc.content_type());
                return str;
            }
            else if (tk == TCKind.tk_fixed)
            {
                return "fixed<"+tc.fixed_digits()+","+tc.fixed_scale()+">";
            }
            else
                return "";
        } catch (org.omg.CORBA.TypeCodePackage.BadKind ex) {
            // cannot happened because "well used"
            return ""; 
        }
    }		
}
