/*====================================================================

created on 4 mars 2003

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2001-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Hameau Fabien.
Contributor(s): ______________________________________.

*/

package org.objectweb.ccm.Containers.MetaInformation;

import org.omg.Components.CCMHome;

/**
 * @author hameau
 */
public class HomeInstanceImpl
extends InstanceImpl 
implements HomeInstance {
  
  protected HomeType              _home_type;
  protected ComponentInstanceList _component_instances;
  protected CCMHome               _home_ref;
  protected String                _home_instance_factory_entrypt;
  protected OperationInstanceList _operation_instance_list;
  protected InterfaceInstance     _equivalent_interface;
  
  public HomeInstanceImpl()
  {
    _home_instance_factory_entrypt = null;
    _component_instances  = new ComponentInstanceListImpl();
    _equivalent_interface = null;
    _home_ref = null;
    _home_type = null;
    _operation_instance_list = new OperationInstanceListImpl();
  }
  
  /* (non-Javadoc)
   * @see org.objectweb.ccm.Containers.HomeInstanceOperations#home_ref()
   */
  public CCMHome home_ref()
  {
    return _home_ref;
  }
  /* (non-Javadoc)
   * @see org.objectweb.ccm.Containers.HomeInstanceOperations#home_ref(org.omg.Components.CCMHome)
   */
  public void home_ref(CCMHome value)
  {
    _home_ref = value;
  }
  /* (non-Javadoc)
   * @see org.objectweb.ccm.Containers.HomeInstanceOperations#home_instance_factory_entrypt()
   */
  public String home_instance_factory_entrypt()
  {
    return _home_instance_factory_entrypt;
  }
  /* (non-Javadoc)
   * @see org.objectweb.ccm.Containers.HomeInstanceOperations#home_instance_factory_entrypt(java.lang.String)
   */
  public void home_instance_factory_entrypt(String value)
  {
    _home_instance_factory_entrypt = value;
  }
  
  public ComponentInstanceList component_instance_list()
  {
    return _component_instances;
  }
  /* (non-Javadoc)
   * @see org.objectweb.ccm.Containers.MetaInformation.HomeInstanceOperations#component_instance_list(org.objectweb.ccm.Containers.MetaInformation.ComponentInstanceList)
   */
  public void component_instance_list(ComponentInstanceList value)
  {
    _component_instances = value;
  }
  /* (non-Javadoc)
   * @see org.objectweb.ccm.Containers.MetaInformation.HomeInstanceOperations#equivalent_interface()
   */
  public InterfaceInstance equivalent_interface()
  {
    return _equivalent_interface;
  }
  /* (non-Javadoc)
   * @see org.objectweb.ccm.Containers.MetaInformation.HomeInstanceOperations#equivalent_interface(org.objectweb.ccm.Containers.MetaInformation.InterfaceInstance)
   */
  public void equivalent_interface(InterfaceInstance value)
  {
    _equivalent_interface = value;
  }
  /* (non-Javadoc)
   * @see org.objectweb.ccm.Containers.MetaInformation.InterfaceInstanceOperations#operation_instance_list()
   */
  public OperationInstanceList operation_instance_list()
  {
    return _operation_instance_list;
  }
  /* (non-Javadoc)
   * @see org.objectweb.ccm.Containers.MetaInformation.InterfaceInstanceOperations#operation_instance_list(org.objectweb.ccm.Containers.MetaInformation.OperationInstanceList)
   */
  public void operation_instance_list(OperationInstanceList value)
  {
    _operation_instance_list = value;
  }
  /* (non-Javadoc)
   * @see org.objectweb.ccm.Containers.MetaInformation.HomeInstanceOperations#create_component_instance()
   */
  public ComponentInstance create_component_instance()
  {
    ComponentInstance result = new ComponentInstanceImpl();
    ComponentType _ctype;
    HomeType _htype;
    // getting the associated home type
    try
    {
      _htype = (HomeType) this.type_reference();
    }
    catch (ClassCastException e)
    {
      System.err.println("Cannot convert type -> home type !!!");
      return null;
    }
    // getting the associated component type
    try
    {
      _ctype = (ComponentType) _htype.type_reference();
    }
    catch (ClassCastException e)
    {
      System.err.println("Cannot convert type -> component type !!!");
      return null;
    }
    
    // filling the component instance properties
    //  setting the MI instance properties 
 
    result.related_instance_kind(instance_kind.HOME_INSTANCE);
    result.instance_reference(this);
    result.type_reference(_ctype);
 
    //// for attributes
 
    AttributeType[] _catts = _ctype.attribute_type_list().get_all_attribute_types();
    if (_catts.length != 0)
    {
      AttributeInstanceImpl _cints;
      
      for (int i = 0; i < _catts.length; i++)
      {
        _cints  = new AttributeInstanceImpl();
        _cints.type_reference(_catts[i]);
        _cints.related_instance_kind(instance_kind.COMPONENT_INSTANCE);
        _cints.instance_reference(this);
        _cints.object_ref(null);
        result.attribute_instance_list().add_attribute_instance(_cints);
      }
    }
    // for supported interfaces
   
    InterfaceType[] _sit =
      _ctype.supported_interfaces().get_all_interface_types();
    if (_sit.length != 0)
    {
      InterfaceInstance _sii;
      for (int i = 0; i < _sit.length; i++)
      {
        _sii = create_interface_instance(_sit[i]);
        _sii.related_instance_kind(instance_kind.COMPONENT_INSTANCE);
        _sii.instance_reference(this);
        _sii.object_ref(null);
        result.supported_interface().add_interface_instance(
          create_interface_instance(_sit[i]));
      }
    }
    // for equivalent interface
   
    InterfaceInstance _eii =
      create_interface_instance(_ctype.equivalent_interface());
    _eii.related_instance_kind(instance_kind.COMPONENT_INSTANCE);
    _eii.instance_reference(this);
    result.equivalent_interface(_eii);
    
    
    // component instance entity fullfilled, ready to be used
    
    this.component_instance_list().add_component_instance(result);
    
    return result;
  }
  
  
  private InterfaceInstance create_interface_instance(InterfaceType it)
  {
  //  System.err.println("### Debug : creating interface instance");
    
    InterfaceInstance result = new InterfaceInstanceImpl();
    OperationInstance tmp_opi;
    result.type_reference(it);
    OperationType[] tmp_ops = it.operation_type_list().get_all_operation_types();
    for (int i = 0; i < tmp_ops.length; i++)
    {
      tmp_opi = new OperationInstanceImpl();
      tmp_opi.related_instance_kind(instance_kind.INTERFACE_INSTANCE);
      tmp_opi.type_reference(tmp_ops[i]);
      tmp_opi.instance_reference(result);
      tmp_opi.object_ref(null);
      result.operation_instance_list().add_operation_instance(tmp_opi);
    }
    return result;
  }

}
