/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.ccm.Deployment;

// Package dependencies
import org.objectweb.util.cmdline.api.CommandLine;

import org.objectweb.util.cmdline.lib.ApplicationBase;
import org.objectweb.util.cmdline.lib.DefaultCommandLine;
import org.objectweb.ccm.descriptor.componentassembly.*;
import org.objectweb.ccm.descriptor.componentassembly.beans.*;
import org.objectweb.ccm.descriptor.componentassembly.ccm.*;

//import hfinder package for setting the deployTable

//import org.objectweb.ccm.Deployment.components.HFinder.*;
/**
 * AssemblyImpl is the top level implementation of the 
 * assembly behaviour. 
 *
 * @author <a href="mailto:frederic.briclet@lifl.fr">Briclet Frederic</a>
 *
 * @version 0.1
 */
public class AssemblyImpl extends org.omg.Components.Deployment.AssemblyPOA {


        
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================


    // xml handler path
    private String xmlFilePath;

    // Console for emittion of message
    private org.objectweb.util.cmdline.api.Console console;

    //Root  of xml tree cad representation
    private ComponentassemblyBean assemblyObject;

    //Table of component handle order by id key
    private java.util.Hashtable general;
    
    // State of the assembly    
    private org.omg.Components.Deployment.AssemblyOperations etat;
    
    //Reference on the NameService
    private org.omg.CosNaming.NamingContextExt ns ;
    
    //Name component involve in the NS registration
    private org.omg.CosNaming.NameComponent [] ncomp;

   
    /**
     * Internal class implementing the inactive state of the assembly
     */
    private class AssemblyImplInactive implements org.omg.Components.Deployment.AssemblyOperations
    {
        
        /**
         * Method to setup an internal table used to store information
         */
        public void setupDeployTable(){
            try{
                /*SAve usefull information to deployment*/
                general.put("BaseDir",org.objectweb.ccm.deploytool.DeploymentProcess.getBaseDir());
            
                /*Store reference on the name service*/
                general.put("__name_service__", 
                            org.objectweb.ccm.CORBA.TheNameService
                            .getNamingContext()
                            .getNamingContext());

                general.put("__component_list__",
                           new java.util.LinkedList());

                try{
                      org.objectweb.openccm.OpenCCM_DCI.DCIManager
                        DCI=
                        org.objectweb.openccm.
                        OpenCCM_DCI.DCIManagerHelper.
                        narrow(org.objectweb.ccm.CORBA
                               .TheORB.resolve_initial_reference("DCIManager"));	
                      
                      general.put("__home_finder__",DCI.provide_home_finder());
                      general.put("__home_registration__",DCI.provide_home_registration());
                      general.put("__dci_information__",DCI.provide_dci_information());

                }
                catch(Exception e){
                    System.err.println
                        ("CCM_deploy >> DCIManager not start,  HomeFinder functionnalities disable !");
                    System.err.println
                        ("CCM_deploy >> To start DCIManager execute dci_start <dci_name> command");
                }  
                /*Put a reference to the server socket to use*/
                java.net.ServerSocket s = new java.net.ServerSocket(0); 
                general.put("__server_socket__",s);
                
            }
            catch(Exception e){
                System.err.println("Problem occured during setting of the deployTable");
            }
        }

        /**
         * Build the assembly org.omg.Components.CreateFailue exception is
         * thrown if a problem occured during deployment process.         
         */
        public void 
        build() 
        throws org.omg.Components.CreateFailure 
        {
            try{          
                //console.message("Load the tree");
                loadCADTree();
                
                setupDeployTable();
                
                //general.put("__Console__",console);
                /*Start the deployment operation*/          
                
                ((ComponentassemblyDeployer)
                 assemblyObject
                 .getExtensionManager().getExtensionByName("ccm.Deployer")
                 .getInstance()).deployPartial(general,assemblyObject);
                
                ((ComponentassemblyDeployer)
                 assemblyObject
                 .getExtensionManager().getExtensionByName("ccm.Deployer")
                 .getInstance()).deployFinal(general,assemblyObject);
                

                /*Setup the component to configuration complete state*/
                for (java.util.ListIterator it =((java.util.LinkedList)
                                                 general.get("__component_list__")).listIterator(0);
                     it.hasNext() ; )
                    {
                        try {
                            ((org.omg.Components.CCMObject)
                             general.get((String)it.next())).configuration_complete();
                        }
                        catch (java.lang.ClassCastException exception) {
                            /*Nothing to do for non component objet like console*/
                        }
                        catch (org.omg.Components.InvalidConfiguration exception) 
                            {
                                exception.printStackTrace();
                                //console.error("Could not switch a CCM Object to configuration complete");
                                return ;
                            }
                    }
            }
            catch(Exception e)
                {
                    e.printStackTrace(); throw new org.omg.Components.CreateFailure();
                }
            
            etat=(org.omg.Components.Deployment.AssemblyOperations) new AssemblyImplActive();
            registerAssembly() ;
        }

        
        /**
         * Internal method allowing the inactive assembly to load
         * tree representation of the xml cad file
         */
        private void loadCADTree()throws org.omg.Components.CreateFailure
        {
            // get the ORB
            org.omg.CORBA.ORB orb
                = org.objectweb.ccm.CORBA.TheORB.getORB();
            //console.message("Got the ORB");
            
            // inits OpenCCM runtime
            org.objectweb.ccm.Components.Runtime.init (orb);
            //console.message("OpenCCM runtime is ready");
            
            /*Creating the java file like with .CAD file*/
            // java.io.File assemblyFile= new java.io.File (xmlFilePath);
            try {
                
                java.util.zip.ZipFile zipFile= 
                    new java.util.zip.ZipFile(xmlFilePath);
                
                general.put("__car_zip__",zipFile);
                
                java.io.InputStreamReader reader ;
                
                for(java.util.Enumeration enum=zipFile.entries();enum.hasMoreElements();)
                    {
                        
                        java.util.zip.ZipEntry ze=(java.util.zip.ZipEntry)enum.nextElement();
                        //System.out.println("Filename:"+ze.getName());
                        if(ze.getName().endsWith(".cad"))
                            {
                                reader=new java.io.InputStreamReader
                                    (zipFile.getInputStream(ze));
                                        
                                assemblyObject
                                    = org.objectweb.ccm.descriptor
                                    .componentassembly.beans.ComponentassemblyBeanImpl
                                    .unmarshalBean(reader,true);
                            }                        
                    }

            } 
            catch (java.io.IOException ex)
                {
                    ex.printStackTrace();
                    console.error("Could not unmarshal the assembly file");
                    throw new org.omg.Components.CreateFailure();
                }
            //console.message("Got the assembly descriptor");
        }
        
        /**
         * Break connection between composant and destroy them
         */
        public void tear_down() throws org.omg.Components.RemoveFailure
        {
            throw new  org.omg.Components.RemoveFailure();    
        }

        
        /**
         * Return the current assembly state    
         */
        public org.omg.Components.Deployment.AssemblyState 
        get_state()
        { 
            return org.omg.Components.Deployment.AssemblyState.INACTIVE;
        }        

    }
  

    /**
     * Internal class which represent an deploied assembly, only
     * tear_down() and get_state() operation are available
     */
    private class AssemblyImplActive implements org.omg.Components.Deployment.AssemblyOperations
    {
        /**
         * Alway thrown CreateFailure exception because the assembly
         * is already deploied. This function is non available
         */
        public void 
        build() 
        throws org.omg.Components.CreateFailure
        {
            throw new org.omg.Components.CreateFailure();
        }
        
        /**
         *    Tear_down the assembly in 2 part:
         *        First tear_down connections
         *        Secondly erease instantiation
         *  Switch the internal state of the assembly to inactive state.
         */
        public void 
        tear_down()
        throws org.omg.Components.RemoveFailure
        {
            try{
                //Unregister the assembly from NS
                unregisterAssembly();
                ((ComponentassemblyDeployer)
                 assemblyObject
                 .getExtensionManager().getExtensionByName("ccm.Deployer")
                 .getInstance()).tearDownConnections(general,(ComponentassemblyImpl)assemblyObject);
                ((ComponentassemblyDeployer)
                 assemblyObject
                 .getExtensionManager().getExtensionByName("ccm.Deployer")
                 .getInstance()).tearDownInstantiations(general,(ComponentassemblyImpl)assemblyObject);
                
                etat=new AssemblyImplInactive();
                
            }
            catch(Exception e){e.printStackTrace();}
            
        }
        /**
         * Break connection between composant and destroy them
         */
        public org.omg.Components.Deployment.AssemblyState 
        get_state()
        {
            return org.omg.Components.Deployment.AssemblyState.INSERVICE;
        }
    }
    
    // ==================================================================
    //
    // Constructor
    //
    // ==================================================================
   
    public 
    AssemblyImpl(String xmlFilePath)
    {
        this.xmlFilePath=xmlFilePath;
        general=new java.util.Hashtable();
        etat=new AssemblyImplInactive();
    }


    private void 
    registerAssembly()
    {
        try{           
            
            // retrieve the corba reference from the assembly object
           org.omg.CORBA.Object ref =
            this._this_object(org.objectweb.ccm.CORBA.TheORB.getORB());
        
            org.omg.Components.Deployment.Assembly assemblyRef =
                org.omg.Components.Deployment.AssemblyHelper.narrow(ref);

            // Obtain the Name Service.
            ns=org.omg.CosNaming.NamingContextExtHelper
                .narrow(org.objectweb.ccm.CORBA
                        .TheNameService
                        .getNamingContext()
                        .getNamingContext());
            /*
              // Bind the server object in the name service.
              ncomp= org.objectweb.ccm.descriptor
              .componentassembly
              .ccm.RegisterwithnamingDeployer
              .rebind(ns,
              System.getProperty("Assemblies_ns_dir")+
              assemblyObject.getId(),
              assemblyRef);
            */
            }
        catch(Exception e){
             System.err.println("Following exception occur during assembly registration");
             System.err.println(e.getMessage());
             System.err.println("Cannot register the assembly"+
                                System.getProperty("Assemblies_ns_dir")+
                                assemblyObject.getId());
             return ;
        }
        System.out.println("The Assembly "+System.getProperty("Assemblies_ns_dir")+
                                assemblyObject.getId()+" is referenced");
    }
    /**
     * Unregister the assembly from the NameService
     */
    private void
    unregisterAssembly()
    {
        try{
            ns.unbind(ncomp);
        }
        catch(Exception e){
             System.err.println("Following exception occur during assembly unregistration");
             System.err.println(e.getMessage());
             System.err.println("Cannot unregister the assembly"+
                                System.getProperty("Assemblies_ns_dir")+
                                assemblyObject.getId());
        }
    }
    /*Delegation operation to the state assembly*/
    public void 
    build() 
    throws org.omg.Components.CreateFailure
    {
        etat.build();
    }


    public void 
    tear_down() 
    throws org.omg.Components.RemoveFailure 
    {
        etat.tear_down();
    }
    


    public org.omg.Components.Deployment.AssemblyState
    get_state()
    {
        return etat.get_state();
    }

    /*public String getId(){return assemblyObject.getId();}*/


    /*public void setConsole(org.objectweb.util.cmdline.api.Console console){
      this.console=console;
      }*/
}
