/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.ccm.Deployment;


public class DCIManagerServer  extends org.objectweb.ccm.CORBA.ApplicationServer
{
	
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    /*The name of the DCI*/
    private String dciName;
    /*The path to used to save the DCI IOR */
    private String dciIORFilePath;
    /*the registratio name*/
    private String registrationName=null ;
    /*Reference of the started DCI*/
    private org.objectweb.openccm.OpenCCM_DCI.DCIManager DCI ;
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **/
    public
    DCIManagerServer()
    {
        // Set EntityResolver for CCM DTDs
        org.objectweb.ccm.descriptor.EntityResolver.setCCMResolver();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    private void
    computeArgs(String args[])
    {
        for(int i=0;i<args.length;i++){
            if(args[i].startsWith("DCINAME="))
               this.dciName=args[i].substring(8);
            else  if(args[i].startsWith("IORPATH="))
                this.dciIORFilePath=args[i].substring(8);              
               }    
    }    
    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  The bootstrap main method.
     **
     **  @param args The command-line arguments.
     **/
    public static void
    main(String args[])
    {
       DCIManagerServer server = new DCIManagerServer();       
        int status = server.start(args);
        System.exit(status);
    }

    // ==================================================================
    //
    // Methods for the org.objectweb.ccm.CORBA.ApplicationServer.
    //
    // ==================================================================

    /**
     **  Before running the CORBA::ORB.
     **
     **  @param args The command line arguments.
     **
     **  @return The status.
     **/
    public int
	before_run(String[] args)
    {
        //used to retrieve in the right order all the needed parameter
        computeArgs(args);
        
        try{
            //Initialize the ORB
            org.objectweb.ccm.Components.Runtime.init(org.objectweb.ccm.CORBA.TheORB.getORB());
            
        
        //Retrieve a DCIManagerHome reference 
            org.objectweb.openccm.OpenCCM_DCI.DCIManagerHome DCIHome=
                org.objectweb.openccm.OpenCCM_DCI.DCIManagerHomeHelper.
                narrow((new DeploymentComponentFactory()).
                       install_home("org.objectweb.openccm.OpenCCM_DCI.cif.DCIManagerHomeImpl.create_home"));
            
            //Create a DCIManager
            DCI= DCIHome.create();
            //Set the DCIName
            DCI.dci_name(dciName);
            
            //Retrieve a AssemblyFactoryHome reference         
            org.objectweb.openccm.OpenCCM_DCI.AssemblyManagerModelHome AMMH=
                org.objectweb.openccm.OpenCCM_DCI.
                AssemblyManagerModelHomeHelper.
                narrow((new DeploymentComponentFactory()).
                       install_home("org.objectweb.openccm.OpenCCM_DCI.cif.AssemblyManagerModelHomeImpl.create_home"));
            //Connect the assembly manager model used to install assembly
            DCI.connect_assembly_manager_model_home(AMMH);             

            org.omg.CosNaming.NamingContextExt nce=
                org.omg.CosNaming
                .NamingContextExtHelper
                .narrow(org.objectweb.ccm.CORBA.TheNameService
                        .getNamingContext().getNamingContext());
            DCI.connect_naming_service(nce); 
                                    
            DCI.configuration_complete();
            

            
          //  if(registrationName!=null){               
          
            registrationName=
                System.getProperty("DCI_ns_dir")
                +dciName;  
                
                org.objectweb.ccm
                    .descriptor.componentassembly.ccm
                    .RegisterwithnamingDeployer
                    .rebind(nce,registrationName,DCI);
            //}
            //register homefinder service 
            org.objectweb.ccm
                .descriptor.componentassembly.ccm
                .RegisterwithnamingDeployer
                .rebind(nce,"OpenCCM/Services/HomeFinder",DCI.provide_home_finder());
            org.objectweb.ccm
                .descriptor.componentassembly.ccm
                .RegisterwithnamingDeployer
                .rebind(nce,"OpenCCM/Services/HomeRegistration",DCI.provide_home_registration());
            
            // Output the IOR of the DCIManager.
            org.objectweb.ccm.CORBA.TheORB.save_IOR(DCI,dciIORFilePath);    
            
            return 0;
        }
        catch(Exception e){ 
            e.printStackTrace();
            System.out.println(e.getMessage());            
        }
        return 0;
        
    }
    
    /**
     **  After running the CORBA::ORB.
     **/
    public void
	after_run()
    {   
        try{
            if(registrationName!=null){
                org.omg.CosNaming.NamingContextExt nce=
                    org.omg.CosNaming
                    .NamingContextExtHelper
                    .narrow(org.objectweb.ccm.CORBA.TheNameService
                            .getNamingContext().getNamingContext());
                
                nce.unbind(nce.to_name(registrationName));

                System.err.println("The "+registrationName+"DCI is unreferenced");
            }
        }
        catch(Exception e)
            {
                System.err.println("Cannot unbind The "+registrationName+"DCI");
            }
            

    }	
    
}
