/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.ccm.Deployment;


public class NodeManagerServer  extends org.objectweb.ccm.CORBA.ApplicationServer
{
	
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    /*The file which will store the node ior*/
    private String nodeIORFilePath;
    /*The node name*/
    private String nodeName;
    /*The NS registration name */
    private String registrationName;
    /*the configuration file path*/
    private String npdFilePath;
    /*The reference on the current DCI*/
    private org.objectweb.openccm.OpenCCM_DCI.DCIManager DCI ;
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
   private void
    computeArgs(String args[])
    {
        for(int i=0;i<args.length;i++){
            if(args[i].startsWith("NODENAME="))
               this.nodeName=args[i].substring(9);
            else  if(args[i].startsWith("IORPATH="))
                this.nodeIORFilePath=args[i].substring(8);      
            else  if(args[i].startsWith("NPDFILE=-I"))                
                this.npdFilePath=args[i].substring(10);     
               }           
    }    

    
      // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **/
    public
    NodeManagerServer()
    {
        // Set EntityResolver for CCM DTDs
        org.objectweb.ccm.descriptor.EntityResolver.setCCMResolver();
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     **  The bootstrap main method.
     **
     **  @param args The command-line arguments.
     **/
    public static void
    main(String args[])
    {
       NodeManagerServer server = new NodeManagerServer();

        int status = server.start(args);
        System.exit(status);
    }

    // ==================================================================
    //
    // Methods for the org.objectweb.ccm.CORBA.ApplicationServer.
    //
    // ==================================================================

    /**
     **  Before running the CORBA::ORB.
     **
     **  @param args The command line arguments.
     **
     **  @return The status.
     **/
    public int
	before_run(String[] args)
    {

	try{
        //Initialize the ORB
        org.objectweb.ccm.Components.Runtime.init(org.objectweb.ccm.CORBA.TheORB.getORB());
        
        computeArgs(args);
             
        //Retrive the DCI reference
        DCI=
            org.objectweb.openccm.
            OpenCCM_DCI.DCIManagerHelper.
            narrow(org.objectweb.ccm.CORBA
                   .TheORB.resolve_initial_reference("DCIManager"));			       
        
        //Retrieve a AssemblyFactoryManagerHome reference 
        org.objectweb.openccm.OpenCCM_DCI.NodeManagerHome nodeHome=
            org.objectweb.openccm.OpenCCM_DCI.NodeManagerHomeHelper.
            narrow((new DeploymentComponentFactory()).
            install_home("org.objectweb.openccm.OpenCCM_DCI.cif.NodeManagerHomeImpl.create_home"));

        //create the node
        org.objectweb.openccm.OpenCCM_DCI.NodeManager 
            node=nodeHome.create_node(npdFilePath);
        
        //Can be use full to avoid to fill a complete 
        // npd file
        if(nodeName!=null)
            node.node_name(nodeName);

        //Retrieve a AssemblyFactoryHome reference         
        org.objectweb.openccm.OpenCCM_DCI.ComponentServerManagerHome CSMH=
            org.objectweb.openccm.OpenCCM_DCI.
            ComponentServerManagerHomeHelper.
            narrow((new DeploymentComponentFactory()).
         install_home("org.objectweb.openccm.OpenCCM_DCI.cif.ComponentServerManagerHomeImpl.create_home"));
        //connect the component server manager home   
        node.connect_component_server_manager_home(CSMH);
        
        //Retrieve a AssemblyFactoryHome reference         
        org.objectweb.openccm.OpenCCM_DCI.ContainerManagerHome CMH=
            org.objectweb.openccm.OpenCCM_DCI.
            ContainerManagerHomeHelper.
            narrow((new DeploymentComponentFactory()).
        install_home("org.objectweb.openccm.OpenCCM_DCI.cif.ContainerManagerHomeImpl.create_home"));
       
        //connect the node management interface
        node.connect_node_management(DCI.provide_node_management());
        
        //Connect the ContainerManagerHome to the  node component
        node.connect_container_manager_home(CMH);
        //configure complete the node
        node.configuration_complete();

        // Register the node if required
      //  if(registrationName!=null){
            org.omg.CosNaming.NamingContextExt nce=
                org.omg.CosNaming
                .NamingContextExtHelper
                .narrow(org.objectweb.ccm.CORBA.TheNameService
                        .getNamingContext().getNamingContext());
           registrationName=
                System.getProperty("Node_ns_dir")
                +nodeName;                                    
                        
            org.objectweb.ccm
                .descriptor.componentassembly.ccm
                .RegisterwithnamingDeployer
                .rebind(nce,registrationName,node);
           // }
            
            // Output the IOR of the DCIManager.
            org.objectweb.ccm.CORBA.TheORB.save_IOR(node,nodeIORFilePath);    
            
        return 0;
	}
	catch(Exception e){ 
        e.printStackTrace();
        System.out.println(e.getMessage());
        
    }
    return 0;

    }
    
    /**
     **  After running the CORBA::ORB.
     **/
    public void
	after_run()
    {        
        try{
            if(registrationName!=null){
                org.omg.CosNaming.NamingContextExt nce=
                    org.omg.CosNaming
                    .NamingContextExtHelper
                    .narrow(org.objectweb.ccm.CORBA.TheNameService
                            .getNamingContext().getNamingContext());
                
                nce.unbind(nce.to_name(registrationName));
                
                System.err.println("The "+registrationName+"Node is unreferenced");
            }
        }
        catch(Exception e)
            {
                System.err.println("Cannot unbind The "+registrationName+"Node");
            }           

    }	
    
}
