/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/


package org.objectweb.ccm.descriptor.componentassembly.ccm;

// Import all the packages required
import org.objectweb.ccm.descriptor.componentassembly.beans.*;
import org.objectweb.ccm.descriptor.componentassembly.*;
import org.objectweb.ccm.descriptor.softpkg.*;
import org.objectweb.ccm.descriptor.softpkg.beans.*;

/**
 *
 * ComponentFileDeployer is used to read xml file information.
 *
 * @author <a href="mailto:Frederic.Briclet@lifl.fr">Briclet Frederic</a>
 *
 * @version 0.1
 */
public class ComponentfileDeployer {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Componentassembly object use to imbricate assembly*/
    private ComponentassemblyBean assemblyObject ;
    /*Prefix for the outputed message*/
    private String prefix="ComponentFileDeployer >> ";

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================
  

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================


    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * DeployPartial method goal is to gather component information and invoke
     * deployPartial method on possible imbricated component.
     * If the files point by the tags fileinarchive is a .CSD file the component
     * informations are load and save in the deployTable, else if the file is a .CAD
     * file the componentAssembly is load and store into the ComponentFileDeployer, then
     * a call to deployPartial on the componentAssembly is done. This call will initialize
     * the instance of imbricated CAD.
     *
     * @param deployTable the internal table use to store information.
     * @param cfi the componentfile to read
     */
    public void 
    deployPartial(java.util.Hashtable deployTable, 
                  Componentfile cfi)
    {
        try{
            
            org.objectweb.ccm
                .descriptor.componentassembly
                .beans.FileinarchiveBean current=
                (org.objectweb.ccm.
                 descriptor.componentassembly
                 .beans.FileinarchiveBean) 
                cfi.getFileinarchive();
            
            java.io.InputStreamReader reader= 
                ((FileinarchiveDeployer)
                 current
                 .getExtensionManager()
                 .getExtensionByName("ccm.Deployer")
                 .getInstance())
                .getInputStreamReader(deployTable,cfi.getFileinarchive());

            //Test if the file is a csd or a cad file
            if(cfi.getFileinarchive().getName().endsWith(".csd")){

                SoftpkgBean soft=SoftpkgBeanImpl.unmarshalBean(reader,true);

                //Browse all the implementation to store the reference of Java implementation
                for(java.util.Iterator i=soft.getImplementationList().iterator();i.hasNext();){
                    ImplementationImpl ii=(ImplementationImpl)i.next();
                    if(((Programminglanguage)ii.getProgramminglanguageList()
                        .get(0)).getName().equalsIgnoreCase("java"))
                        {
                            deployTable.put(ii.getId(),ii);
                        }
                }
                deployTable.put(cfi.getId(),soft);                
            }
            
            else if(cfi.getFileinarchive().getName().endsWith(".cad")){
                /* The file is a cad, the loading of the componentassembly*/
                assemblyObject
                    = ComponentassemblyBeanImpl.unmarshalBean(reader,true);
                
                /* Do the first part componentassembly deployment*/
                ((ComponentassemblyDeployer)
                 assemblyObject
                 .getExtensionManager().getExtensionByName("ccm.Deployer")
                 .getInstance()).deployPartial(deployTable,assemblyObject);
                /*store the componentassembly*/
                deployTable.put(cfi.getId(),assemblyObject);
            }
        }
        catch(Exception e)
            {
                System.err.println(prefix+"Cannot open CSD file");
                e.printStackTrace() ;
            }
    }

    /**
     * If a imbricated cad is point, the connections between component is done throw
     * the call of deployFinal on imbricated componentassembly.
     * If the pointed file is a csd nothing to do.
     *
     * @param deployTable the internal table use to store information.
     * @param cfi the componentfile to deploy
     */
    public void 
    deployFinal(java.util.Hashtable deployTable, 
                Componentfile cfi)
    {
        if(cfi.getFileinarchive().getName().endsWith(".cad")){
            ((ComponentassemblyDeployer)
             ((ComponentassemblyBean)deployTable.get(cfi.getId()))
             .getExtensionManager().getExtensionByName("ccm.Deployer")
             .getInstance()).deployFinal(deployTable,assemblyObject);
        }
    }

    /**
     * Just remove from hashtable the tree representation of .CSD
     *
     * @param deployTable the internal table use to store information.
     * @param cfi the componentfile to tear_down
     */
    public void 
    tearDownConnections(java.util.Hashtable deployTable,
                        Componentfile cfi)
    {
        
        if(cfi!=null&&
           cfi.getFileinarchive()!=null&&
           cfi.getFileinarchive().getName().endsWith(".cad"))
            {
                ((ComponentassemblyDeployer)
                 ((ComponentassemblyBean)deployTable.get(cfi.getId()))
                 .getExtensionManager().getExtensionByName("ccm.Deployer")
                 .getInstance()).tearDownConnections(deployTable,(ComponentassemblyBeanImpl)assemblyObject);
            }
    }

    /**
     * Just remove from hashtable the tree representation of .CSD
     *
     * @param deployTable the internal table use to store information.
     * @param cfi the componentfile to tear_down
     */
    public void 
    tearDownInstantiations(java.util.Hashtable deployTable,
                           Componentfile cfi)
    {
        if(cfi.getFileinarchive().getName().endsWith(".cad")){
            ((ComponentassemblyDeployer)
             ((ComponentassemblyBean)deployTable.get(cfi.getId()))
             .getExtensionManager().getExtensionByName("ccm.Deployer")
             .getInstance()).tearDownInstantiations(deployTable,(ComponentassemblyBeanImpl)assemblyObject);
        }
        deployTable.remove(cfi.getId());
    }
}
