/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm;


// import all required packages
import org.objectweb.ccm.descriptor.componentassembly.beans.*;
import org.objectweb.ccm.descriptor.componentassembly.*;

/**
 * ConnectEventDeployer make the event connection
 *
 * @author <a href="mailto:frederic.briclet@lifl.fr">Briclet Frederic</a>
 *
 * @version 0.1
 */


public class ConnecteventDeployer {
     // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /* Interanal exception used to manage the connection*/

    private class CannotResolveReferenceException extends Exception{};
    
    /** Cookie whicth store the connection information*/
    private org.omg.Components.Cookie cookie; 
    
    /** Reference to subscriber */ 
    private org.omg.Components.CCMObject subscriber;


    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================


    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================


    /**
     * Resolve and existing interface and return it like the consumed 
     * object. If any interface can be found an exception is thrown
     *
     * @param deployTable the internal table use to store information.
     * @param ei the existing interface to connect
     * @return The existing interface
     */
    private  org.omg.Components.EventConsumerBase 
    proceedExistingInterface(java.util.Hashtable deployTable,
                             Existinginterface ei)
    throws Exception
    {
        return org.omg.Components.EventConsumerBaseHelper
            .narrow(resolveReference(deployTable,ei.getFindby()));        
    }
    
    /**
     * Internal method to resolve and retrieve the publisher. First it 
     * try to find the publisher reference with the componentinstantiationref,
     * if it is null it try to resolve the reference with registration 
     * services. If the both way failed an exception is thrown.
     *
     * @param deployTable the internal table use to store information.
     * @param pp the publishesport to retrieve.
     * @return Return the publishes port
     */
    private org.omg.Components.CCMObject 
    proceedPublishesPort(java.util.Hashtable deployTable,
                         Publishesport pp)
    throws Exception
    {
        // retrieve the publisher throws the instantiationref if it's null
        if(pp.getComponentinstantiationref()!=null)
            return (org.omg.Components.CCMObject)
                deployTable.get(pp.getComponentinstantiationref().getIdref());
        // try to retrieve the reference throw registration services    
        return org.omg.Components.
               CCMObjectHelper.narrow(resolveReference(deployTable,pp.getFindby()));        
    }    
    
    /**
     * Internal method to resolve and retrieve the emitter. First it try to find 
     * the emitter reference with the componentinstantiationref, if it is null
     * it try to resolve the reference with registration services.
     * If the both way failed an exception is thrown.
     *
     * @param deployTable the internal table use to store information.
     * @param ep the emitsport to retrieve.
     * @return Return the emits port
     */
    private org.omg.Components.CCMObject 
    proceedEmitsPort(java.util.Hashtable deployTable, Emitsport ep)
    throws Exception
    {
        if(ep.getComponentinstantiationref()!=null)
            return (org.omg.Components.CCMObject)
                deployTable.get(ep.getComponentinstantiationref().getIdref());
        
        return org.omg.Components.
              CCMObjectHelper.narrow(resolveReference(deployTable,ep.getFindby()));        
    }    
    
    /**
     * Internal method to resolve and retrieve the comsumedObject.
     * First it try to find the consumer reference with the componentinstantiationref,
     * if it is null, it try to resolve the reference with registration services.
     * If the both way failed an exception is thrown.
     * If the consumer is found the method return the consumed object invoking 
     * on the consumer the get_consumer method with consumeidentifier to parameter. 
     *
     * @param deployTable the internal table use to store information.
     * @param cp the consumesport to retrieve.
     * @return Return the consumes port
     */
    private  org.omg.Components.EventConsumerBase 
    proceedConsumesPort(java.util.Hashtable deployTable , Consumesport cp)
    throws Exception
    {
        
        org.omg.Components.CCMObject consumer ;
    
        if(cp.getComponentinstantiationref()!=null)
            consumer =(org.omg.Components.CCMObject)
                deployTable.get(cp.getComponentinstantiationref().getIdref());                
        
        else consumer=org.omg.Components.
                      CCMObjectHelper.narrow(resolveReference(deployTable,cp.getFindby()));
    
        return consumer.get_consumer(cp.getConsumesidentifier().getValue());
    }

   /**
     * Internal method allowing to resolve the Findby tag using
     * the extensio manager tool. 
     *
     * @param deployTable  the internal table used to store information.
     * @param fb The findby information to process 
     */       
    private org.omg.CORBA.Object 
    resolveReference(java.util.Hashtable deployTable,Findby fb)
    throws Exception
    {
        return  ((FindbyDeployer)
                 ((FindbyBean)fb)
                 .getExtensionManager()
                 .getExtensionByName("ccm.Deployer")
                 .getInstance())
                 .resolveReference(deployTable,fb);
    }
    

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
   
    
    /**
     * Proceed the connectevent connection :
     *            - Retrieve the consumedObject
     *            - Retrieve the subscriber
     *            - Connect the consumer or subscribe the consumer
     * ConnectEventDeployer is a leaf of execution deployer three no more
     * recursives call to deploy method are done.
     * Both emitsport and publishesport are supported but following the
     * specification just the first will be process if both are defined.
     *
     * @param deployTable the internal table use to store information.
     * @param cei the connectevent to deploy     
     */
    public void 
    deploy(java.util.Hashtable deployTable,
           Connectevent cei)
    {

        try{
            
            // Get the consumes object
            org.omg.Components.EventConsumerBase consumedObject=null ;
            if(cei.getConsumesport()!=null)
                {
                    consumedObject=
                        proceedConsumesPort(deployTable , cei.getConsumesport());
                }
            else if(cei.getExistinginterface()!=null)
                {
                    consumedObject=
                        proceedExistingInterface(deployTable,cei.getExistinginterface());
                }                                    
            
            if(cei.getPublishesport()!=null){
                //retrieve the subscriber
                subscriber=proceedPublishesPort(deployTable,cei.getPublishesport());
                //Subscribe to the object
                cookie=subscriber.subscribe(cei.getPublishesport()
                                            .getPublishesidentifier()
                                            .getValue()
                                            ,consumedObject);
            }
            else if(cei.getEmitsport()!=null){
                //retrieve the subscriber
                subscriber=proceedEmitsPort(deployTable,cei.getEmitsport());
                //Subscribe to the object
                subscriber.connect_consumer(cei.getEmitsport()
                                            .getEmitsidentifier()
                                            .getValue()
                                            ,consumedObject);
            }            
        }
        catch(Exception e){ 
            System.err.println("Cannot establish the <"+cei.getId()+" >connectevent connection");
            //e.printStackTrace();

        }
    }




    /**
     *Tear down connection : unsubscrive the publisher     
     *
     * @param deployTable the internal table use to store information.
     * @param cei the connectevent to tear_down
     * @return Return the corba object found
     */
    public void 
    tearDown(java.util.Hashtable deployTable,
             Connectevent cei)
    {
        try{
            //If the connection wasn't established
            if(subscriber==null)return ;
            // call the unsubscribe method
            if(cei.getPublishesport()!=null){
               
                subscriber.unsubscribe(cei.getPublishesport()
                                       .getPublishesidentifier()
                                       .getValue()
                                       ,cookie);
            }else if(cei.getEmitsport()!=null){
                
                subscriber.disconnect_consumer(cei.getEmitsport()
                                               .getEmitsidentifier().getValue());
            }        
        }
        catch(Exception e){e.printStackTrace(); }
    }

}
