/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm;

// importation of the needed packages
import org.objectweb.ccm.descriptor.componentassembly.beans.*;
import org.objectweb.ccm.descriptor.componentassembly.*;
/**
 * A connectinterfaceDeployer to establish connectinterface connction
 *
 * @author <a href="mailto:frederic.briclet@lifl.fr">Briclet Frederic</a>
 *
 * @version 0.1
 */
public class ConnectionsDeployer{
    
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================


    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Deploy and establish all the connections describe in te CAD, it includes
     * the homeinterface and the connect event( connecthomes are not yiet supported).
     * The principle of this function is to browse all the defined connection
     * and call recursively the deploy function on each element. To make the invokation
     * the extension manger tool is use
     * TODO : connecthome
     *
     * @param deployTable the internal table use to store information.
     * @param ci the connections to deploy   
     */
    public void 
    deploy(java.util.Hashtable deployTable, Connections ci)
    {
        // browse the connectinterface list
        for(java.util.Iterator i= ci.getConnectinterfaceList().iterator();i.hasNext();) {
            
            ConnectinterfaceBean current=(ConnectinterfaceBean)i.next();
            ((ConnectinterfaceDeployer)
             current
             .getExtensionManager().getExtensionByName("ccm.Deployer")
             .getInstance()).deploy(deployTable,current);
        }
         
        //browse the connectevent list
        for(java.util.Iterator i= ci.getConnecteventList().iterator();i.hasNext();) {
            
            ConnecteventBean current=(ConnecteventBean)i.next();
            ((ConnecteventDeployer)
             current
             .getExtensionManager().getExtensionByName("ccm.Deployer")
             .getInstance()).deploy(deployTable,current);
        }
    }
    
    /**
     * Tear down all the connection.
     *
     * @param deployTable the internal table use to store information.
     * @param ci the connections to tear_down      
     */
    public void 
    tearDown(java.util.Hashtable deployTable, Connections ci)
    {
        for(java.util.Iterator i= ci.getConnectinterfaceList().iterator();i.hasNext();) 
            {
                ConnectinterfaceBean current=(ConnectinterfaceBean)i.next();
                ((ConnectinterfaceDeployer)
                 current
                 .getExtensionManager().getExtensionByName("ccm.Deployer")
                 .getInstance()).tearDown(deployTable,current);
            }
        for(java.util.Iterator i= ci.getConnecteventList().iterator();i.hasNext();) 
            {
                ConnecteventBean current=(ConnecteventBean)i.next();
                ((ConnecteventDeployer)
                 current
                 .getExtensionManager().getExtensionByName("ccm.Deployer")
                 .getInstance()).tearDown(deployTable,current);
            }
    }
}
