/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): Christophe Contreras.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm;

// Importation of all the required package
import org.objectweb.ccm.descriptor.componentassembly.beans.*;
import org.objectweb.ccm.descriptor.componentassembly.*;

/**
 * FindbyDeployer allow to resolve a find by tags returning the CORBA object found
 *
 * @author <a href="mailto:frederic.briclet@lifl.fr">Briclet Frederic</a>
 *
 * @version 0.1
 */
public class FindbyDeployer{

    public class CannotResolveReferenceException extends Exception{};  
    /**Cookie whitch store the connection information*/


    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal method.
    //
    // ==================================================================


    /**
     * Internal method in charge to found the reference throw the homefinder.
     * It search first the HomeFinder reference into the nameService and secondary
     * use this homefinder reference to try to resolve the home name precised.
     * If the home interface is found it is return, else an exception is raised
     *
     * @param deployTable  the internal table use to store information.
     * @param hf The homefinder tags to resolve
     * @return Return the home found
     */
    private  org.omg.Components.CCMHome
    proceedWithHomeFinder(java.util.Hashtable deployTable,
                          Homefinder hf)
    throws Exception
    {
        if(!deployTable.containsKey("__home_finder__"))
            throw new CannotResolveReferenceException();
       /*Invoke a search by name into the homefinder*/
        return ((org.omg.Components.HomeFinder)
                deployTable.get("__home_finder__"))
               .find_home_by_name(hf.getName());
        //return null;
    }
      
    
     /**
      * Internal fonction to resolve and retrieve a reference with the name service. 
      * The reference look up is defined in the Nameservice object tag. If any object 
      * can be found with this reference an exception is thrown.
      *
      * @param deployTable  the internal table use to store information.
      * @param service the namingservice tags to resolve
      * @return Return the corba object found
      */   
    private org.omg.CORBA.Object
    proceedWithNameService(java.util.Hashtable deployTable,
                           Namingservice service)
    throws Exception
    {
        if(!deployTable.containsKey("__name_service__"))
            throw new CannotResolveReferenceException();
        org.omg.CosNaming.NamingContextExt context=
            (org.omg.CosNaming.NamingContextExt)deployTable.get("__name_service__");
        return context.resolve(context.to_name(service.getName()));
       
        //return org.omg.Components.CCMObjectHelper.narrow(obj);
    }  


    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    /**
     * Internal method to resolve a reference point out by a findby component.
     * First it with try to find the object with a stringifield reference.
     * Secondly it try to resolve with name service.
     * Note: If two ways to find are defined only the stringifiedobjectref will
     * be proceed, following the specification only on ways would be defined.
     * TODO: processing with Trader
     *
     * @param deployTable  the internal table use to store information.
     * @param fb the Findby element to resolve
     * @return Return the corba object found
     */   
    public org.omg.CORBA.Object 
    resolveReference(java.util.Hashtable deployTable, 
                     Findby fb)
    throws Exception
    {
        //TODO : reslving component with trader
        if(fb!=null ) {
            if(fb.getNamingservice()!=null)
                return proceedWithNameService(deployTable,fb.getNamingservice());
            else if(fb.getStringifiedobjectref()!=null)
                return org.objectweb.ccm.CORBA.TheORBSingleton.getORB().
                    string_to_object(fb.getStringifiedobjectref().getValue());
            else if(fb.getHomefinder()!=null)
                return proceedWithHomeFinder(deployTable,fb.getHomefinder());
        }
        throw new CannotResolveReferenceException();
    }   


}
