/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm;

//import the needed packages
import org.objectweb.ccm.descriptor.componentassembly.beans.*;
import org.objectweb.ccm.descriptor.componentassembly.*;
import org.objectweb.ccm.descriptor.softpkg.*;
import org.objectweb.ccm.descriptor.softpkg.beans.*;

/**
 * HomeplacementDeployer install a home and configure it. Il also browse
 * all the componentinstantiation to build.
 *
 * @author <a href="mailto:frederic.briclet@lifl.fr">Briclet Frederic</a>
 *
 * @version 0.1
 */
public class HomeplacementDeployer {
    
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    
    /**Internal exception for deployment managing*/        
    private class CannotSetupHomeException extends Exception{};
    /**The component server target */ 
    private org.omg.Components.Deployment.ComponentServer server_cs ;
    /**The component installation target*/
    private org.omg.Components.Deployment.ComponentInstallation server_inst;
    /**The container target */
    private org.omg.Components.Deployment.Container server_cont;
    /**The home produced*/
    private org.omg.Components.CCMHome homePlaced;
    /*Descriptor containing the path to ccd file*/
    org.objectweb.ccm.descriptor.softpkg.Descriptor descriptor;


    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    // ==================================================================
    //
    // private methods.
    //
    // ==================================================================
    
    /**
     * Setup the dependencies defined in softpkg tags
     *
     * @param deployTable the internal table use to store information.
     * @param hi the homeplacement to process
     */
    private void
    setupSoftpkgDependency(java.util.Hashtable deployTable,
                           Homeplacement hi)
    throws java.io.IOException,
           org.omg.Components.Deployment.InvalidLocation,
           org.omg.Components.Deployment.InstallationFailure
    {
        
        java.io.File file;
        String absolutePath;
        
        SoftpkgBean softpkg=(SoftpkgBean)deployTable.get(hi.getComponentfileref().
                                                         getIdref());
        
        /*retrieve path to  CCD file*/
        if(softpkg.getDescriptorList()!=null)
            {
                descriptor=
                    (org.objectweb.ccm.descriptor.softpkg.Descriptor)
                    softpkg.getDescriptorList().get(0);
            }
                
        if(softpkg.getDependencyList()==null)
            return;
        // parse the softpkg dependency list and install its
        for (java.util.Iterator dep_itrator = softpkg.getDependencyList().iterator() ;
             dep_itrator.hasNext() ; )
            {
                org.objectweb.ccm.descriptor.softpkg.beans.DependencyBean dep
                    = (org.objectweb.ccm.descriptor.softpkg.beans.DependencyBean) 
                      dep_itrator.next();
                
                ((org.objectweb.ccm.descriptor.softpkg.ccm.DependencyDeployer)
                 dep
                 .getExtensionManager()
                 .getExtensionByName("ccm.Deployer")
                 .getInstance())
                 .resolveDependency(deployTable,server_inst,dep);
            }
        
    }

    
    
    /**
     * Interanal method in charge to init retrieve the componentServe needed
     * for homeplacement deployment.
     *
     * @param deployTable the internal table use to store information.
     * @param dest the component installation location
     */
    private void 
    setComponentInstantiation(java.util.Hashtable deployTable,
                              Destination dest)
    throws Exception
    {
        // Resolving nameservice problem  Obtain the component servers.    
        server_cs=((DestinationDeployer)((DestinationBean)dest)
           .getExtensionManager().getExtensionByName("ccm.Deployer")
           .getInstance()).getComponentServer(deployTable,dest);
        server_inst=((DestinationDeployer)((DestinationBean)dest)
             .getExtensionManager().getExtensionByName("ccm.Deployer")
             .getInstance()).getComponentInstallation(deployTable,dest);             
        
    }

    /**
     * Internal method in charge to setup the container  and to create the home.
     * 
     * @param deployTable the internal table use to store information.
     * @param hi Homeplacement to setup.  
     */
    private org.omg.Components.CCMHome 
    setupHome(java.util.Hashtable deployTable,
              Homeplacement hi) 
    throws Exception 
    {
        
        ImplementationBean implemToInstall=
            (ImplementationBean)deployTable.get(hi.getComponentimplref().getIdref());


        if(descriptor==null){
            descriptor=
            (org.objectweb.ccm.descriptor.softpkg.Descriptor)
                implemToInstall.getDescriptorList().get(0);
           
        }

        // Install archives necessary to this homeplacement only the first archive will be install
        for( java.util.Iterator i=implemToInstall.getCodeList().iterator();i.hasNext();){
            CodeBean cb=(CodeBean)i.next();            
             
            org.objectweb.ccm
                .descriptor.softpkg
                .beans.FileinarchiveBean current=
                (org.objectweb.ccm.
                 descriptor.softpkg
                 .beans.FileinarchiveBean) 
                cb.getFileinarchive();
            
                
            String fileAdress =
                ((org.objectweb.ccm.descriptor.softpkg.ccm.FileinarchiveDeployer)
                 current
                 .getExtensionManager()
                 .getExtensionByName("ccm.Deployer")
                 .getInstance())
                .startHttpServer(deployTable,cb.getFileinarchive());                
            
            server_inst.install
                (implemToInstall.getId(),fileAdress);

        }
        // parse the implementation dependency list and install its
        if(implemToInstall.getDependencyList()!=null)
            {
                
                
                for (java.util.Iterator dep_itrator = implemToInstall.getDependencyList().iterator() ;
                     dep_itrator.hasNext() ; )
                    {

                        org.objectweb.ccm.descriptor.softpkg.beans.DependencyBean dep
                            = (org.objectweb.ccm.descriptor.softpkg.beans.DependencyBean) 
                            dep_itrator.next();
                
                        ((org.objectweb.ccm.descriptor.softpkg.ccm.DependencyDeployer)
                         dep
                         .getExtensionManager()
                         .getExtensionByName("ccm.Deployer")
                         .getInstance())
                            .resolveDependency(deployTable,server_inst,dep);

                                
                    }
            }                   
    
    
        org.omg.Components.ConfigValue[] config
            = new org.omg.Components.ConfigValue[0];
        
   
        //create the container and save its configuration
        server_cont= server_cs.create_container(config);        

        // create the home with container configuration
        try
            {        
                for( java.util.Iterator i=implemToInstall.getCodeList().iterator();i.hasNext();){
                    CodeBean  code =(CodeBean) i.next();
                    if(code.getEntrypoint()!=null)
                        return server_cont.install_home(implemToInstall.getId(),
                                                        code.getEntrypoint().getValue().trim(), config);
                } 
                
               
            }catch(Error err){System.out.println(err.getMessage());}
        
        throw new CannotSetupHomeException();            
  }


    /**
     * Internal method to setup the componentinstantiation children of this homeplacement.
     * The deploy method is call on each browsed elements
     *
     * @param deployTable the internal table use to store information.
     * @param homePlaced the home to use for instantiation
     * @param hi ComponentInstantiation information.  
     */
    private void 
    proceedComponentInstantiation(java.util.Hashtable deployTable ,
                                  org.omg.Components.CCMHome homePlaced,
                                  Homeplacement hi) 
    throws Exception 
    {
        // browse the componentinstantiation taf
        for (java.util.Iterator i= hi.getComponentinstantiationList().iterator() ;
             i.hasNext() ; )
            {
                ComponentinstantiationBean current=(ComponentinstantiationBean)i.next();
                ((ComponentinstantiationDeployer)
                 current
                 .getExtensionManager().getExtensionByName("ccm.Deployer")
                 .getInstance()).deploy(deployTable,homePlaced,current);
            }        
    }
    
    /**
     * Internal to proceed the registration to registration service
     * like nameservice, homefinder and trading service.
     * 
     * TODO : Simplify the test of homefinder list
     *
     * @param deployTable the internal table use to store information.
     * @param homePlaced the home to register
     * @param hi the register information
     */
    private void 
    proceedHomeRegistration(java.util.Hashtable deployTable,
                            org.omg.Components.CCMHome homePlaced,
                            Homeplacement hi )
    throws Exception
    {

        // proceed with binding
        if(hi.getRegisterwithnamingList()!=null&&
           hi.getRegisterwithnamingList().size()!=0){
            
            for(java.util.Iterator i=hi.getRegisterwithnamingList().iterator();i.hasNext();){
                RegisterwithnamingBean current=(RegisterwithnamingBean)i.next();
                ((RegisterwithnamingDeployer)
                 current
                 .getExtensionManager().getExtensionByName("ccm.Deployer")
                 .getInstance()).rebind(deployTable,current,homePlaced);              
            
            }
        }
        
        try{
            // test if some homefinder registration must be done
            if(hi.getRegisterwithhomefinderList()!=null&&
               hi.getRegisterwithhomefinderList().size()>0){
                
                for(java.util.Iterator i=hi.getRegisterwithhomefinderList().iterator();
                    i.hasNext();)
                    {
                        RegisterwithhomefinderBean current=(RegisterwithhomefinderBean)i.next();
                        ((RegisterwithhomefinderDeployer)
                         current
                         .getExtensionManager().getExtensionByName("ccm.Deployer")
                         .getInstance()).register_home(deployTable,
                                                       current,
                                                       descriptor,
                                                       homePlaced);                      
                    }
            }
        }
        catch(Exception e)
            {
                System.err.println
                (">> HomeFinder Registration >> Problem occured during homefinder registration "); 
            }
        // test if some homefinder registration must be done
        if(hi.getRegisterwithtraderList()!=null&&
           hi.getRegisterwithtraderList().size()>0){
            
            
            // proceed with trader registration
            for (java.util.Iterator trader_reg_i = 
                     hi.getRegisterwithtraderList().iterator() ; 
                 trader_reg_i.hasNext() ; ) {
                //Proceed with registration
                RegisterwithtraderBean reg = (RegisterwithtraderBean)trader_reg_i.next();
                RegisterwithtraderDeployer dep;
                java.lang.Object dep_ref = 
                         reg.getExtensionManager()
                         .getExtensionByName("ccm.Deployer").getInstance();
                dep = (RegisterwithtraderDeployer)dep_ref;
                dep.deploy(homePlaced, reg);
            }
        }   

            

    }   


    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
    /**
     * Deploy is in charge to instance and setup the home on the precise 
     * destination. Special destiantion parameter is given for homeplacement
     * imbricated into hostcollocation or processcollocation.
     * There are 8 main traitments into this method :<br>
     *     <ul>
     *       <li>Retrieve the component_instantiation</li>
     *       <li>Gather information on the home instance( with csd file)</li>
     *       <li>Installation of needed package on target destination</li>
     *       <li>create a container for this home with correct configurations</li>
     *       <li>Instantiate the home </li>
     *       <li>Store the home reference into deployTable, extension manager tool</li>
     *       <li>Register the home on nameservice and homefinder if necessary </li>
     *       <li>Instantiate the component contained by this home</li>
     *     </ul>  
     * @param deployTable the internal table use to store information.
     * @param dest the target destination
     * @param hi Homeplacement information to use.
     */
    
    public void 
    deploy(java.util.Hashtable deployTable, 
           Destination dest,
           Homeplacement hi )
    {
        try {
            //if dest parameter is null the homeplacement 
            //destination must be consider
            if(dest==null)
                dest=hi.getDestination();
            
            //setup the componentinstantiation server
            setComponentInstantiation(deployTable,dest);
            
            setupSoftpkgDependency(deployTable,hi); 

            // setup the container and install homes with container configuration.
            homePlaced =setupHome(deployTable,hi) ;
            
            // storing the reference in the DeployInfo extension of the descriptor
            ( (HomeplacementDeployInfo)
              ((HomeplacementBean) hi).getExtensionManager().
              getExtensionByName("ccm.DeployInfo")
              .getInstance() ).setCorbaReference(homePlaced);

            // store the reference in the partition
            deployTable.put(hi.getId(),homePlaced);
            
            //Register the home in registration services definded                         
            proceedHomeRegistration(deployTable,homePlaced,hi);         
            
            /*setup the componentinstantiation*/
            proceedComponentInstantiation(deployTable,homePlaced,hi)        ;
            System.out.println("Home "+hi.getId()+"is  deployed");  
        }
        catch(Exception e){
            System.err.println("Probleme occured during the deployement of"+hi.getId());
            e.printStackTrace();

        }   
        
    }

    

    /**
     *Tear down the homeplacement but dont remove the file archive
     *
     * @param deployTable the internal table use to store information.
     * @param dest the target destination
     * @param hi Homeplacement information to use.
     */
    public void 
    tearDown(java.util.Hashtable deployTable, 
             Destination dest ,
             Homeplacement hi)
    {
        try
        {
            if(dest==null)
            dest=hi.getDestination();
            // Remove all the instantiation        
            for (java.util.Iterator i= hi.getComponentinstantiationList().iterator() ;
                 i.hasNext() ; ){
                //((ComponentinstantiationImpl)i.next()).tearDown(deployTable);
                ComponentinstantiationBean current=(ComponentinstantiationBean)i.next();
                try{
                    ((ComponentinstantiationDeployer)
                     current
                     .getExtensionManager().getExtensionByName("ccm.Deployer")
                     .getInstance()).tearDown(deployTable,current);
                }
                catch(Exception e){System.out.println("Cannot remove the instance");
                e.printStackTrace();
                }
            }                                    
            
            String ho="";
            try
            {
                
                if(hi.getRegisterwithnamingList()!=null&&
                   hi.getRegisterwithnamingList().size()!=0){
                    
                    
                    for(java.util.Iterator i=hi.getRegisterwithnamingList().iterator();i.hasNext();)
                        {
                            RegisterwithnamingBean current=(RegisterwithnamingBean)i.next();
                            ((RegisterwithnamingDeployer)
                             current
                             .getExtensionManager().getExtensionByName("ccm.Deployer")
                             .getInstance()).unbind();
                            
                        }
                }
            }
            catch(Exception e)
            {
                System.err.println("Cannot unbind "+ho+" from name service");                    
            }
            
            try 
            {
                
                if(hi.getRegisterwithtraderList()!=null&&
                   hi.getRegisterwithtraderList().size()!=0)
                {
                    
                    //Browse all the Trading Service registration tag
                    for (java.util.Iterator trader_reg_i = hi.getRegisterwithtraderList().iterator() ; 
                         trader_reg_i.hasNext() ; ) {
                             //Proceed with tear down
                             RegisterwithtraderBean reg = (RegisterwithtraderBean)trader_reg_i.next();
                             RegisterwithtraderDeployer dep;
                             java.lang.Object dep_ref = 
                             reg.getExtensionManager().getExtensionByName("ccm.Deployer").getInstance();
                             dep = (RegisterwithtraderDeployer)dep_ref;
                             dep.tearDown(reg);            
                         }
                }
            } 
            catch(Exception e)
            {
                System.err.println("Cannot from trading service");
            }

            for(java.util.Iterator i=hi.getRegisterwithhomefinderList().iterator();
                i.hasNext();)
            {
                RegisterwithhomefinderBean current=(RegisterwithhomefinderBean)i.next();
                ((RegisterwithhomefinderDeployer)
                 current
                 .getExtensionManager().getExtensionByName("ccm.Deployer")
                 .getInstance()).unregister_home();                                        
            } 
            
            // Remove the component server
            ((DestinationDeployer)((DestinationBean)dest)
             .getExtensionManager().getExtensionByName("ccm.Deployer")
             .getInstance()).removeComponentServer(server_cs);
            
            deployTable.remove(hi.getId());
        }
        catch(Exception e)
        {
            System.err.println(">> HomeFinder Registration >> Problem occured during homefinder unregistration ");
        }       

    }  
}
