/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm;

//Importation of the needed packages
import org.objectweb.ccm.descriptor.componentassembly.beans.*;
import org.objectweb.ccm.descriptor.componentassembly.*;
//import org.objectweb.ccm.Deployment.components.HFinder.*;
import org.objectweb.ccm.descriptor.softpkg.*;
import org.objectweb.ccm.descriptor.softpkg.beans.*;
//import org.objectweb.ccm.descriptor.corbacomponent.*;
//import org.objectweb.ccm.descriptor.corbacomponent.beans.*;
/**
 * Specific deployer allowing to register/unregister a home
 * in the homefinder.
 *
 * @author <a href="mailto:frederic.briclet@lifl.fr">Briclet Frederic</a>
 *
 * @version 0.1
 */

public class RegisterwithhomefinderDeployer {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /* the home registration cookie*/
    private org.omg.Components.Cookie cookie;
    /* Reference on the homeregistration interface*/
    private org.omg.Components.HomeRegistration hr;
    /*registration home name*/
    private String registrationName;
    /*System.err.println prefix*/
    private String prefix=">> HomeFinder Registration >>";
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================

    /**
     * Public method to register a home.
     *
     * @param deployTable the internal table use to store information.
     * @param rwhfi home information to use for registering.
     * @param descriptor descriptor containing CCD file path.
     * @param homeToRegister The home to register.  
     */
    public void
    register_home(java.util.Hashtable deployTable ,
                  RegisterwithhomefinderBean rwhfi,
                  org.objectweb.ccm.descriptor.softpkg.Descriptor descriptor,
                  org.omg.Components.CCMHome homeToRegister)
    throws Exception
    {    

        if(!deployTable.containsKey("__home_registration__"))
            {
       
                System.err.println
                   (prefix+ "Cannot found HomeFinder reference, the HomeFinder service hasn't be started");
                System.err.println
                   (prefix+ "To start the HomeFinder service from console just execute finder_start script");
                System.err.println
                    (prefix+"The HomeFinder registration will be skipped");
         /*Skip the registration*/
         return ;
        }
        hr=(org.omg.Components.HomeRegistration)deployTable.get("__home_registration__");
        String homeRepid="IDL:org.omg/Components/CCMHome:1.0";
        String componentRepid="IDL:org.omg/Components/CCMObject:1.0";
        try{         
  

            if(descriptor==null|| descriptor.getFileinarchive()==null){
                  System.err.println(prefix+" Cannot found a valid CCD descriptor for"+
                     registrationName +" home registration");
                     System.err.println(prefix+"componentrepid and homerepid cannot be filled for"+
                     registrationName +" home registration");
             }           
            else{
                 
                 org.objectweb.ccm
                    .descriptor.softpkg
                    .beans.FileinarchiveBean current=
                    (org.objectweb.ccm.
                     descriptor.softpkg
                    .beans.FileinarchiveBean) 
                    descriptor.getFileinarchive();
            
            java.io.InputStreamReader reader= 
                ((org.objectweb.ccm
                    .descriptor.softpkg
                    .ccm.FileinarchiveDeployer)
                 current
                 .getExtensionManager()
                 .getExtensionByName("ccm.Deployer")
                 .getInstance())
                .getInputStreamReader(deployTable,current);   
                 
                 org.objectweb.ccm.descriptor
                    .corbacomponent.beans   
                    .CorbacomponentBean  ccb=
                 org.objectweb.ccm.descriptor
                    .corbacomponent.beans.CorbacomponentBeanImpl
                    .unmarshalBean(reader,true);
                      
                  homeRepid=ccb.getHomerepid().getRepid();
                  componentRepid=ccb.getComponentrepid().getRepid();                    
            }
        }      
        catch(Exception e){
            System.err.println(prefix+"Cannot retrieve ComponentRepid and HomeRepid use default values");
        }
        
        try{
            registrationName=rwhfi.getName();
            cookie=
                /*((org.omg.Components.HomeRegistration)deployTable.get("__home_registration__"))*/
                hr.register_home(homeToRegister,
                                 homeRepid,
                                 componentRepid,
                                 registrationName);     
       }    
        catch(Exception e){
            System.err.println(prefix+"Problem occured during homefinder registration of"+ rwhfi.getName());
            System.err.println(e.getMessage());
        }        
    }
    
    /**
     * Public method to unregister the home registered by this deployer instance.
     */
    public void
    unregister_home()
    throws Exception
    {
        if(hr==null)
            return ;
        
        try{           
            if(cookie!=null)
                hr.unregister_home(cookie);    
        }
        catch(Exception e){            
            System.err.println(prefix+" Problem occured during unregistration of "+registrationName);
            System.err.println(e.getMessage());
        }
    }
        

}
