/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.generator.cif.lib;

// Package dependencies.
import org.objectweb.corba.generator.java.ast.api.*;
import org.objectweb.corba.generator.java.ast.lib.*;
import org.objectweb.corba.ast.api.Declaration;
import org.objectweb.corba.ast.api.DeclarationKind;
import org.objectweb.corba.ast.api.InterfaceDecl;
import org.objectweb.corba.ast.api.AttributeDecl;
import org.objectweb.corba.ast.api.OperationDecl;
import org.objectweb.corba.ast.api.ProvidesDecl;
import org.objectweb.corba.ast.api.ConsumesDecl;
import org.objectweb.corba.ast.api.InterfacePortDecl;


/**
 * This class generates CIF implementation templates files for a component.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class ComponentImplMapping
  implements org.objectweb.corba.generator.cif.api.ComponentImplMapping
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Utility class to convert types in Java*/
    static public org.objectweb.corba.generator.translator.idl2java.api.IDL_JavaTranslator translator_;

    /** The composition containing the component. */
    private org.objectweb.corba.generator.cif.api.CompositionInfo comp_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     *
     * @param composition - The composition to manage.
     **/
    public ComponentImplMapping(org.objectweb.corba.ast.api.CompositionDecl composition)
    {
        comp_ = new org.objectweb.corba.generator.cif.lib.CompositionInfo(composition);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Inits the static field.
     **/
    static
    {
        translator_ = new org.objectweb.corba.generator.translator.idl2java.lib.IDL_JavaTranslator();
    }

    /**
     * Call mapping actions for component business operations and attributes.
     * These operations and attributes can come from Monolithic executor
     * (base monolithic executor, provides and consumes related operations),
     * supported interfaces, and component attributes.  </br>
     * Implementation of provides related operations is checked only
     * with the operation name (starting with "get_"). This could
     * result in some trouble in some rare cases (e.g. an operation
     * caming from a supported interface and which name starts with "get_").
     *
     * @param clazz - Add methods to the Component class.
     */
    private void
    businessOperations(ClassObject clazz)
    {
        Declaration[] decls = null;
        InterfaceDecl itf = null;
        AttributeDecl attr = null;
        OperationDecl op = null;
        MethodObject method = null;

        itf = comp_.getComponent().getLocalMainMapping();
        decls = itf.getContents(false, DeclarationKind.dk_operation+
                                       DeclarationKind.dk_attribute);

        for (int i=0; i<decls.length; i++)
        {
            // Map Attributes
            if (decls[i].getDeclKind() == DeclarationKind.dk_attribute)
            {
                attr = (AttributeDecl)decls[i];

                method = translator_.map_attribute_accessor( (org.objectweb.corba.ast.api.AttributeDecl)decls[i] );
                method.getImpl().setMacro("BUSINESS_OP");
                method.getImpl().addContextValue("translator", translator_);
                method.getImpl().addContextValue("obj", decls[i]);
                clazz.addMethod(method);

                method = translator_.map_attribute_mutator( (org.objectweb.corba.ast.api.AttributeDecl)decls[i] );
                if (method != null)
                {
                    method.getImpl().setMacro("TODO");
                    clazz.addMethod(method);
                }
            }
            else
            // Map Operations
            {
                op = (OperationDecl)decls[i];

                if ( (!op.getParent().getAbsoluteName().startsWith("::Components")) &&
                     (!op.getName().equals("get_")) )
                {
                    method = translator_.map_operation(op);
                    method.getImpl().setMacro("BUSINESS_OP");
                    method.getImpl().addContextValue("translator", translator_);
                    method.getImpl().addContextValue("obj", op);
                    clazz.addMethod(method);
                }
            }
        }
    }

    /**
     * Generates all component provided facets operations to implement.
     *
     * @param clazz - Add methods to the Component class.
     */
    private void
    facetsOperations(ClassObject clazz)
    {

        Declaration[] decls = null;
        ProvidesDecl provide = null;
        ConsumesDecl consume = null;

        // Get "provides" and "consumes" declaration
        decls = comp_.getComponentExecutor().getMainSegmentFacets(comp_.getComponent());

        for (int i = 0; i<decls.length; i++)
        {
            if(decls[i].getDeclKind() == DeclarationKind.dk_provides)
            {
                provide = (ProvidesDecl)decls[i];
                translator_.mapInterfaceOps(clazz, provide.getLocalMapping());
            }
            else
            {
                consume = (ConsumesDecl)decls[i];
                translator_.mapInterfaceOps(clazz, consume.getEvent().getLocalConsumerMapping());
            }
        }
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.generator.cif.api.ComponentImplMapping.
    //
    // ==================================================================

    /**
     * Generate Component template class.
     *
     * @param repository - The java repository.
     **/
    public void
    toJavaComponentTemplate(Repository repository)
    {
        ClassObject clazz = null;
        ConstructorObject ct = null;

        clazz = new ClassObjectImpl(comp_.getComponent().getName()+"Impl");
        clazz.addComment(" This is the CIDL-based implementation of the");
        clazz.addComment(" OMG IDL3 "
                         + comp_.getComponent().getId()
                         + " component type.");
        clazz.addComment("");
        clazz.addComment(" @author OpenCCM CIF_Jimpl Compiler.");
        clazz.setModifier(ModifierKindImpl.mk_public);
        clazz.setPackage( translator_.getPackage(comp_.getHomeExecutor()) );
        clazz.addInheritedObject( translator_.getPackage(comp_.getHomeExecutor()) + "." +
                                  comp_.getComponentExecutor().getName() );

        // Add the default constructor
        ct = new ConstructorObjectImpl();
        clazz.addConstructor(ct);

        // Add Component methods
        businessOperations(clazz);
        facetsOperations(clazz);

        // Add object to the java repository
        repository.addObject(clazz);
    }

    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================

}
