/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.generator.cif.lib;

// Package dependencies.
import org.objectweb.corba.ast.api.CompositionDecl;
import org.objectweb.corba.ast.api.HomeDecl;
import org.objectweb.corba.ast.api.HomeExecutorDecl;
import org.objectweb.corba.ast.api.ComponentDecl;
import org.objectweb.corba.ast.api.ExecutorDecl;
import org.objectweb.corba.ast.api.SegmentDecl;
import org.objectweb.corba.ast.api.DeclarationKind;
import org.objectweb.corba.ast.api.Declaration;

/**
 * This class provides easy acces to composition elements.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class CompositionInfo
  implements org.objectweb.corba.generator.cif.api.CompositionInfo
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The composition to map. */
    private CompositionDecl composition_;

    /** The home to map. */
    private HomeDecl home_;

    /** The binding home executor. */
    private HomeExecutorDecl home_exec_;

    /** The component managed by the home to map. */
    private ComponentDecl comp_;

    /** The component executor binding to the component. */
    private ExecutorDecl comp_exec_;

    /** Segments contained in the component executor. */
    private SegmentDecl[] segments_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     *
     * @param composition - The composition to manage.
     **/
    public CompositionInfo(CompositionDecl composition)
    {
        Declaration[] decls = null;

        composition_ = composition;

        /** Get the home executor */
        decls = composition.getContents(true, DeclarationKind.dk_home_executor);
        // A composition has 1 home executor.
        home_exec_ = (org.objectweb.corba.ast.api.HomeExecutorDecl)decls[0];

        /** Get the component executor */
        decls = home_exec_.getContents(true, DeclarationKind.dk_executor);
        // A Home Executor has 1 component executor.
        comp_exec_ = (org.objectweb.corba.ast.api.ExecutorDecl)decls[0];

        /** Get the component */
        comp_ = home_exec_.getHomeType().getManagedComponent();

        /** Get the home */
        home_ = home_exec_.getHomeType();

        /** Get segments */
        decls = comp_exec_.getContents(true, DeclarationKind.dk_segment);
        segments_ = new SegmentDecl[decls.length];
        for (int i=0; i<decls.length; i++)
        {
            segments_[i] = (SegmentDecl)decls[i];
        }
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.corba.generator.cif.api.CompositionInfo.
    //
    // ==================================================================

    /**
     * Get the Component declaration.
     **/
    public CompositionDecl
    getComposition()
    {
        return composition_;
    }

    /**
     * Get the Home declaration.
     **/
    public HomeDecl
    getHome()
    {
        return home_;
    }

    /**
     * Get the Home executor declaration.
     **/
    public HomeExecutorDecl
    getHomeExecutor()
    {
        return home_exec_;
    }

    /**
     * Get the Component declaration.
     **/
    public ComponentDecl
    getComponent()
    {
        return comp_;
    }

    /**
     * Get the Component executor declaration.
     **/
    public ExecutorDecl
    getComponentExecutor()
    {
        return comp_exec_;
    }

    /**
     * Get segment declarations.
     **/
    public SegmentDecl[]
    getSegments()
    {
        return segments_;
    }

    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================

}
