/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.corba.generator.psdl.lib;

/** To access AST. */
import org.objectweb.corba.ast.api.StorageHomeKeyDecl;
import org.objectweb.corba.ast.api.AbstractStorageTypeDecl;
import org.objectweb.corba.ast.api.StorageTypeStateMemberDecl;

/** To access Java AST. */
import org.objectweb.corba.generator.java.ast.api.*;
import org.objectweb.corba.generator.java.ast.lib.*;

/** Others. */
import org.objectweb.corba.generator.common.lib.GenerationException;


/**
 * This class generates the mapping for a PSDL key.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</A>
 */
public class PSDLKeyMapping
  implements org.objectweb.corba.generator.psdl.api.PSDLKeyMapping
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The PSDL key Declaration. */
    private StorageHomeKeyDecl key_;

    // ===========================================================
    //
    // Constructors.
    //
    // ===========================================================

    /**
     * The default constructor.
     *
     * @param key - The Key to map.
     */

    public PSDLKeyMapping(StorageHomeKeyDecl key)
    {
        // Init internal state
        key_ = key;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Map a key definition into Java methods.
     *
     * @param generator - The PSDL to Java generator.
     * @param ast - The storage type managed by the storage home.
     * @param method - The Key mapping object.
     */
    public void
    mapFactoryParameters(
           org.objectweb.corba.generator.translator.idl2java.api.PSDL_JavaTranslator translator,
           AbstractStorageTypeDecl ast,
           MethodObject method
                        )
    {
        String[] members = null,
                 members_type = null,
                 states_name = null;
        StorageTypeStateMemberDecl sm = null;
        org.objectweb.corba.ast.api.StorageTypeDecl st = null;

        members = key_.getMemberList().getStrings();
        members_type = new java.lang.String[members.length];
        states_name = new java.lang.String[members.length];
        try{
            st = (org.objectweb.corba.ast.api.StorageTypeDecl) ast;
        }catch(java.lang.ClassCastException ex){
            st = null;
        }
        for (int i=0; i<members.length; i++)
        {
            // Retrieve the corresponding field
            sm = ast.getState(members[i]);
            members_type[i] = translator.toJava( sm.getType() );
            states_name[i] = "_" + members[i] + "_state";
            if (st == null)
            {
                if ( PSDLMappingTools.getMarshalledType(st, sm) != null)
                    states_name[i] = "_" + members[i] + "_marshalled";
            }
        }
        method.getImpl().addContextValue("fields_names", members);
        method.getImpl().addContextValue("states_names", states_name);
        method.getImpl().addContextValue("fields_types", members_type);
        method.getImpl().addContextValue("nb", new Integer(members.length) );
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Map a key definition into Java methods.
     *
     * @param generator - The PSDL to Java generator.
     * @param obj - The StorageHome mapping object.
     * @param ast - The abstract storage type defining this key.
     */
    public void
    toJava(org.objectweb.corba.generator.psdl.api.PSDL2JavaGenerator generator,
           InterfaceObject obj,
           AbstractStorageTypeDecl ast)
    throws GenerationException
    {
        MethodObject method = null;

        // Add the find_by_<key_name> operation
        method = new MethodObjectImpl();
        method.addComment("The find_by_"+key_.getName()+" method.");
        method.setName("find_by_"+key_.getName());
        method.setReturnType( generator.getTranslator().getAbsoluteName(ast) );
        StateMemberMapping.mapStateListToParameters
            ( generator,
              key_.getMemberList().getStrings(),
              method,
              ast );
        method.addException("org.omg.CosPersistentState.NotFound");
        method.getImpl().setMacro("SH_KEY_FIND_BY");
        mapFactoryParameters(generator.getTranslator(), ast, method);
        method.getImpl().addContextValue("st_type", generator.getTranslator().getAbsoluteName(ast));
        obj.addMethod(method);

        // Add the find_ref_by_<key_name> operation
        method = new MethodObjectImpl();
        method.addComment("The find_ref_by_"+key_.getName()+" method.");
        method.setName("find_ref_by_"+key_.getName());
        method.setReturnType("byte[]");
        StateMemberMapping.mapStateListToParameters
            ( generator,
              key_.getMemberList().getStrings(),
              method,
              ast );
        mapFactoryParameters(generator.getTranslator(), ast, method);
        method.getImpl().setMacro("SH_KEY_FIND_REF_BY");
        method.getImpl().addContextValue("key_name", key_.getName());
        obj.addMethod(method);
    }

}
