/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Offroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.trace.PI;

/**
 * @author offroy@lifl.fr
 *
 * used to read common thing from a RequestInfo
 */

public abstract class RequestInfoReader {
	protected int id_unique = 0;

	/**
	 * default log level for CORBA Interceptors 
	 */
	protected short logLevel = 1;

	/**
	 * Service context identifiers
	 */
	protected final int REQUEST_CONTEXT_ID = 100;
	protected final int REPLY_CONTEXT_ID = 101;

	/**
	 * Constructor 
	 * @param logLevel
	 **/
	public RequestInfoReader(short logLevel) {
		setLogLevel(logLevel);
	}

    /************************************************************************
     * Protected Method 
     ***********************************************************************/
	/**
	 * Display PortableInterceptor::RequestInfo
	 */
	protected String displayRequestInfo(
		org.omg.PortableInterceptor.RequestInfo info,
		short logLevel) {
		String res = "";
		res = res
				+ displayRequestID(info)
				+ displayOperation(info)
				+ displayArguments(info)
				+ displayExceptions(info)
				+ displayResult(info)
				+ displayResponseExpected(info)
				+ displayReplyStatus(info);
		if (logLevel > 1) {
			res = res + displayRequestInfoMore(info);
		}
		return res;
	}

	/**
	 * Display PortableInterceptor::RequestInfo RequestID
	 * @param info
	 * @return String
	 */
	protected String displayRequestID(
		org.omg.PortableInterceptor.RequestInfo info) {
		return "request id = " + info.request_id();
	}

	/**
	 * Display PortableInterceptor::RequestInfo Operation 
	 * @param info
	 * @return String
	 */
	protected String displayOperation(
		org.omg.PortableInterceptor.RequestInfo info) {
		return "\toperation = " + info.operation() + "\t";
	}

	/**
	 * Display PortableInterceptor::RequestInfo ResponseExpected
	 * @param info
	 * @return String
	 */
	protected String displayResponseExpected(
		org.omg.PortableInterceptor.RequestInfo info) {
		return "\tresponse expected = "
			+ (info.response_expected() ? "true" : "false");
	}

	/**
	 * Display PortableInterceptor::RequestInfo Arguments 
	 * @param info
	 * @return String
	 */
	protected String displayArguments(
		org.omg.PortableInterceptor.RequestInfo info) {
		String res = "\targuments = ";
		try {
			org.omg.Dynamic.Parameter[] args = info.arguments();
			if (args.length == 0) {
				res = res + "(no arguments)";
			} else {
				for (int i = 0; i < args.length; i++) {
					res = res + "\n";
					if (args[i].mode == org.omg.CORBA.ParameterMode.PARAM_IN) {
						res = res + "in ";
					} else if (
						args[i].mode
							== org.omg.CORBA.ParameterMode.PARAM_OUT) {
						res = res + "out ";
					} else { // case PARAM_INOUT:
						res = res + "inout ";
					}
					res = res + args[i].argument;
				}
			}
			res = res + "\n";
		} catch (org.omg.CORBA.BAD_INV_ORDER ex) {
			//res = res + "Operation not supported in this context";
		} catch (org.omg.CORBA.NO_RESOURCES ex) {
			//res = res + "Operation not supported in this environment";
		}
		return res;
	}

	/**
	 * Display PortableInterceptor::RequestInfo Exception 
	 * @param info
	 * @return String
	 */
	protected String displayException(
		org.omg.PortableInterceptor.RequestInfo info) {
		String res = "";
		try {
			org.omg.CORBA.Any exception = requestOpException(info);
			res = res + "\t" + getLabelException() + exception;
		} catch (org.omg.CORBA.BAD_INV_ORDER ex) {
			//res = res + "Operation not supported in this context";
		}
		return res;
	}

	/**
	 * Display PortableInterceptor::RequestInfo Exceptions
	 * @param info
	 * @return String
	 */
	protected String displayExceptions(
		org.omg.PortableInterceptor.RequestInfo info) {
		String res = "\texceptions = ";
		try {
			org.omg.CORBA.TypeCode[] exceptions = info.exceptions();
			if (exceptions.length == 0) {
				res = res + "\n" + "(no exceptions)";
			} else {
				res = res + "\n";
				for (int i = 0; i < exceptions.length; i++)
					res = res + exceptions[i];
			}
		} catch (org.omg.CORBA.BAD_INV_ORDER ex) {
			//res = res + "Operation not supported in this context";
		} catch (org.omg.CORBA.NO_RESOURCES ex) {
			//res = res + "Operation not supported in this environment";
		}
		return res;
	}

	/**
	 * Display PortableInterceptor::RequestInfo Result 
	 * @param info
	 * @return String
	 */
	protected String displayResult(
		org.omg.PortableInterceptor.RequestInfo info) {
		String res = "";
		try {
			org.omg.CORBA.Any result = info.result();

			res = res + "\tresult = ";
			res = res + result;
		} catch (org.omg.CORBA.BAD_INV_ORDER ex) {
			//res = res + "Operation not supported in this context";
		} catch (org.omg.CORBA.NO_RESOURCES ex) {
			//res = res + "Operation not supported in this environment";
		}
		return res;
	}

	/**
	 * Display PortableInterceptor::RequestInfo Reply Status
	 * @param info
	 * @return String
	 */
	protected String displayReplyStatus(
		org.omg.PortableInterceptor.RequestInfo info) {
		String res = "";
		try {
			short status = info.reply_status();

			res = res + "\treply status = ";
			if (status == org.omg.PortableInterceptor.SUCCESSFUL.value)
				res = res + "SUCCESSFUL";
			else if (
				status == org.omg.PortableInterceptor.SYSTEM_EXCEPTION.value)
				res = res + "SYSTEM_EXCEPTION";
			else if (
				status == org.omg.PortableInterceptor.USER_EXCEPTION.value)
				res = res + "USER_EXCEPTION";
			else if (
				status == org.omg.PortableInterceptor.LOCATION_FORWARD.value)
				res = res + "LOCATION_FORWARD";
			//else if (status == org.omg.PortableInterceptor.
			//     LOCATION_FORWARD_PERMANENT.value)
			//res = res +"LOCATION_FORWARD_PERMANENT";
			else if (
				status == org.omg.PortableInterceptor.TRANSPORT_RETRY.value)
				res = res + "TRANSPORT_RETRY";
			else
				res = res + "UNKNOWN_REPLY_STATUS";
		} catch (org.omg.CORBA.BAD_INV_ORDER ex) {
			//res = res + "Operation not supported in this context";
		}
		return res;
	}
	/* Display PortableInterceptor::RequestInfo More */
	protected String displayRequestInfoMore(
		org.omg.PortableInterceptor.RequestInfo info) {
		return displaySlotData(info)
			+ displayRequestServiceContext(info)
			+ displayReplyServiceContext(info);
	}

	/**
	 * Display PortableInterceptor::RequestInfo SlotData
	 * @param info
	 * @return String
	 */
	protected String displaySlotData(
		org.omg.PortableInterceptor.RequestInfo info) {
		String res = "\n";
		/*try{
		org.omg.CORBA.Any slotData = info.get_slot(mySlotId_);
		
		res = res +"slot data " + mySlotId_ + " = \n";
		
		res = res +slotData;
		res = res +"\n";
		}catch(org.omg.PortableInterceptor.InvalidSlot ex){}
		*/
		return res;
	}

	/**
	 * Display PortableInterceptor::RequestInfo RequestServiceContext 
	 * @param info
	 * @return String
	 */
	protected String displayRequestServiceContext(
		org.omg.PortableInterceptor.RequestInfo info) {
		String res = "";
		try {
			org.omg.IOP.ServiceContext context =
				info.get_request_service_context(REQUEST_CONTEXT_ID);

			res = res + "request service context " + REQUEST_CONTEXT_ID + " = ";
			for (int i = 0; i < context.context_data.length; i++)
				res = res + getOctet(context.context_data[i]);
		} catch (org.omg.CORBA.BAD_INV_ORDER ex) {
			//res = res + "Operation not supported in this context";
		} catch (org.omg.CORBA.BAD_PARAM ex) {
			//res = res + "No entry found";
		}
		return res;
	}

	/**
	 * Display PortableInterceptor::RequestInfo ReplyServiceContext 
	 * @param info ResuestInfo to use
	 **/
	protected String displayReplyServiceContext(
		org.omg.PortableInterceptor.RequestInfo info) {
		String res = "";
		try {
			org.omg.IOP.ServiceContext context =
				info.get_reply_service_context(REPLY_CONTEXT_ID);

			res = res + "\treply service context " + REPLY_CONTEXT_ID + " = ";
			for (int i = 0; i < context.context_data.length; i++)
				res = res + getOctet(context.context_data[i]);
		} catch (org.omg.CORBA.BAD_INV_ORDER ex) {
			//res = res + "Operation not supported in this context";
		} catch (org.omg.CORBA.BAD_PARAM ex) {
			//res = res + "No entry found";
		}
		return res;
	}

	/**
	 *  Get next message id 
	 * @return int 
	 **/
	protected int nextId() {
		return id_unique++;
	}

	/**
	* Sets the logLevel.
	* @param logLevel The logLevel to set
	*/
	public void setLogLevel(short logLevel) {
		this.logLevel = logLevel;
	}

	/**
	 * Returns the logLevel.
	 * @return short
	 **/
	public short getLogLevel() {
		return logLevel;
	}

	/* Convert CORBA::Octet to string */
	protected String getOctet(byte octet) {
		String prefix;
		if (octet < 0x10)
			prefix = "<0";
		else
			prefix = "<";

		return prefix + Integer.toHexString(octet) + ">";
	}

    /************************************************************************
     * Abstract implementation  Method 
     ***********************************************************************/
	protected abstract String getLabelException();

	protected abstract String displayMore(
		org.omg.PortableInterceptor.RequestInfo info);

	protected abstract org.omg.CORBA.Any requestOpException(
		org.omg.PortableInterceptor.RequestInfo info);

}
