# ====================================================================
#
# OpenCCM: The Open CORBA Component Model Platform
# Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
# Contact: openccm@objectweb.org
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
# Initial developer(s): Pierre Carpentier.
# Contributor(s): ______________________________________.
#
# ====================================================================
#
#  This class feeds the AST from the differents elements in the
#  UML repository. This file is build according to the UML Profile
#  for CORBA (formal/02-04-01) [1] and to the UML Profile for
#  CORBA Components (Version 3.1) [2].
#
# ====================================================================

# TO BE COMPLETED : CORBAConstant (AnyValue), fixed, arrays

import NotCreatedException;
import UMLOperations;

class ASTFeeder
{

    # =================================================================
    # Empty Constructor for the AST feeder.
    # =================================================================
    proc __ASTFeeder__(self)
    {
        self.umlOperations = UMLOperations.UMLOperations();
        self._types = {};
        self._to_generate = {};
        self._generated_module = array.create(0);

        JAVA.declare("org.objectweb.corba.ast.lib");
        JAVA.declare("org.objectweb.ccm");
        JAVA.declare("java.util");
        ir = CORBA.ORB.resolve_initial_references("InterfaceRepository");
        self.ast = org.objectweb.corba.ast.lib.Repository(org.objectweb.ccm.ComponentRepositoryHelper.narrow(ir));

        self.scope = self.ast.startFileScope("");
        self.scope.create();
        self.prefix = "";
    }


    # =================================================================
    # Puts the type in the hashtable.
    # element - The UML element corresponding to the type
    # type - The type itself
    # =================================================================
    proc putType (self, element, type)
    {
        self._types[element] = type;
    }


    # =================================================================
    # Gets the type from the hashtable. If the type is not present in
    # the hashtable, the method throws the NotCreatedException.
    # element - The UML element corresponding to the type
    # return - The type itself
    # =================================================================
    proc getType (self, element)
    {
        if (!self.typeExist(element)) {
            throw NotCreatedException.NotCreatedException(element.name());
        }
        return self._types[element];
    }


    # =================================================================
    # Tests if the type is present in the hashtable.
    # element - The UML element corresponding to the type
    # return - true if the type is present, else false
    # =================================================================
    proc typeExist (self, element)
    {
        return self._types.containsKey(element);
    }


    # =================================================================
    # Sets the element as generated.
    # element - The UML element
    # =================================================================
    proc setGenerated (self, element)
    {
        if (self._to_generate.containsKey(element))
            self._to_generate.remove(element);
    }


    # =================================================================
    # Feeds the Interface Repository from the UML model.
    # umlPackage - The UML Package
    # =================================================================
    proc createModel(self, umlPackage)
    {
        self.umlPackage = umlPackage;
        self.undefinedType = false;
        try {
            self.umlOperations.setUMLPackage(umlPackage);
            packages = umlPackage.model_management_ref.package_ref.all_of_type_package;
            self.create(packages, self.scope);
            while (self._to_generate.size > 0) {
                oldSize = self._to_generate.size;
                for element in self._to_generate.keys {
                    self.createElement(element, self._to_generate[element]);
                }
                if (oldSize == self._to_generate.size) {
                    self.undefinedType = true;
                }
            }
        } catch (exception) {
            for module in self._generated_module {
                module.destroy();
            }
            throw exception;
        }
    }


    # =================================================================
    # Create some elements of the model into the Interface Repository.
    # modelElements - The elements
    # currentScope - The current scope in which create the element
    # =================================================================
    proc create (self, modelElements, currentScope)
    {
        for element in modelElements {
            self.createElement(element, currentScope);
        }
    }


    # =================================================================
    # Create one element of the model into the Interface Repository.
    # element - The element
    # currentScope - The current scope in which create the element
    # =================================================================
    proc createElement (self, element, currentScope)
    {
        stereotypes = element.stereotype();
        if (stereotypes.length == 0) {
            self.createContents(element, currentScope);
        } else {
            try {
                for stereotype in stereotypes {
                    eval("self.create" + stereotype.name() + "(element, currentScope);");
                }
                self.setGenerated(element);
            } catch (NotCreatedException.NotCreatedException nce) {
                self._to_generate[element] = currentScope;
                if (self.undefinedType)
                    throw nce;
            } catch (NotFound nf) {
                self.createContents(element, currentScope);
            }
        }
    }

    proc createContents (self, element, currentScope) {
        if (element._is_a(UML.Core.Attribute)) {
            self.createAttribute(element, currentScope);
        } else if (element._is_a(UML.Core.Operation)) {
            self.createOperation(element, currentScope)
        } else if (element._is_a(UML.Core.UmlClass)) {
            # Some UML Tools put Exception into the Exception class.
            for ownedEl in element.owned_element() {
                if (ownedEl._is_a(UML.CommonBehavior.Exception)) {
                    self.createElement(ownedEl, currentScope);
                }
            }
        }
    }


    # =================================================================
    #   [1] - 3.5.1 - CORBA Basic Types
    # =================================================================
    proc typeOfObject (self, obj)
    {
        try {
            typename = obj.type().name();
            typenameLower = typename.toLowerCase();
            if (typenameLower == "short")
                return self.ast.getShortType();
            if (typenameLower == "long")
                return self.ast.getLongType();
            if (typenameLower == "long long")
                return self.ast.getLongLongType();
            if (typenameLower == "double")
                return self.ast.getDoubleType();
            if (typenameLower == "long double")
                return self.ast.getLongDouble();
            if (typenameLower == "unsigned short")
                return self.ast.getUShortType();
            if (typenameLower == "unsigned long")
                return self.ast.getULongType();
            if (typenameLower == "unsigned long long")
                return self.ast.getULongLongType();
            if (typenameLower == "any")
                return self.ast.getAnyType();
            if (typenameLower == "boolean")
                return self.ast.getBooleanType();
            if (typenameLower == "string")
                return self.ast.getStringType();
            if (typenameLower == "octet")
                return self.ast.getOctetType();
            if (typenameLower == "void")
                return self.ast.getVoidType();
            if (typenameLower == "char")
                return self.ast.getCharType();
            if (typenameLower == "wchar")
                return self.ast.getWCharType();
            if (typenameLower == "float")
                return self.ast.getFloatType();
            if (typenameLower == "wstring")
                return self.ast.getWStringType();
            if (typenameLower == "typecode")
                return self.ast.getTypeCodeType();
            if (typenameLower == "principal")
                return self.ast.getPrincipalType();
            if (typenameLower == "object")
                return self.ast.getObjectType();
            if (typenameLower == "valuebase")
                return self.ast.getValueBaseType();
            if (typenameLower == "native") {
                if (!self.typeExist(obj)) {
                    native = self.currentModule.startNative(obj.name());
                    native.create();
                    self.putType(obj, native);
                }
                return self.getType(obj);
            }
            if (typenameLower.index("fixed") == 0) {
                token = java.util.StringTokenizer(typenameLower, "fixed<>, ");
                if (token.countTokens() == 2) {
                    digits = long._fromString(token.nextToken());
                    scale = long._fromString(token.nextToken());
                    return self.ast.createFixedType(digits, scale);
                }
            }
        } catch (exception) {
            return self.getType(obj);
        }
        return self.getType(obj.type());
    }


    # =================================================================
    #   [1] - 3.5.4 - Module
    # =================================================================
    proc createCORBAModule (self, element, currentScope)
    {
        oldPrefix = self.prefix;
        module = currentScope.declareModule(element.name());
        self.currentModule = module;
        for tv in element.tagged_value() {
            if (tv.type().tag_type() == "typePrefix" || tv.type().tag_type() == "CORBATypePrefix") {
                self.prefix = tv.data_value()[0];
            }
        }
        module.setPrefix(self.prefix);
        module.create();
        self._generated_module.append(module);
        self.create(element.owned_element(), module);
        self.prefix = oldPrefix;
    }


    # =================================================================
    #   [1] - 3.5.6 - Interface
    # =================================================================
    proc createCORBAInterface (self, element, currentScope)
    {
        try {
            if (element.is_abstract()) {
                interf = currentScope.declareAbstractInterface(element.name());
            } else {
                isLocal = "false";
                for tv in element.tagged_value() {
                    if (tv.type().tag_type() == "isLocal" || tv.type().tag_type() == "CORBAIsLocal") {
                        isLocal = tv.data_value()[0];
                    }
                }
                if (isLocal.toLowerCase() == "true") {
                    interf = currentScope.declareLocalInterface(element.name());
                } else {
                    interf = currentScope.declareInterface(element.name());
                }
            }
            for g in element.generalization() {
                if (self.umlOperations.isStereotyped(g.parent(), "CORBAInterface"))
                    interf.getSupportedInterfaceList().add(self.getType(g.parent()));
            }
            interf.create();
            self.create(element.feature(), interf);
            self.putType(element, interf);
            # println("> Interface created : ", element.name());
        } catch (NotCreatedException.NotCreatedException nce) {
            interf.destroy();
            throw nce;
        }
    }


    # =================================================================
    #   [1] - 3.5.7 - Value Types
    # =================================================================
    proc createCORBAValue (self, element, currentScope)
    {
        value = currentScope.declareValue(element.name());
        if (element.is_abstract())
            value.setAbstract(true);
        self.createCORBAValueContents(value, element);
        value.create();
    }


    # =================================================================
    #   [1] - 3.5.7 - Value Types - CORBACustomValue
    # =================================================================
    proc createCORBACustomValue (self, element, currentScope)
    {
        value = currentScope.declareValue(element.name());
        if (element.is_abstract())
            value.setAbstract(true);
        value.setCustom(true);
        self.createCORBAValueContents(value, element);
        value.create();
    }


    # =================================================================
    #   [1] - 3.5.7 - Value Types - CORBAValue & CORBACustomValue Contents
    # =================================================================
    proc createCORBAValueContents (self, value, element)
    {
        try {
            # generalizations
            for g in element.generalization() {
                if (self.umlOperations.isStereotyped(g, "CORBAValueSupports")) {
                    self.createCORBAValueSupports(g.parent(), value);
                } else {
                    if (g.parent().is_abstract()) {
                        value.getAbstractValueList().add(self.getType(g.parent()));
                    } else {
                        value.setBaseValue(self.getType(g.parent()));
                    }
                }
            }
            value.create();
            # attributes
            for statemember in element.feature() {
                if (statemember._is_a(UML.Core.Attribute)) {
                    self.createAttribute(statemember, value, true);
                }
            }
            # attributes in association
            self.createAssociationAttribute(element, value, true);
            # operations
            for operation in element.feature() {
                if (operation._is_a(UML.Core.Operation)) {
                    if (self.umlOperations.isStereotyped(operation, "CORBAValueFactory")) {
                        op = value.startInitializer(operation.name());
                        value.getInitializerList().add(op);
                    } else {
                        op = value.startOperation(operation.name());
                        if (self.umlOperations.isStereotyped(operation, "oneway"))
                            op.setOneway();
                    }
                    self.setOperationParameters(op, operation);
                    self.setOperationExceptions(op, operation);
                    op.create();
                }
            }
            self.putType(element, value);
        } catch (NotCreatedException.NotCreatedException nce) {
            value.destroy();
            throw nce;
        }
    }


    # =================================================================
    #   [1] - 3.5.7 - Value Types - CORBAValueSupports
    # =================================================================
    proc createCORBAValueSupports (self, element, value)
    {
        value.getSupportedInterfaceList().add(self.getType(element));
    }


    # =================================================================
    #   [1] - 3.5.7 - Value Types - CORBATruncatable
    # =================================================================
    proc createCORBATruncatable (self, element, value)
    {
        value.setTruncatable(true);
        value.getAbstractValueList().add(self.getType(element));
    }


    # =================================================================
    #   [1] - 3.5.7 - Value Types - CORBAValueFactory
    # =================================================================


    # =================================================================
    #   [1] - 3.5.8 - CORBA Wrapper Types
    # =================================================================


    # =================================================================
    #   [1] - 3.5.9 - Typedef
    # =================================================================


    # =================================================================
    #   [1] - 3.5.10 - Boxed Value Types
    # =================================================================
    proc createCORBABoxedValue (self, element, currentScope)
    {
        try {
            value = currentScope.declareValue(element.name());
            if (element.is_abstract())
                value.setAbstract(true);
            value.create();
            for statemember in self.umlOperations.allAttributes(element) {
                member = value.startValueMember(statemember.name());
                if (statemember.visibility() == UML.DataTypes.VisibilityKind.vk_private)
                    member.setPrivate()
                else
                    member.setPublic();
                member.setType(self.typeOfObject(feature));
                member.create();
            }
            self.putType(element, value);
        } catch (NotCreatedException.NotCreatedException nce) {
            value.destroy();
            throw nce;
        }
    }


    # =================================================================
    #   [1] - 3.5.11 - Constant Declaration - CORBAConstants (Class)
    # =================================================================
    proc createCORBAConstants (self, element, currentScope)
    {
        for constant in element.feature() {
            self.createCORBAConstant(constant, currentScope);
        }
    }


    # =================================================================
    #   [1] - 3.5.11 - Constant Declaration - CORBAConstant (Attribute)
    #   TO BE COMPLETED : AnyValue
    # =================================================================
    proc createCORBAConstant (self, element, currentScope)
    {
        try {
            constant = currentScope.startConstant(element.name());
            constant.setType(self.typeOfObject(element));
            # value = AnyValue(element.initial_value().body());
            # setAnyValue TO BE COMPLETED.
            constant.setAnyValue(CORBA.ORB.create_any());
            constant.create();
            self.putType(element, constant);
        } catch (NotCreatedException.NotCreatedException nce) {
            constant.destroy();
            throw nce;
        }
    }


    # =================================================================
    #   [1] - 3.5.14 - CORBAStruct
    # =================================================================
    proc createCORBAStruct (self, element, currentScope)
    {
        try {
            struct = currentScope.declareStruct(element.name());
            struct.create();
            for attrib in self.umlOperations.allAttributes(element) {
                if (self.umlOperations.isStereotyped(attrib, "CORBAAnonymousSequence")) {
                    struct.getMemberList().addMember(attrib.name(), self.getCORBAAnonymousSequence(attrib, currentScope));
                } else {
                    struct.getMemberList().addMember(attrib.name(), self.typeOfObject(attrib));
                }
            }
            struct.create();
            self.putType(element, struct);
        } catch (NotCreatedException.NotCreatedException nce) {
            struct.destroy();
            throw nce;
        }
    }


    # =================================================================
    #   [1] - 3.5.15 - Discriminated Union
    # =================================================================
    proc createCORBAUnion (self, element, currentScope)
    {
        try {
            union = currentScope.declareUnion(element.name());
            self.createCORBAUnionSwitch(element, union);
            self.createCORBAUnionSwitchEnd(element, union);
            union.create();
            self.putType(element, union);
        } catch (NotCreatedException.NotCreatedException nce) {
            union.destroy();
            throw nce;
        }
    }

    proc createCORBAUnionSwitch (self, element, union)
    {
        for attrib in self.umlOperations.allAttributes(element) {
            if (self.umlOperations.isStereotyped(attrib.type(), "switch") || self.umlOperations.isStereotyped(attrib, "switch")) {
                union.setDiscriminator(self.typeOfObject(attrib));
                for asso in self.umlOperations.associations(element) {
                    for end in asso.connection() {
                        if (end.participant() != element) {
                            for case in self.listCase(end.tagged_value()) {
                                union.getMemberList().addMember(end.name(), self.getType(end.participant()), self.caseLabels(case, attrib.type()));
                            }
                        }
                    }
                }
            }
        }
    }

    proc createCORBAUnionSwitchEnd (self, element, union)
    {
        for end in self.umlOperations.navigableOppositeEnds(element) {
            if (self.umlOperations.isStereotyped(end.participant(), "switchEnd") || self.umlOperations.isStereotyped(end, "switchEnd")) {
                union.setDiscriminator(self.getType(end.participant()));
                union.create();
                for attrib in self.umlOperations.allAttributes(element) {
                    for case in self.listCase(attrib.tagged_value()) {
                        union.getMemberList().addMember(attrib.name(), self.typeOfObject(attrib), self.caseLabels(case, end.participant()));
                    }
                }
            }
        }
    }

    proc listCase(self, taggedValues)
    {
        value = "";
        listcase = array.create(0);
        for tv in taggedValues {
            if (tv.type().tag_type() == "Case") {
                value = tv.data_value()[0];
            }
        }
        token = java.util.StringTokenizer(value, "(), ");
        while (token.hasMoreTokens()) {
            listcase.append(token.nextToken());
        }
        return listcase;
    }

    proc caseLabels(self, case, switch)
    {
        typename = switch.name().toLowerCase();
        anyValue = self.ast.createAnyValue();
        if (self.umlOperations.isStereotyped(switch, "CORBAEnum")) {
            for feat in switch.feature() {
                if (feat.name() == case) {
                    anyValue.setAsEnum(self.getType(switch).getIDLType().type(), self.getIDLOrder(feat), case);
                }
            }
        }
        if (typename == "long long")
            anyValue.setAsInteger(long._fromString(case));
        if (typename == "string")
            anyValue.setAsString(case);
        if (typename == "wstring")
            anyValue.setAsWString(case);
        if (typename == "char")
            anyValue.setAsChar(case[0]);
        if (typename == "wchar")
            anyValue.setAsWChar(case[0]);
        if (typename == "fixed")
            anyValue.setAsFixed(case);
        if (typename == "double")
            anyValue.setAsFloating(double._fromString(case));
        if (typename == "boolean") {
            if (case.toLowerCase() == "true")
                anyValue.setAsBoolean(true)
            else
                anyValue.setAsBoolean(false);
        }
        return anyValue;
    }

    proc getIDLOrder (self, feature)
    {
        for tv in feature.tagged_value() {
            if (tv.type().name() == "IDLOrder" || tv.type().name() == "CORBAIDLOrder") {
                value = tv.data_value()[0];
                return long._fromString(value);
            }
        }
    }


    # =================================================================
    #   [1] - 3.5.16 - CORBAEnum
    # =================================================================
    proc createCORBAEnum (self, element, currentScope)
    {
        try {
            enum = currentScope.startEnum(element.name());
            list = enum.getMemberList();
            index = 0;
            for feature in element.feature() {
                list.add(self.getCORBAEnumElement(element, index));
                index = index + 1;
            }
            enum.create();
            self.putType(element, enum);
            # println("> Enumeration created : ", element.name());
        } catch (NotCreatedException.NotCreatedException nce) {
            enum.destroy();
            throw nce;
        }
    }

    proc getCORBAEnumElement (self, element, idlOrder)
    {
        features = element.feature();
        for feature in features {
            for tv in feature.tagged_value() {
                if (tv.type().name() == "IDLOrder" || tv.type().name() == "CORBAIDLOrder") {
                    if (tv.data_value()[0] == idlOrder._toString()) {
                        return feature.name();
                    }
                }
            }
        }
        throw NotCreatedException.NotCreatedException(element.name() + " - IDLOrder : " + idlOrder);
    }


    # =================================================================
    #   [1] - 3.5.17 - Exception
    # =================================================================
    proc createCORBAException (self, element, currentScope)
    {
        try {
            excep = currentScope.startException(element.name());
            excep.create();
            self.create(element.feature(), excep);
            self.putType(element, excep);
            # println("> Exception created : ", element.name());
        } catch (NotCreatedException.NotCreatedException nce) {
            excep.destroy();
            throw nce;
        }
    }


    # =================================================================
    #   [1] - 3.5.19 - CORBASequence
    # =================================================================
    proc createCORBASequence (self, element, currentScope)
    {
        try {
            sequence = self.ast.createSequenceType(1, self.typeOfObject(element));
            sequence.create();
            alias = currentScope.startAlias(element.name());
            alias.setType(sequence);
            self.putType(element, alias);
            # println("> Sequence created : ", element.name());
        } catch (NotCreatedException.NotCreatedException nce) {
            sequence.destroy();
            throw nce;
        }
    }


    # =================================================================
    #   [1] - 3.5.19 - CORBAAnonymousSequence
    # =================================================================
    proc getCORBAAnonymousSequence (self, element, currentScope)
    {
        return self.ast.createSequenceType(1, self.typeOfObject(element));
    }


    # =================================================================
    #   [1] - 3.5.20 - Array - CORBAArray
    # =================================================================


    # =================================================================
    #   [1] - 3.5.20 - Array - CORBAAnonymousArray
    # =================================================================


    # =================================================================
    #   [1] - 3.5.21 - Fixed Type
    # =================================================================


    # =================================================================
    #   [1] - 3.13 - Operation Declaration
    # =================================================================
    proc createOperation (self, element, currentScope)
    {
        try {
            operation = currentScope.startOperation(element.name());
            self.setOperationParameters(operation, element);
            self.setOperationExceptions(operation, element);
            self.setOperationContext(operation, element);
            operation.create();
            # println("> Operation created : ", element.name());
        } catch (NotCreatedException.NotCreatedException nce) {
            operation.destroy();
            throw nce;
        }
    }



    # =================================================================
    #   [1] - 3.13.2 - Operation Parameters Declaration
    # =================================================================
    proc setOperationParameters (self, operation, element)
    {
        parameterList = operation.getParameterList();
        for parameter in element.parameter() {
            if (self.umlOperations.isStereotyped(parameter, "CORBAAnonymousSequence"))
                typeParameter = self.getCORBAAnonymousSequence(parameter, currentScope)
            else
                typeParameter = self.typeOfObject(parameter);
            if (parameter.kind() == UML.DataTypes.ParameterDirectionKind.pdk_return) {
                operation.setType(typeParameter);
            } else if (parameter.kind() == UML.DataTypes.ParameterDirectionKind.pdk_in) {
                parameterList.addInParam(parameter.name(), typeParameter);
            } else if (parameter.kind() == UML.DataTypes.ParameterDirectionKind.pdk_inout) {
                parameterList.addInOutParam(parameter.name(), typeParameter);
            } else if (parameter.kind() == UML.DataTypes.ParameterDirectionKind.pdk_out) {
                parameterList.addOutParam(parameter.name(), typeParameter);
            }
        }
    }


    # =================================================================
    #   [1] - 3.13.3 - Operation Raises Expressions
    # =================================================================
    proc setOperationExceptions (self, operation, element)
    {
        AContextRaisedSignalLinkSet = self.umlPackage.common_behavior_ref.a_context_raised_signal_ref.all_a_context_raised_signal_Links();
        for crsl in AContextRaisedSignalLinkSet {
            if (crsl.context._is_equivalent(element)) {
                operation.getExceptionList().add(self.getType(crsl.raised_signal));
            }
        }
    }


    # =================================================================
    #   [1] - 3.13.4 - Operation Context Expressions
    # =================================================================
    proc setOperationContext (self, operation, element)
    {
        # Not present in the XMI file.
        # operation.getContextList().add(...);
    }


    # =================================================================
    #   [1] - 3.14 - Attribute Declaration
    # =================================================================
    proc createAttribute (self, element, currentScope, isValueMember=false)
    {
        if (isValueMember) {
            attrib = currentScope.startValueMember(element.name());
            if (element.visibility() == UML.DataTypes.VisibilityKind.vk_private)
                attrib.setPrivate()
            else
                attrib.setPublic();
        } else {
            attrib = currentScope.startAttribute(element.name());
        }
        if (self.umlOperations.isStereotyped(element, "readonly"))
            attrib.setReadonly();
        if (self.umlOperations.isStereotyped(element, "CORBAAnonymousSequence"))
            attrib.setType(self.getCORBAAnonymousSequence(attrib, currentScope))
        else
            attrib.setType(self.typeOfObject(element));
        attrib.create();
        # println("> Attribute created : ", element.name());
    }


    # =================================================================
    #   [1] - 3.14 - Attribute Declaration (by Association)
    # =================================================================
    proc createAssociationAttribute (self, element, currentScope, isValueMember=false)
    {
        for asso in self.umlOperations.associations(element) {
            isAssociationAttribute = false;
            readonly = false;
            anonymousSequence = false;
            try {
                if (asso.stereotype().length == 0)
                    isAssociationAttribute = true;
                if (self.umlOperations.isStereotyped(asso, "readonly"))
                    readonly = true;
                if (self.umlOperations.isStereotyped(asso, "CORBAAnonymousSequence"))
                    anonymousSequence = true;
            } catch (Reflective.NotSet notset) {
                isAssociationAttribute = true;
            }
            if (isAssociationAttribute) {
                if (asso.connection()[0].participant() == element) {
                    name = asso.connection()[1].name();
                    type = asso.connection()[1].participant();
                    visibility = asso.connection()[1].visibility();
                } else {
                    name = asso.connection()[0].name();
                    type = asso.connection()[0].participant();
                    visibility = asso.connection()[0].visibility();
                }
                if (isValueMember) {
                    attrib = currentScope.startValueMember(name);
                    if (visibility == UML.DataTypes.VisibilityKind.vk_private)
                        attrib.setPrivate()
                    else
                        attrib.setPublic();
                } else {
                    attrib = currentScope.startAttribute(name);
                    if (readonly) attrib.setReadonly();
                }
                if (anonymousSequence)
                    attrib.setType(self.getCORBAAnonymousSequence(type, currentScope))
                else
                    attrib.setType(self.getType(type));
                attrib.create();
                # println("> Attribute created : ", element.name());
            }
        }
    }


    # =================================================================
    #   [2] - 4.1.2.1 - Component
    # =================================================================
    proc createCORBAComponent (self, element, currentScope)
    {
        try {
            compo = currentScope.declareComponent(element.name());
            compo.create();
            for g in element.generalization() {
                if (self.umlOperations.isStereotyped(g.parent(), "CORBAInterface")) {
                    compo.getSupportedInterfaceList().add(g.parent());
                } else if (self.umlOperations.isStereotyped(g.parent(), "CORBAComponent")) {
                    compo.setBaseComponent(self.getType(g.parent()));
                }
            }
            self.create(element.feature(), compo);
            compo.create();
            self.putType(element, compo);
            # println("> Component created : ", element.name());
        } catch (NotCreatedException.NotCreatedException nce) {
            compo.destroy();
            throw nce;
        }
    }


    # =================================================================
    #   [2] - 4.1.2.2 - Facet and Receptacle - CORBAProvides
    # =================================================================
    proc createCORBAProvides (self, element, currentScope)
    {
        connect = element.connection();
        interface = "notset";
        compo = "notset";
        name = "";
        if (self.umlOperations.isStereotyped(connect[0].participant(), "CORBAInterface")
            || self.umlOperations.isStereotyped(connect[0].participant(), "CORBAHome")) {
            interface = self.getType(connect[0].participant());
            compo = self.getType(connect[1].participant());
            name = connect[0].name();
        } else {
            interface = self.getType(connect[1].participant());
            compo = self.getType(connect[0].participant());
            name = connect[1].name();
        }
        provides = compo.startProvides(name);
        provides.setInterface(interface);
        provides.create();
        # println("> Provides created : ", name);
    }


    # =================================================================
    #   [2] - 4.1.2.2 - Facet and Receptacle - CORBAUses
    # =================================================================
    proc createCORBAUses (self, element, currentScope)
    {
        connect = element.connection();
        interface = "notset";
        compo = "notset";
        name = "";
        if (self.umlOperations.isStereotyped(connect[0].participant(), "CORBAInterface")
            || self.umlOperations.isStereotyped(connect[0].participant(), "CORBAHome")) {
            interface = self.getType(connect[0].participant());
            compo = self.getType(connect[1].participant());
            name = connect[0].name();
        } else {
            interface = self.getType(connect[1].participant());
            compo = self.getType(connect[0].participant());
            name = connect[1].name();
        }
        uses = compo.startUses(name);
        multiple = "no";
        for tv in element.tagged_value() {
            if (tv.type().tag_type() == "CORBAIsMultiple") {
                multiple = tv.data_value();
            }
        }
        if (multiple == "yes") {
            uses.setMultiple();
        }
        uses.setInterface(interface);
        uses.create();
        # println("> Uses created : ", name);
    }


    # =================================================================
    #   [2] - 4.1.2.3 - Event
    # =================================================================
    proc createCORBAEvent (self, element, currentScope)
    {
        try {
            event = currentScope.declareEvent(element.name());
            if (element.is_abstract())
                event.setAbstract(true);
            # Not present in the UML Profile for CORBA.
            #if (element.is_custom())
            #    event.setCustom(true);
            event.create();
            features = element.feature();
            for feature in features {
                self.createAttribute(feature, event, true);
            }
            self.createAssociationAttribute(element, event, true);
            self.putType(element, event);
            # println("> Event created : ", element.name());
        } catch (NotCreatedException.NotCreatedException nce) {
            event.destroy();
            throw nce;
        }
    }


    # =================================================================
    #   [2] - 4.1.2.3 - Event - CORBAConsumes
    # =================================================================
    proc createCORBAConsumes (self, element, currentScope)
    {
        connect = element.connection();
        event = "notset";
        compo = "notset";
        name = "";
        if (self.umlOperations.isStereotyped(connect[0].participant(), "CORBAEvent")) {
            event = self.getType(connect[0].participant());
            compo = self.getType(connect[1].participant());
            name = connect[0].name();
        } else {
            event = self.getType(connect[1].participant());
            compo = self.getType(connect[0].participant());
            name = connect[1].name();
        }
        consumes = compo.startConsumes(name);
        consumes.setEvent(event);
        consumes.create();
        # println("> Consumes created : ", name);
    }


    # =================================================================
    #   [2] - 4.1.2.3 - Event - CORBAEmits
    # =================================================================
    proc createCORBAEmits (self, element, currentScope)
    {
        connect = element.connection();
        event = "notset";
        compo = "notset";
        name = "";
        if (self.umlOperations.isStereotyped(connect[0].participant(), "CORBAEvent")) {
            event = self.getType(connect[0].participant());
            compo = self.getType(connect[1].participant());
            name = connect[0].name();
        } else {
            event = self.getType(connect[1].participant());
            compo = self.getType(connect[0].participant());
            name = connect[1].name();
        }
        emits = compo.startEmits(name);
        emits.setEvent(event);
        emits.create();
        # println("> Emits created : ", name);
    }


    # =================================================================
    #   [2] - 4.1.2.3 - Event - CORBAPublishes
    # =================================================================
    proc createCORBAPublishes (self, element, currentScope)
    {
        connect = element.connection();
        event = "notset";
        compo = "notset";
        name = "";
        if (self.umlOperations.isStereotyped(connect[0].participant(), "CORBAEvent")) {
            event = self.getType(connect[0].participant());
            compo = self.getType(connect[1].participant());
            name = connect[0].name();
        } else {
            event = self.getType(connect[1].participant());
            compo = self.getType(connect[0].participant());
            name = connect[1].name();
        }
        publishes = compo.startPublishes(name);
        publishes.setEvent(event);
        publishes.create();
        # println("> Publishes created : ", name);
    }


    # =================================================================
    #   [2] - 4.1.2.4 - Component Home
    # =================================================================
    proc createCORBAHome (self, element, currentScope)
    {
        try {
            primarykey = false;
            associations = self.umlOperations.associations(element);
            for asso in associations {
                if (self.umlOperations.isStereotyped(asso, "CORBAManages")) {
                    assoManages = asso;
                    if (self.umlOperations.isStereotyped(asso, "CORBAPrimaryKey")) {
                        primarykey = true;
                    }
                }
            }
            connect = assoManages.connection();
            if (self.umlOperations.isStereotyped(connect[0].participant(), "CORBAHome")) {
                managedComponent = connect[1].participant();
            } else {
                managedComponent = connect[0].participant();
            }
            home = currentScope.startHome(element.name());
            mc = self.getType(managedComponent);
            home.setManagedComponent(mc);
            for g in element.generalization() {
                if (self.umlOperations.isStereotyped(g.parent(), "CORBAHome")) {
                    home.setBaseHome(self.getType(g.parent()));
                } else if (self.umlOperations.isStereotyped(g.parent(), "CORBAInterface")) {
                    home.getSupportedInterfaceList().add(self.getTypes(g.parent()));
                }
            }
            if (primarykey) {
                try {
                    primary = self.getType(assoManages.name());
                } catch (exception) {
                    primary = currentScope.declareValue(assoManages.name());
                    if (assoManages.is_abstract())
                        primary.setAbstract(true);
                    self.createCORBAValueContents(primary, assoManages);
                    primaryKeyBase = Components.PrimaryKeyBase;
                    primary.getAbstractValueList().add(self.ast.lookupId("IDL:omg.org/Components/PrimaryKeyBase:1.0"));
                    primary.create();
                    self.putType(assoManages.name(), primary);
                }
                home.setPrimaryKey(primary);
            }
            home.create();
            self.create(element.feature(), home);
            home.create();
            self.putType(element, home);
            # println("> Home created : ", element.name());
        } catch (NotCreatedException.NotCreatedException nce) {
            home.destroy();
            throw nce;
        }
    }


    # =================================================================
    #   [2] - 4.1.2.4 - Component Home - CORBAFactory
    # =================================================================
    proc createCORBAFactory (self, element, currentScope)
    {
        try {
            homeFactory = currentScope.startFactory(element.name());
            homeFactoryParam = homeFactory.getParameterList();
            self.setOperationParameters(homeFactory, element);
            self.setOperationExceptions(homeFactory, element);
            homeFactory.create();
        } catch (NotCreatedException.NotCreatedException nce) {
            homeFactory.destroy();
            throw nce;
        }
    }


    # =================================================================
    #   [2] - 4.1.2.4 - Component Home - CORBAHomeFinder
    # =================================================================
    proc createCORBAFinder (self, element, currentScope)
    {
        try {
            homeFinder = currentScope.startFinder(element.name());
            self.setOperationParameters(homeFinder, element);
            self.setOperationExceptions(homeFinder, element);
            homeFinder.create();
        } catch (NotCreatedException.NotCreatedException nce) {
            homeFinder.destroy();
            throw nce;
        }
    }


}
