/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ---------------------------------------------

====================================================================*/

package org.objectweb.openccm.OpenCCM_DCI.cif;
import org.objectweb.openccm.OpenCCM_DCI.*;

/**
 * This is the implementation of the AssemblyInstanceManagement facet
 * provided by the AssemblyManagerModel component.
 *
 * This interface allows the management of all the AssemblyManager produce
 * by an AssemblyManagerModel.
 *
 * This class inherits from the skeleton
 * generated by the OpenCCM's CIF to Java mapping generator.
 *
 * @author <a href="mailto:Frederic.Briclet@lifl.fr">Briclet Frederic</A>
 */
public class AssemblyInstanceManagementSegImpl  
    extends org.objectweb.openccm.OpenCCM_DCI
            .AssemblyManagerModelSessionComposition
            .AssemblyInstanceManagementSeg   
{

        // ==================================================================
    //
    // Internal class.
    //
    // ==================================================================
    /**
     * This class implements the Iterator behavior on the AssemblyManager
     * Model connections. 
     */
    private class AssemblyManagerIterator        
    {
        //The current array of AssemblyManagerModelConnection
        private org.objectweb.openccm.OpenCCM_DCI.
            AssemblyManagerModelPackage.assembly_managersConnection []
            connect;
        //The current index on the array;
        private int index;

        /**
         * No parameter required only the DCI of the uppon class
         * is directly used.
         * At the creation a rewind is automaticaly done.
         */
        public AssemblyManagerIterator()
        { 
            rewind();
        }

        /**
         * Return the next AssemblyManagerModel
         * @return the next AssemblyManagerModel
         */
        public org.objectweb.openccm.OpenCCM_DCI.AssemblyManager
        next()
        throws java.util.NoSuchElementException
        {
            if(index==connect.length)
                throw new  java.util.NoSuchElementException();           
            index++;
            return connect[index-1].objref;
        }    

         /**
         * Return the current AssemblyManagerModel
         * @return the current AssemblyManagerModel
         */
        public org.objectweb.openccm.OpenCCM_DCI.AssemblyManager
        current()
        {            
            return connect[index-1].objref;
        }    

        public org.omg.Components.Cookie
        currentCookie()
        {
            return connect[index-1].ck;
        }    

        /**
         * Test the end of the iterator
         * @return the test result
         */
        public boolean
        hasNext()
        {
            return index<connect.length;
        }
        
        public void
        remove()
        {
            //Empty implementation
        }
        /**
         * Clear the current iterator, initializing it value with the
         * most recent connection.
         */
        public void
        rewind()
        {     
            //initialize to the first index
            index=0;            
            //initialize the array of model
            connect=AMM.get_connections_assembly_managers();
        }

        /**
         *Return the iterator length
         * @return the current iterator length
         */
        public int
        length()
        {
            return connect.length;
        }


    }

    // ==================================================================
    //
    // Internal state
    //
    // ==================================================================
    // The main segment
    private org.objectweb.openccm.OpenCCM_DCI.AssemblyManagerModelCCM AMM;
    // The descriptor to use
    private String xml_descriptor;
    // Iterator on the AssemblyManager
    private AssemblyManagerIterator amIterator;
    //Prefix for clarifying output print
    private String prefix="AssemblyInstanceManagement >> ";
    //Private boolean state -> to replace by state pattern
    private boolean removed;
    //Private table to retrieve a  cookie from a AssemblyManager instanceUUID;
    //  private java.util.Hashtable cookies;
    
    // ==================================================================
    //
    // internal methods.
    //
    // ==================================================================
    private AssemblyManager 
    getAssemblyManager(String instanceUUID)
    throws org.ist.coach.DCI.UnknownAssembly
    {
        for(amIterator.rewind();amIterator.hasNext();)
            if(amIterator.next().ass_id().equals(instanceUUID))
                return amIterator.current();
        throw new org.ist.coach.DCI.UnknownAssembly(); 
    }
    
    // ==================================================================
    //
    // public methods.
    //
    // ==================================================================
    /**
     * Public method for setting the XML code to process.
     * An AssemblyManagerIterator is also open for all the other
     * processing.
     */
    public void 
    setupXML(String xml_descriptor)
    {
        this.xml_descriptor=xml_descriptor;
        AMM=(org.objectweb.openccm
             .OpenCCM_DCI.AssemblyManagerModelCCM)get_context(); 
        amIterator= new AssemblyManagerIterator();        
    }
    
    public void 
    disconnectRemovedAssembly(org.ist.coach.DCI.AssemblyManager am)
    throws Exception
    {     
        //System.err.println(prefix+"AssemblyId:"+ass_id);
        //System.err.println(prefix+cookies.toString());
        //System.err.println(prefix+"Clef detenue"+cookies.containsKey(ass_id));
        amIterator.rewind();
        //System.err.println(prefix+"Nb connections:"+amIterator.length());
        
        for(amIterator.rewind();amIterator.hasNext();)
            if(am._is_equivalent(amIterator.next()))
                AMM.disconnect_assembly_managers
                    (amIterator.currentCookie());               
        
        
        //System.err.println(prefix+"Remove doner");
        //System.err.println(prefix+"Nb connections:"+amIterator.length());
        

                
    }

    // ==================================================================
    //
    // AssemblyInstanceManagement methods.
    //
    // ==================================================================
    public void
    uninstall()
    throws org.ist.coach.DCI.UninstallationFailure,
           org.ist.coach.DCI.UnknownAssembly,
           org.ist.coach.DCI.AssemblyUsed
    {
        if(removed)
            return ;
        try{
            removed=true;
            for(amIterator.rewind();
                amIterator.hasNext();)
                {
                    amIterator.next().remove();
                    AMM.disconnect_assembly_managers(amIterator.currentCookie());    
                }
            AMM.remove();           
        }
        catch(org.omg.Components.RemoveFailure rf)
            {
                System.err.println(prefix+"Error occured during removing of an AssemblyManager");
                System.err.println(prefix+rf.getMessage());
                throw new org.ist.coach.DCI.UninstallationFailure();
            }
        catch(org.omg.Components.ExceededConnectionLimit ecl)
            {
                System.err.println(prefix+"Error occured during disconnection of an AssemblyManager");
                System.err.println(prefix+ecl.getMessage());
                throw new org.ist.coach.DCI.UninstallationFailure();
            }
    }

    /**
     * This method allow to make an instance of AssemblyManagerModel
     * @return the assembly uuid
     */
    public String
    create_assembly_instance()
    throws org.ist.coach.DCI.CreationFailure
    {
        try{
            //create the AssemblyManager componentt
            org.omg.Components.Cookie ck=   
                AMM.get_connection_assembly_factory()
                .create_assembly_XML(xml_descriptor);
            //Retrieve the assembly manager reference
            org.objectweb.openccm.OpenCCM_DCI.AssemblyManager AM=        
                AMM.get_connection_assembly_factory()
                .lookup_assembly_manager(ck);
            //connect the assembly manager to the assembly manager model
            AMM.connect_assembly_managers(AM);
            // cookies.put(AM.ass_id(),AMM.connect_assembly_managers(AM));
            //subscribe the Model to the assembly
            AM.subscribe_assembly_manager_removed_publisher
                (AMM.get_consumer_assembly_manager_removed_consumer());            
            AM.configuration_complete();
            // build the assembly
            AM.provide_assembly().build();
            
            // return the assembly id
            return AM.ass_id();        
        }
        catch(Exception e){
            System.err.println("Cannot create an assembly instance");
            e.printStackTrace();
            
            throw new org.ist.coach.DCI.CreationFailure();
        }

    }
    
    /**
     * This method return all the id of the running 
     * assemblies.
     * @return the assemblies uuid.
     */
    public String[]
    get_assembly_instances()
    throws org.ist.coach.DCI.UnknownAssembly
    {  
        //Create the buffer list
        java.util.LinkedList ll = new java.util.LinkedList();
        //Fill the list
        for(amIterator.rewind();amIterator.hasNext();)
            ll.addLast(amIterator.current().ass_id());
        //return the result
        return (String[])ll.toArray(new String[ll.size()]);            
    }

    /**
     * Return the assembly manager whitch have the specified
     * instance_id give in parameter.
     * @return the binded assembly manager
     */
    public AssemblyManager
    get_assembly_manager(String instance_id)
    throws org.ist.coach.DCI.UnknownAssembly
    {
        return getAssemblyManager(instance_id);
    }

    /**
     * Destroy the specified instance_id
     * @param instance_id the instance_id of the AssemblyManager to destroy
     */
    public void
    destroy_assembly_instance(String instance_id)
    throws org.ist.coach.DCI.DestroyError
    {
        try{
            getAssemblyManager(instance_id).remove() ;
            AMM.disconnect_assembly_managers(amIterator.currentCookie());   
            return ;
        }
        catch(Exception e){
            throw new org.ist.coach.DCI.DestroyError();
        }
    }
}
