/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ---------------------------------------------

====================================================================*/

package org.objectweb.openccm.OpenCCM_DCI.cif;

import org.objectweb.openccm.OpenCCM_DCI.*;

/**
 * This is the implementation of the DCIDeployment facet provided by the
 * DCIManager component.
 * This class inherits from the skeleton
 * generated by the OpenCCM's CIF to Java mapping generator.
 *
 * @author <a href="mailto:Frederic.Briclet@lifl.fr">Briclet Frederic</A>
 */
public class DCIDeploymentSegImpl  
    extends org.objectweb.openccm
            .OpenCCM_DCI.DCIManagerSessionComposition
            .DCIDeploymentSeg    
{
    
    // ==================================================================
    //
    // Internal class.
    //
    // ==================================================================
    /**
     * This class implements the Iterator behavior on the AssemblyManager
     * Model connections. 
     */
    private class AssemblyManagerModelIterator        
    {
        //The current array of AssemblyManagerModelConnection
        private org.objectweb.openccm.OpenCCM_DCI.
            DCIManagerPackage.assembly_manager_modelsConnection []
            connect;
        //The current index on the array;
        private int index;

        /**
         * No parameter required only the DCI of the uppon class
         * is directly used.
         * At the creation a rewind is automaticaly done.
         */
        public AssemblyManagerModelIterator()
        { 
            rewind();
        }

        /**
         * Return the next AssemblyManagerModel
         * @return the next AssemblyManagerModel
         */
        public org.objectweb.openccm.OpenCCM_DCI.AssemblyManagerModel
        next()
        throws java.util.NoSuchElementException
        {
            if(index==connect.length)
                throw new  java.util.NoSuchElementException();            
            index++;
            return connect[index-1].objref;
        }    

         /**
         * Return the current AssemblyManagerModel
         * @return the current AssemblyManagerModel
         */
        public org.objectweb.openccm.OpenCCM_DCI.AssemblyManagerModel
        current()
        {
            return connect[index-1].objref;
        }    

        /**
         * Test the end of the iterator
         * @return the test result
         */
        public boolean
        hasNext()
        {
            return index<connect.length;
        }
        
        public void
        remove()
        {
            //Empty implementation
        }
        /**
         * Clear the current iterator, initializing it value with the
         * most recent connection.
         */
        public void
        rewind()
        {     
            //initialize to the first index
            index=0;            
            //initialize the array of model
            connect=DCI.get_connections_assembly_manager_models();
        }

        /**
         *Return the iterator length
         * @return the current iterator length
         */
        public int
        length()
        {
            return connect.length;
        }


    }
    
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    // String prefix for clarifying output message
    private String prefix="DCIDeploymentSeg >> ";
    // Iterator on the AssemblyManagerModel
    private AssemblyManagerModelIterator ammIterator;
    //The reference on the DCIManager component interface
    private org.objectweb.openccm.OpenCCM_DCI.DCIManagerCCM DCI;  

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    private org.objectweb.openccm.OpenCCM_DCI.AssemblyManagerModel
    getAssemblyManagerModel(String assemblyUUID)
    throws org.ist.coach.DCI.UnknownAssembly
    {        
        for(ammIterator.rewind();
            ammIterator.hasNext(); )    
            if(ammIterator.next().assemblyUUID().equals(assemblyUUID))
                //return the AssemblyManagerModel instance
                return ammIterator.current();

        throw new org.ist.coach.DCI.UnknownAssembly();
    }
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================




    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    public void setupDCI(){
        DCI=(org.objectweb.openccm.OpenCCM_DCI.DCIManagerCCM)get_context(); 
        ammIterator=new AssemblyManagerModelIterator();
    }
    

    
    /**
     * This methods allows to install a archive given in parameter
     * the archive to used.
     * @param archive The archive in a byte array format
     * @return The assembly uuid.
     */    
    public String
    install_with_archive(byte[] archive)
    throws org.omg.Components.Deployment.InstallationFailure,
           org.ist.coach.DCI.AlreadyInstalled,
           org.ist.coach.DCI.InvalidArchive
    {
        return "not_implemented";
    }
    
    /**
     * This methods allows to install a descriptor giving in parameter
     * the xml code to use.
     * @param descriptor The xml code to use.
     * @return The assembly uuid.
     */    
    public String
    install_with_descriptor(String descriptor)
    throws org.omg.Components.Deployment.InstallationFailure,
           org.ist.coach.DCI.AlreadyInstalled
    {
        //Test if the connection to AssemblyFactory
        if(DCI.get_connection_assembly_factory()==null)
            throw  new org.omg.Components.Deployment.InstallationFailure();
        
        try
            {
                //create an AssemblyManagerModel
                org.objectweb.openccm.OpenCCM_DCI.AssemblyManagerModel amm=
                    DCI.get_connection_assembly_manager_model_home()
                    .create_assembly_model(descriptor);        
                //connect the assemblyfactory to AssemblyManagerModel
                amm.connect_assembly_factory(DCI.get_connection_assembly_factory());
                //connect the AssemblyManagerModel to the DCI
                DCI.connect_assembly_manager_models(amm);
                //configure complete the AssemblyManagerModel
                amm.configuration_complete();
                //return the assembly uuid
                return amm.assemblyUUID();
            }
        catch(Exception e)
            {
                throw new org.omg.Components.Deployment.InstallationFailure();
            }
    }    
        
    /**
     * Update an installed archive.
     * @param old_assemblyUUID the archive to replace
     * @param archive The updating archive
     */
    public String
    update_with_archive(String old_assemblyUUID,
                        byte[] archive)
    throws org.ist.coach.DCI.UninstallationFailure,
           org.ist.coach.DCI.UnknownAssembly,
           org.omg.Components.Deployment.InstallationFailure,
           org.ist.coach.DCI.AlreadyInstalled,
           org.ist.coach.DCI.InvalidArchive
    {        
        return "not_implemented";
    }


    /**
     * Update an installed descriptor.
     * @param old_assemblyUUID the archive to replace
     * @param archive The updating archive
     */
    public String
    update_with_descriptor(String old_assemblyUUID,
                           String descriptor)
    throws org.ist.coach.DCI.UninstallationFailure,
           org.ist.coach.DCI.UnknownAssembly,
           org.omg.Components.Deployment.InstallationFailure,
           org.ist.coach.DCI.AlreadyInstalled
    {
        return "not_implemented";
    }


   
     /**
     * Uninstall a assembly model an destroy all the created instance
     * @param assemblyUUID the model to uninstall
     */
    public void
    uninstall(String assemblyUUID)
    throws org.ist.coach.DCI.UninstallationFailure,
           org.ist.coach.DCI.UnknownAssembly,
           org.ist.coach.DCI.AssemblyUsed
    {
        
        getAssemblyManagerModel(assemblyUUID)
            .provide_assembly_instance_management()
            .uninstall();
    }


    /**
     * This method create an assembly manager instance from
     * the specified AssemblyManagerModel.
     *
     * @param assemblyUUID the AssemblyManagerModel identifiant
     * @return instanceUUID of the AssemblyManager created
     */
    public String
    create_assembly_instance(String assemblyUUID)
    throws org.ist.coach.DCI.CreationFailure
    {
        try{
            // retrieve the right AssemblyManagerModel and create the instance
            return
                getAssemblyManagerModel(assemblyUUID)
                .provide_assembly_instance_management()
                .create_assembly_instance();
        }
        catch(Exception e){
            System.err.println(prefix+
                           "Problem occured during the creation of instance");
            System.err.println(prefix+e.getMessage());
            throw new  org.ist.coach.DCI.CreationFailure();
        }
    }


    /**
     * This method all the AssemblyManager which have been
     * created from the assemblyUUID model
     *
     * @param assemblyUUID the AssemblyManagerModel identifiant
     * @return All the AssemblyManager found
     */
    public String[]
    get_assembly_instances(String assemblyUUID)
    throws org.ist.coach.DCI.UnknownAssembly
    {
        //Retrieve all the connection with the AssemblyManagerModel
        return
            getAssemblyManagerModel(assemblyUUID)
            .provide_assembly_instance_management()
            .get_assembly_instances();
    }

    

    /**
     * This method return the AssemblyManager which have 
     * the same instance_id that specified in parameter
     * @param instance_id the instance_id required
     * @return the AssemblyManager found.
     */
    public org.ist.coach.DCI.AssemblyManager
    get_assembly_manager(String instance_id)
    throws org.ist.coach.DCI.UnknownAssembly
    {
        //Browse all the model to find the good AssemblyManager
        for(ammIterator.rewind();
            ammIterator.hasNext();)
            if(ammIterator.next()
               .provide_assembly_instance_management()
               .get_assembly_manager(instance_id)!=null)
                return ammIterator.current()
                    .provide_assembly_instance_management()
                    .get_assembly_manager(instance_id);
        //The instance_id was not found due to a bas specified instance_id
        throw new  org.ist.coach.DCI.UnknownAssembly();
        
    }

     /**
     * This method destroy the AssemblyManager which have 
     * the same instance_id that specified in parameter
     * @param instance_id the instance_id required
     */
    public void
    destroy_assembly_instance(String instance_id)
    throws org.ist.coach.DCI.DestroyError
    {        
        //Browse all the model to find the good AssemblyManager
        for(ammIterator.rewind(); ammIterator.hasNext();)
            {
                try{
                    ammIterator.next()       
                        .provide_assembly_instance_management()
                        .get_assembly_manager(instance_id);           
                    ammIterator.current()
                        .provide_assembly_instance_management()
                        .destroy_assembly_instance(instance_id);
                    return ;
                }
                catch(org.ist.coach.DCI.UnknownAssembly ua){}    
                    
            }        
        //The instance_id was not found due to a bas specified instance_id
        throw new org.ist.coach.DCI.DestroyError() ;
    }

    /**
     * This method return an  assemblyUUID array
     * of all the installed archive and descriptor.    
     * @return all the installed assemblyUUID
     */
    public String[]
    get_assemblies()
    {
        java.util.LinkedList ll=new java.util.LinkedList();
        
        //Fill the array        
        for(ammIterator.rewind();
            ammIterator.hasNext();)
            ll.addLast(ammIterator.next().assemblyUUID());
        //return the array
        return (String[])ll.toArray(new String[ll.size()]);
    }

    /**
     * Return the XML code of the assemblyUUID.
     * @param assemblyUUID The uuid to use
     * @return The xml code found.
     */
    public String
    lookup_assembly(String assemblyUUID)
    throws org.ist.coach.DCI.UnknownAssembly{
        return getAssemblyManagerModel(assemblyUUID)
            .xml_descriptor();
    }

}
