/*===========================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Sylvain Leblanc.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.openccm.browser.CosTrading;

/** The administration console's API imports */
import org.objectweb.util.browser.api.MenuItem;
import org.objectweb.util.browser.api.MenuItemTreeView;
import org.objectweb.util.browser.api.TreeView;

/** The java API's imports */
import javax.swing.JOptionPane;
import java.awt.Component;

/** The OMG CosTrading imports. */
import org.omg.CosTrading.Link;
import org.omg.CosTrading.FollowOption;

import org.objectweb.openccm.browser.CosTrading.gui.*;

/**
 * This action adds a new link between two traders.
 *
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 * @version 0.1
 */
public class AddLink implements MenuItem {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Used to treat an error while evaluating the query. Common
     * action is to display an error message.
     *
     * @param message The error message to display.
     */
    protected void fatalError(String message) {
        JOptionPane.showMessageDialog(null, message, "Error on add", JOptionPane.ERROR_MESSAGE);
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    public int getStatus(TreeView arg0){
        return MenuItem.ENABLED_STATUS;
    }

    public void actionPerformed(MenuItemTreeView e) {

        Link link_ref = (Link) e.getSelectedObject();
        AddLinkPanel lp = new AddLinkPanel();

        int result =
            JOptionPane.showOptionDialog(
                (Component) e.getActionEvent().getSource(),
                lp,
                "Create a new link",
                JOptionPane.OK_CANCEL_OPTION,
                JOptionPane.PLAIN_MESSAGE,
                null,
                null,
                null);

        if (result == 0) {
            String name = lp.getName();
            org.omg.CosTrading.Lookup look_ref = lp.getLookupRef();
            org.omg.CosTrading.FollowOption def_pass_on = lp.getDefPassOn();
            org.omg.CosTrading.FollowOption limiting = lp.getLimiting();
            
            if (name != null && name.length() > 0 )
                try {
                    link_ref.add_link(name, look_ref, def_pass_on, limiting);
                } catch (org.omg.CosTrading.LinkPackage.IllegalLinkName ex) {
                    fatalError("Invalid name !");
                } catch (org.omg.CosTrading.LinkPackage.DuplicateLinkName ex) {
                    fatalError("Invalid name !");
                } catch (org.omg.CosTrading.InvalidLookupRef ex) {
                    fatalError("Invalid ref !");
                } catch (org.omg.CosTrading.LinkPackage.DefaultFollowTooPermissive ex) {
                    fatalError("Default follow rule too permissive !");
                } catch (org.omg.CosTrading.LinkPackage.LimitingFollowTooPermissive ex) {
                    fatalError("Limiting follow rule too permissive !");
                }
            else
                fatalError("Invalid parameters !");
        }
    }
}
