/*===========================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Sylvain Leblanc.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.openccm.browser.CosTrading;

/** The administration console's API imports */
import org.objectweb.util.browser.api.Context;
import org.objectweb.util.browser.api.Wrapper;
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.core.naming.DefaultName;
import org.objectweb.util.browser.core.naming.DefaultEntry;

/** The OMG Trading Service imports. */
import org.omg.CosTrading.RegisterPackage.OfferInfo;

/**
 * Basic implementation of the Context interface for managing
 * org.omg.CosTrading.RegisterPackage.OfferInfo classes.  Object of
 * this type are obtained by the <code>list_offer</code> method of the
 * <code>org.omg.CosTrading.Admin</code> interface.
 *
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 * @version 0.1
 */
public class OfferInfoContext 
  implements Context, Wrapper
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The OfferInfo object. */
    private OfferInfo offerInfo_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for interface Context.
    //
    // ==================================================================

    public Entry[] getEntries() {
        Entry[] values = new Entry[1];
        values[0] = new DefaultEntry(offerInfo_.reference, new DefaultName("Related Service"), this);
        return values;
    }

    // ==================================================================
    //
    // Additional public methods for Wrapper interface.
    //
    // ==================================================================

    /**
     * Fix the wrapped object
     *
     * @param object The object to wrap
     *
     */
    public void setWrapped(Object object) {
        offerInfo_ = (OfferInfo) object;
    }

    /**
     * Gets the wrapped object
     *
     * @return The wrapped object
     *
     */
    public Object getWrapped() {
        return offerInfo_;
    }

}
