/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.browser.Deployment;

import java.awt.Component;
import javax.swing.JOptionPane;
import java.io.File;
import java.io.BufferedReader;
import java.io.FileReader;
import javax.swing.Box;

import org.objectweb.openccm.browser.menu.OpenCCMBrowserConstants;
import org.objectweb.util.browser.api.MenuItemTreeView;
import org.objectweb.util.browser.api.MenuItem;
import org.objectweb.util.browser.api.TreeView;
import org.objectweb.util.browser.gui.api.DialogAction;
import org.objectweb.util.browser.gui.api.DialogBox;
import org.objectweb.util.browser.gui.lib.DefaultDialogBox;
import org.objectweb.util.browser.gui.lib.LabelBox;

import org.omg.Components.Deployment.Container;

/**
 * Installs a new home from a Container.
 * You have to specify : 
 * <ul>
 * <li>The UUID id.</li>
 * <li>The entry point.</li>
 * </ul>
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class InstallHome 
  implements MenuItem, DialogAction {

    /** The box to specify id */
    protected LabelBox id_;

    /** The box to specify entry point */
    protected LabelBox entryPoint_;

    protected Container container_;

    /**
     * Create a box containing all the box to specify all the params
     */
    protected void createBox(DialogBox dialogBox) {
        id_ = new LabelBox("UUID");
        dialogBox.addElementBox(id_);
        entryPoint_ = new LabelBox("Entry Point");
        dialogBox.addElementBox(entryPoint_);
    }

    public int getStatus(TreeView arg0){
        return MenuItem.ENABLED_STATUS;
    }

    public void actionPerformed(MenuItemTreeView e) throws Exception {
        container_ = (Container) e.getSelectedObject();
        DialogBox dia = new DefaultDialogBox("Install a home");      
        createBox(dia);
        dia.setValidateAction(this);
        dia.setValidateLabel("Install");
        dia.setCancelLabel("Cancel");
        dia.show();
    }

    /**
     * Executes an action
     */
    public void executeAction() throws Exception {
        String UUID = id_.getLabel();
        String entryPt = entryPoint_.getLabel();
        org.omg.Components.ConfigValue[] config = new org.omg.Components.ConfigValue[0];
        container_.install_home(UUID, entryPt, config);
    }

}




