/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.browser;

/** The administration console's API*/
import org.objectweb.util.browser.api.Tree;
import org.objectweb.util.browser.core.api.ContextContainer;
import org.objectweb.util.browser.core.api.TreeConfiguration;
import org.objectweb.util.browser.core.naming.DefaultContextContainer;
import org.objectweb.util.browser.core.common.DefaultTreePanel;
import org.objectweb.util.browser.core.common.DefaultViewPanel;
import org.objectweb.util.browser.core.common.DynamicTree;

import org.objectweb.openccm.browser.menu.Menu;
import org.objectweb.openccm.browser.menu.TreeDialogSingleton;

/** The java API's imports */
import java.io.File;
import java.awt.Dimension;
import java.awt.BorderLayout;
import javax.swing.JFrame;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.BorderFactory;

/**
 * This class defines the main frame
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class Main extends JFrame {

    /**
     * Populates the tree
     */
    protected void populateTree(Tree tree) {

        // Adds the name service into the initial context
        try {
            org.omg.CORBA.Object o = org.objectweb.ccm.CORBA.TheORB.getORB().resolve_initial_references("NameService");
            tree.addEntry("NameService", o, 2);
        } catch (org.omg.CORBA.ORBPackage.InvalidName e1) {
            // NS is not started
        } catch (org.omg.CORBA.TRANSIENT e1) {
	        // Ignore (throws when the NameService is not started)
	    }

        // Adds the interface repository into the initial context
        try {
            org.omg.CORBA.Object o = org.objectweb.ccm.CORBA.TheORB.getORB().resolve_initial_references("InterfaceRepository");
            tree.addEntry("InterfaceRepository", o);
        } catch (org.omg.CORBA.ORBPackage.InvalidName e1) {
            // IR is not started
        }

        // Adds the trading service into the initial context
        try {
            org.omg.CORBA.Object o = org.objectweb.ccm.CORBA.TheORB.getORB().resolve_initial_references("TradingService");
            tree.addEntry("TradingService", o);
        } catch (org.omg.CORBA.ORBPackage.InvalidName e1) {
            // Trader is not started
        } catch (org.omg.CORBA.TRANSIENT e1) {
            // Ignore (throws when the TradingService is not started)
        }

    }

    protected void initializeTreeDialog(TreeConfiguration treeConfiguration) {
        DynamicTree tree = new DynamicTree();
        tree.setNewBrowserProperty(treeConfiguration.getBrowserProperty());
        tree.setNewContextProperty(treeConfiguration.getContextProperty());

        // Adds the name service into the initial context
        try {
            org.omg.CORBA.Object o = org.objectweb.ccm.CORBA.TheORB.getORB().resolve_initial_references("NameService");
            tree.addEntry("NameService", o, 2);
        } catch (org.omg.CORBA.ORBPackage.InvalidName e1) {
            // Trader is not started
        } catch (org.omg.CORBA.TRANSIENT e1) {
            // Ignore (throws when the TradingService is not started)
        }

        // Adds the trading service into the initial context
        try {
            org.omg.CORBA.Object o = org.objectweb.ccm.CORBA.TheORB.getORB().resolve_initial_references("TradingService");
            tree.addEntry("TradingService", o);
        } catch (org.omg.CORBA.ORBPackage.InvalidName e1) {
            // NS is not strated
        } catch (org.omg.CORBA.TRANSIENT e1) {
	        // Ignore (throws when the NameService is not started)
	    }

        TreeDialogSingleton.setInstance(tree);
    }

    /**
     * Default constructor
     */
    public Main(String contextPropertyFile, String[] browserPropertyFile) {

        super("OpenCCM Browser"); 

        DefaultViewPanel viewPanel = new DefaultViewPanel();

        DynamicTree tree = new DynamicTree();
        
        tree.setNewContextProperty(contextPropertyFile);
        tree.setNewBrowserProperty(browserPropertyFile);
        populateTree((Tree)tree);
        tree.setTargetPanel(viewPanel);
        
        // Creates the StatusBar
        JTextField statusBar = new JTextField();
        statusBar.setBorder(BorderFactory.createLoweredBevelBorder());
        statusBar.setEditable(false);
        tree.setStatusField(statusBar);

        // Initializes the tree using in JDialog Boxes,
        initializeTreeDialog((TreeConfiguration) tree);

        // Creates the menu
        Menu menu = new Menu(tree);
        setJMenuBar(menu.getMenuBar());

        getContentPane().setLayout(new BorderLayout());
        //		getContentPane().add(menu.getToolBar(),BorderLayout.NORTH);
        getContentPane().add(new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, true, new DefaultTreePanel(tree), viewPanel), BorderLayout.CENTER);
        getContentPane().add(statusBar,BorderLayout.SOUTH);

        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        pack();

        // Centers the frame
        Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setLocation((screenSize.width - getWidth()) / 2, (screenSize.height - getHeight()) / 2);

        setVisible(true);

    }

}
