/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.browser.menu;

/** The administration console's imports */
import org.objectweb.util.browser.core.common.DynamicTree;
import org.objectweb.util.browser.gui.api.DialogAction;
import org.objectweb.util.browser.gui.api.DialogBox;
import org.objectweb.util.browser.gui.lib.DefaultDialogBox;
import org.objectweb.util.browser.gui.lib.LabelBox;
import org.objectweb.util.browser.gui.lib.FileChooserBox;
import org.objectweb.util.browser.gui.lib.TreeBox;


/** The java API's imports */
import javax.swing.JOptionPane;
import java.io.File;
import java.awt.Dimension;
import java.awt.Component;
import javax.swing.ImageIcon;
import java.io.BufferedReader;
import java.io.FileReader;

/** The CORBA's imports */
import org.omg.CORBA.ORB;

/**
 * This class represents the action which loads an IOR file into to initial context.
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class LoadIORFileAction 
    extends AbstractMenuAction 
    implements DialogAction {

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    /** The name of the object */
    protected LabelBox name_ = null;

    /** The IOR file to use to bind the object */
    protected FileChooserBox iorFile_ = null;

    /** The tree used to select the object to bind */
    protected TreeBox treeBox_ = null;

    //==================================================================
    //
    // Constructors.
    //
    //==================================================================

    /**
     * The default constructor
     */
    public LoadIORFileAction(String nom, ImageIcon image, String desc, Integer mnemonic, DynamicTree tree, Component parent) {
        super(nom, image, desc, mnemonic, tree);
    }

    //==================================================================
    //
    // Internal methods.
    //
    //==================================================================

    /**
     * Creates a box containing all the box to specify all the params
     */
    protected void createBox(DialogBox dialogBox) {
        name_ = new LabelBox("Object's name");
        dialogBox.addElementBox(name_);
        iorFile_ = new FileChooserBox("IOR file",JFileChooserSingleton.getInstance(OpenCCMBrowserConstants.IOR_FILE_CHOOSER),false);        
        dialogBox.addElementBox(iorFile_);
        treeBox_ = new TreeBox(TreeDialogSingleton.getInstance(),false);  
        treeBox_.setPreferredSize(new Dimension(350, 150));
        dialogBox.addElementBox(treeBox_);
    }

    //==================================================================
    //
    // Public methods.
    //
    //==================================================================

    /**
     * Shows a JFileChooser component and tries to load the new configuration file
     */
    public void actionPerformed(java.awt.event.ActionEvent ae) {
        DialogBox dialogBox = new DefaultDialogBox("Adds a CORBA object");      
        createBox(dialogBox);
        dialogBox.setValidateAction(this);
        dialogBox.show();
    }
    
    //==================================================================
    //
    // Public methods for DialogAction interface.
    //
    //==================================================================

    /**
     * Executes an action
     */
    public void executeAction() throws Exception {
        File file = iorFile_.getFile();
        Object selectedObject = treeBox_.getObject();
        org.omg.CORBA.Object object = null;
        if (file != null) {
            String ior = null;
            try {
                BufferedReader in = new BufferedReader(new FileReader(file));
                ior = in.readLine();
            } catch (java.io.FileNotFoundException err) {
                System.out.println(file + " : File not found !");
            } catch (java.io.IOException err) {
                System.out.println(file + " : Error while reading !");
            }
            if (ior != null && ior.startsWith("IOR:")) {
                ORB orb = org.objectweb.ccm.CORBA.TheORB.getORB();
                object = orb.string_to_object(ior);
            }
        } else if (selectedObject != null) {
            try {
                object = (org.omg.CORBA.Object) selectedObject;
            } catch (ClassCastException e1) {
                throw new Exception("You must select a org.omg.CORBA.Object \n Found : " + selectedObject.getClass().getName());
            }
        }
        if (object != null)
            tree_.addEntry(name_.getLabel(), object);
        else
            throw new Exception("The object to bind is missing !");
    }

}
