/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.browser.menu;

/** The administration console's imports */
import org.objectweb.util.browser.core.common.DynamicTree;

/** The java API's imports */
import java.awt.event.KeyEvent;
import java.awt.Event;
import javax.swing.JPanel;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JToolBar;
import javax.swing.ImageIcon;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.KeyStroke;

/**
 * This class represents the menu,
 * it creates a JMenu and its associated JToolBar.
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class Menu extends JPanel {

    /** The menuBar */
    protected JMenuBar menuBar_;

    /** The menuToolBar */
    protected JToolBar toolBar_;

    /** The tree */
    protected DynamicTree tree_;

    /** Returns an ImageIcon, or null if the path is invalid. */
    protected static ImageIcon createIcon(String imageName) {
        String imgLocation = "icons/menu/" + imageName;

        java.io.File imageFile = new java.io.File(imgLocation);

        //java.net.URL imageURL = Menu.class.getResource(imgLocation);
        if (!imageFile.exists()) {
            // System.out.println("Resource not found: " + imgLocation);
            return null;
        } else {
            return new ImageIcon(imgLocation);
        }
    }

    protected void createFileMenu() {
        JMenu fileMenu = new JMenu("File");
        fileMenu.setMnemonic(KeyEvent.VK_F);

        JMenuItem menuItem = null;
        JButton button = null;

        // To load a new configuration file
        // Add to menu
        Action loadConfigurationFileAction =
            new LoadConfigurationFileAction("Load a configuration file", createIcon("fileopen.png"), "Load a new configuration file", new Integer(KeyEvent.VK_O), tree_, this);
        menuItem = new JMenuItem(loadConfigurationFileAction);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, Event.CTRL_MASK));
        menuItem.setIcon(null);
        fileMenu.add(menuItem);
        // Add to toolbar
        button = new JButton(loadConfigurationFileAction);
        if (button.getIcon() != null) {
            button.setText("");
        }
        toolBar_.add(button);

        // Add to menu
        // 		Action loadJarFileAction =
        // 			new LoadJarFileAction(
        // 				"Load a jar file",
        // 				createIcon("fileopen.png"),
        // 				"Load a new jar file",
        // 				new Integer(KeyEvent.VK_O),
        // 				tree_,
        // 				this);
        // 		menuItem = new JMenuItem(loadJarFileAction);
        // 		menuItem.setIcon(null);
        // 		fileMenu.add(menuItem);

        // To refresh the tree
        // Add to menu
        Action refreshAction = new RefreshAction("Refresh the tree", createIcon("reload.png"), "Refreshes the tree", new Integer(KeyEvent.VK_R), tree_);
        menuItem = new JMenuItem(refreshAction);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, Event.CTRL_MASK));
        menuItem.setIcon(null);
        fileMenu.add(menuItem);
        // Add to toolbar
        button = new JButton(refreshAction);
        if (button.getIcon() != null) {
            button.setText("");
        }
        toolBar_.add(button);

        // The quit button
        Action quitAction = new QuitAction("Quit", null, "Quit the application", new Integer(KeyEvent.VK_Q), tree_);
        menuItem = new JMenuItem(quitAction);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, Event.CTRL_MASK));
        menuItem.setIcon(null);
        fileMenu.add(menuItem);

        menuBar_.add(fileMenu);
    }

    protected void createTreeMenu() {
        JMenu treeMenu = new JMenu("Tree");
        treeMenu.setMnemonic(KeyEvent.VK_T);

        JMenuItem menuItem = null;
        JButton button = null;

        // To load a new IOR file
        // Add to menu
        Action loadIORFileAction = new LoadIORFileAction("Add a CORBA Object", null, "Load a new IOR file", new Integer(KeyEvent.VK_L), tree_, this);
        menuItem = new JMenuItem(loadIORFileAction);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, Event.CTRL_MASK));
        menuItem.setIcon(null);
        treeMenu.add(menuItem);

        menuBar_.add(treeMenu);
    }

    /**
     * Constructor
     */
    public Menu(DynamicTree tree) {
        tree_ = tree;
        menuBar_ = new JMenuBar();
        toolBar_ = new JToolBar();
        toolBar_.setFloatable(false);
        createFileMenu();
        createTreeMenu();
    }

    /**
     * Gets the GUI menu bar
     *
     * @return The associated JMenuBar
     */
    public JMenuBar getMenuBar() {
        return menuBar_;
    }

    /**
     * Gets the GUI tool bar
     *
     * @return The associated JToolBar
     */
    public JToolBar getToolBar() {
        return toolBar_;
    }

}
