/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Contreras
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.openccm.deployment.gui;

// Package dependencies
import org.objectweb.util.browser.api.ExtendedActionListener;
import org.objectweb.util.browser.api.ExtendedActionEvent;
import org.objectweb.ccm.descriptor.componentassembly.gui.ComponentassemblyZipper;

/**
 * This is the action class for building an archive for an assembly
 *
 */
public class AssemblyPackager
  implements ExtendedActionListener
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     *  The constructor taking an event as parameter
     */
    public void actionPerformed(ExtendedActionEvent e)
    {
        String action = e.getActionEvent().getActionCommand();

        // gets the calling instance
        org.objectweb.apollon.framework.Bean zeus_instance
            = (org.objectweb.apollon.framework.Bean)
                e.getTreeView().getValue();

        // gets the calling entry
        org.objectweb.util.browser.naming.api.Entry entry
         = e.getTreeView().getEntry();

        String descriptor_filename
         = entry.getOWName().getOWId();

        // displays a dialog box to select archive name
        java.io.File zipfile;
        javax.swing.JFileChooser dialogBox
         = new javax.swing.JFileChooser(System.getProperty("user.dir"));
        int returnVal = dialogBox.showSaveDialog(new javax.swing.JPanel());

        if (returnVal == javax.swing.JFileChooser.APPROVE_OPTION)
        {
            zipfile = dialogBox.getSelectedFile().getAbsoluteFile();

            // creates a new zip factory for this package
            ZipFactory zip_factory
            = new ZipFactory();

            zip_factory.setZipName(zipfile.getAbsolutePath());

            // marshals the descriptor to add it in the archive
            java.io.File descfile
             = new java.io.File(
                descriptor_filename
                );
            try
            {
                zeus_instance.marshalBean(descfile);
            } catch (Exception except) {
                except.printStackTrace();
            }

            zip_factory
                .addFileToZip(
                    descfile.getPath()
                    );

            // launches the ZIP extensions in the SOFTPKG framework
            ComponentassemblyZipper zipper
            = (ComponentassemblyZipper) zeus_instance
                .getExtensionManager()
                .getExtensionByName("gui.Zipper")
                .getInstance();
            zipper.zipCall(
                (org.objectweb.ccm.descriptor.componentassembly.beans.ComponentassemblyBean)
                    zeus_instance
                , zip_factory);

            // builds the archive after file retrieving
            try
            {
                zip_factory.buildZip();
            } catch (Exception exc) {
                System.err.println("Assembly archive could not be built. See following stacktrace");
                exc.printStackTrace();
            }
        }
    }
}
