/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.services.pss.jdo.lib;

import org.objectweb.openccm.services.pss.common.lib.PIDHelper;

/**
 * This class establish a session with a JDO implementation.
 * It implements the org.objectweb.openccm.services.pss.api.Connector interface.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class Session
     extends org.objectweb.openccm.services.pss.jdo.lib.CatalogBase
  implements org.objectweb.openccm.services.pss.jdo.api.Session
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     *
     * @param id - The catalog id.
     * @param pm - The associted JDO Persistence Manager.
     */
    public Session(int id, javax.jdo.PersistenceManager pm)
    {
        super(id, pm);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for org.omg.CosPersistentState.Catalog
    //
    // ==================================================================

    /**
     * Retrieve a storage object with its pid.
     */
    public java.lang.Object
    find_by_pid(byte[] the_pid)
    throws org.omg.CosPersistentState.NotFound
    {
        org.omg.CosPersistentState.TransactionalSession[] sessions = null;
        org.omg.CosPersistentState.CatalogBase catalog = null;
        org.omg.CosPersistentState.StorageHomeBase sh = null;

        // Get the object catalog
        // TO DO : add the transaction parameter to sessions() method
        sessions = get_connector().sessions(null);
        if (sessions != null)
        {
            org.objectweb.openccm.services.pss.common.api.CatalogBase cat = null;
            boolean found = false;
            int i = 0;

            while ((i<sessions.length) && (!found))
            {
                cat = (org.objectweb.openccm.services.pss.common.api.CatalogBase)sessions[i];
                if (cat.get_id() == PIDHelper.get_CatalogID(the_pid))
                    found = true;
                else
                    i++;
            }
            catalog = sessions[i];
        }
        else
        {
            // TO DO : Raise an exception here
            //         It should not happen !!
            catalog = this;
        }

        // Get the object storage home with its Repository ID
        sh = catalog.find_storage_home(PIDHelper.get_RID(the_pid));

        return sh.find_by_short_pid(PIDHelper.get_short_pid(the_pid));
    }

    /**
     * Operation flush
     */
    public void
    flush()
    {
        commit_tx();
    }

    /**
     * Operation refresh
     */
    public void
    refresh()
    {
        rollback_tx();
    }

    /**
     * Operation free_all
     */
    public void
    free_all()
    {
        // For each Storage Home, free registered storage types incarnations
        // TO DO
    }

    /**
     * Operation close
     */
    public void
    close()
    {
        flush();
        get_persistence_manager().close();
    }

    // ==================================================================
    //
    // Methods for org.objectweb.openccm.services.pss.common.api.CatalogBase
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for org.objectweb.openccm.services.pss.jdo.api.CatalogBase
    //
    // ==================================================================
}
