/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.naming.api;

/**
 * Standard interface for {@link Entry Entry} factories.
 *
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @version 0.1
 */

public interface EntryFactory {
	
    /**
     * The standard name of this interface to use
     * in the context of the Fractal framework.
     */
    static final String ENTRY_FACTORY = EntryFactory.class.getName();

    /**
     * Creates a new {@link Entry Entry} with a value, a name, and a context.
     *
     * The value, name, and context of the created {@link Entry Entry}
     * are set respectively to the given value, name, and context.
     *
     * @param value The value of the {@link Entry Entry} to create.
     * @param name The name of the {@link Entry Entry} to create.
     * @param context The context of the {@link Entry Entry} to create.
     *
     * @return The created {@link Entry Entry}.
     *
     * @postcondition
     *        return.getOWValue() == value
     *  and    return.getOWName() == name
     *  and return.getOWContext() == context;
     */
    Entry newOWEntry(Object value, Name name, Context context);
}
