/*===========================================================================

ObjectWeb Util API
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.naming.wrappers.common;

/**
 * Abstract base class.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @version 0.1
 */

abstract public class AbstractObject {
	
    // ==================================================================
    //
    // No internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // No constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // No internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for class Object.
    //
    // ==================================================================

    /**
     * Returns a string representation of the target entry.
     *
     * By default:
     * getClass().getName() + '[' + { name=value } + ']'
     *
     * @return A string representation of the target entry.
     */
    public String toString() {
        // The string buffer into the string representation is appended.
        StringBuffer sb = new StringBuffer();

        // Appends the full class name.
        sb.append(getClass().getName());
        sb.append('[');

        // Appends internal state to the string buffer.
        toStringBuffer(sb);

        sb.append(']');
        return sb.toString();
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Appends internal state representation into a given string buffer.
     *
     * This method does not update the target object.
     *
     * @param sb The string buffer where pairs <name,value> are appended.
     *
     * @postcondition isNotModified();
     */
    abstract public void toStringBuffer(StringBuffer sb);
}
