/*===========================================================================

ObjectWeb Util Launcher package.
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ___________________________________________.

===========================================================================*/

package org.objectweb.util.launcher;


/**
 * Parses a Java object wihch representes an XML Configuration.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 * @version 0.1
 */
public interface XMLDecoder
{
    /**
     * Parses the properties file.
     *
     * @param file The property file containing the configuration.
     *
     * @return The status (true if ok, else false).
     */
    public boolean
    parseFile(String file);

    /**
     * Set the run id and update variables.
     *
     * @param id - The run configuration to set.
     *
     * @return The status. True if ok, else false.
     */
    public boolean
    setRunId(String run_id);

    /**
     * Get the class to launch.
     *
     * @return The class name.
     */
    public String
    getMainClassName();

    /**
     * Get the url list.
     *
     * @return An array of urls.
     */
    public String[]
    getUrls();

    /**
     * Get the arguments list.
     *
     * @return An array of arguments.
     */
    public String[]
    getArguments();

    /**
     * Get properties.
     *
     * @return A Properties object containing all properties.
     */
    public java.util.Properties
    getProperties();
}
