/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Offroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.monolog.provider.lib;

import java.util.HashMap;

import org.objectweb.util.monolog.provider.api.TopicProvider;
import org.objectweb.util.monolog.provider.lib.topics.BundleTopics;
import org.objectweb.util.monolog.provider.lib.topics.TopicsMapSingleton;
import org.objectweb.util.monolog.provider.lib.topics.TopicsPool;

/**
 * @author Jerome Offroy
 *
 * used to create Topics and to maintain a pool of currently used Topics
 */
public class TopicProviderImpl implements TopicProvider {
	protected static TopicsPool tm_ = TopicsMapSingleton.getTopicsMap();

    /**
     * @see org.objectweb.util.monolog.provider.api.TopicProvider#createTopic(String, Class, String[])
     */
    public String createTopic(String name, Class ref, String[] description) {
        String realTopic = buildTopicNameBundle(ref) + BundleTopics.getPreTopic("TOPIC_SEPARATOR") + name;

        if (ref != null) {
            // add the new Topic and description to the structure 
            // which maintain topics declaration and description            
            tm_.put(ref,realTopic,description);
        }
        return realTopic;
    }

    /**
     * @see org.objectweb.util.monolog.provider.api.TopicProvider#getTopic(Object)
     */
    public String getTopic(Class ref) {
        String res = tm_.getName(ref);
        if (res == null) {
            res = "";
        }
        return res;
    }

    /**
     * @see org.objectweb.util.monolog.provider.api.TopicProvider#getTopics()
     */
    public String[] getTopics() {
        return tm_.getAllName();
    }

    /**
     * @see org.objectweb.util.monolog.provider.api.TopicProvider#getTopicDescription(String)
     */
    public String[] getTopicDescription(String name) {
        return tm_.getDescription(name);
    }

    /**
     * Method getTopicsByRef.
     * @return HashMap
     */
    public HashMap getTopicsByRef() {
        return tm_.getTopicsByRef();
    }

    /*************************************************
     * Private methods
     *************************************************/
    private String buildTopicNameBundle(Class ref) {
        String res = "default";
        res = BundleTopics.getPreTopic(ref.getPackage());
        return res;
    }
}
