/*====================================================================

ObjectWeb Util CommandLine Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.cmdline.lib;

// Package dependencies.
import org.objectweb.util.cmdline.api.Console;
import org.objectweb.util.cmdline.api.OptionSilentMessage;

/**
 * This is a default implementation of the
 * the command line silent message.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public class DefaultOptionSilentMessage
     extends DefaultOptionBase
  implements OptionSilentMessage
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The associated console. */
    Console console_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the associated console.
     *
     * @param console The associated console.
     */
    public
    DefaultOptionSilentMessage(Console console)
    {
        // Calls the DefaultOptionBase constructor.
        super(new String[] { "--silent"},
              new String[0],
              "Silent output messages");

	// Inits internal state.
        setConsole(console);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.Option
    //
    // ==================================================================

    /**
     * Consumes command line arguments from an iterator.
     *
     * @param iterator The command line argument iterator.
     *
     * @return true if the option has consumed arguments.
     */
    public void
    consume(org.objectweb.util.cmdline.api.Iterator iterator)
    {
        console_.setSilentMessage(true);
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cmdline.api.ConsoleHolder
    //
    // ==================================================================

    /**
     * Obtains the associated console.
     *
     * @return The associated console.
     */
    public Console
    getConsole()
    {
        return console_;
    }

    /**
     * Sets the associated console.
     *
     * @param console The associated console.
     */
    public void
    setConsole(Console console)
    {
        console_ = console;
    }

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.OptionSilentMessage
    //
    // ==================================================================

    // ==================================================================
    //
    // Other public methods.
    //
    // ==================================================================
}
