/*====================================================================

ObjectWeb Util Preprocessor Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.cpp.lib;

// Package dependencies.
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.io.BufferedReader;
import java.io.Reader;
import java.io.OutputStream;
import java.io.IOException;

import org.objectweb.util.misc.api.ExceptionWrapper;

/**
 * This provides some helper functions on Java IO streams.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public class IOHelper
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Static public methods.
    //
    // ==================================================================

    public static void
    close(InputStream input)
    {
        try
        {
            input.close();
        }
        // Wraps any java.io.IOException exception.
        catch(IOException exc)
        {
            throw new ExceptionWrapper(exc);
        }
    }

    public static void
    close(Reader input)
    {
        try
        {
            input.close();
        }
        // Wraps any java.io.IOException exception.
        catch(IOException exc)
        {
            throw new ExceptionWrapper(exc);
        }
    }

    public static void
    close(OutputStream output)
    {
        try
        {
            output.close();
        }
        // Wraps any java.io.IOException exception.
        catch(IOException exc)
        {
            throw new ExceptionWrapper(exc);
        }
    }

    public static void
    dump(Reader input,
         PrintStream output)
    {
        // Creates a buffered input reader stream.
        BufferedReader in = new BufferedReader(input);

        // Displays the content of the buffered input stream.
        try
        {
            String line = null;
            while( (line = in.readLine()) != null)
                output.println(line);
        }
        // Wraps any java.io.IOException exception.
        catch(IOException exc)
        {
            throw new ExceptionWrapper(exc);
        }
        finally
        {
            // Always closes the input stream.
            close(in);
        }
    }

    public static void
    dump(InputStream input,
         PrintStream output)
    {
        dump(new InputStreamReader(input), output);
    }
}
