/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): .

====================================================================*/

package org.objectweb.util.launcher ;


import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;


/**
 * Specialization of the Monolog trace mechanism into a TraceSystem
 * for dealing with trace in the scope of the Launcher.<BR>
 * <p>
 *   The TraceTemplate class provides implementation of the trace 
 *   interface based on Monolog implementation.
 * </p>
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.1
 */
public class TraceTemplate 
    implements Trace
{
    /** underlying Monolog logger reference*/
    protected Logger logger_ = null;

    /**
     * Default Construtor
     *
     * @param log - Monolog logger to use
     */
    public TraceTemplate(Logger log) { logger_ = log; }

    /**
     * Displays a debug message
     *
     * @param msg - debugging message to print
     */
    public void debug(String msg) { logger_.log(BasicLevel.DEBUG, msg); }

    /**
     * Displays a debug message
     *
     * @param msg - information message to print
     */
    public void info(String msg) { logger_.log(BasicLevel.INFO, msg); }

    /**
     * Displays a debug message
     *
     * @param msg - warning message to print
     */
    public void warn(String msg) { logger_.log(BasicLevel.WARN, msg); }

    /**
     * Displays a debug message
     *
     * @param msg - error message to print
     */
    public void error(String msg) { logger_.log(BasicLevel.ERROR, msg); }

    /**
     * Displays a debug message
     *
     * @param msg - fatal message to print
     */
    public void fatal(String msg) { logger_.log(BasicLevel.FATAL, msg); }
}
