/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): .

====================================================================*/

package org.objectweb.util.launcher.cmdline ;



import org.objectweb.util.cmdline.api.Option ;
import org.objectweb.util.cmdline.lib.ApplicationBase ;

import org.objectweb.util.launcher.CommandJava ;
import org.objectweb.util.launcher.CommandBuilder ;
import org.objectweb.util.launcher.CommandProvider ;
import org.objectweb.util.launcher.XBootClassLoader ;
import org.objectweb.util.launcher.TraceSystem ;
import org.objectweb.util.launcher.parser.XMLParserImpl ;
import org.objectweb.util.launcher.parser.Parser ;



/**
 * LauncherApplication describe the treaments to apply<BR>
 * <p>
 * this structure considers only characters list
 * </p>
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.1
 */
public class LauncherApplication
    extends ApplicationBase
{
    /**
     * Default Constructor
     */
    public LauncherApplication() {
        super(new LauncherCmdline(new XMLParserImpl()));
    }
    
    
    /**
     * Obtains the mailing list associated to this development.
     *
     * @return The mailing list associated to this development.
     */
    public String getMailingList() {
        return "architecture@objectweb.org";
    }
    
    /**
     * Complete the incoming CommandJava with the content of
     * the argument line
     *
     * @param cmd - the command to complete
     */
    protected void process() {
        Option[] opt = getCommandLine().getOptions();
        boolean default_run = true ;
        for (int i = 0 ; i < opt.length ; i++) {
            try {
                CommandBuilder builder = (CommandBuilder)opt[i];
                CommandJava[] cmd = builder.complete();
                default_run = !(cmd.length!=0 && default_run);
                for (int j=0 ; j < cmd.length ; j++)
                    cmd[j].run();
            } catch(ClassCastException ex) {}
        }        
        if (default_run) {
            LauncherCmdline cmdline = (LauncherCmdline) getCommandLine();            
            CommandProvider.get(cmdline.getParser(),"default").run();
        }
    }
    
    
    /**
     * Starts the application.
     *
     * @param args The command line arguments.
     *
     * @return The status.
     */
    public int start(java.lang.String[] args) {
        ((LauncherCmdline)getCommandLine()).getParser().load(args[0]);
        
        try {
            process();
        } catch(Exception ex) {
            TraceSystem.get("launcher").error("ERROR !!! "+ex);
            return -1;
        } finally {
            return 0;
        }
    }
}

