/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): .

====================================================================*/

package org.objectweb.util.launcher.parser ;


import java.util.Properties ;

import org.objectweb.util.launcher.StringList ;


/**
 * Representation of an option<BR>
 * <p>
 * An option describe additionnal informations on an application.
 * It contains:
 * <li>
 *   <ul>an identifier</ul>
 *   <ul>a list of arguments</ul>
 *   <ul>a list of properties</ul>
 *   <ul>a list of Java classes</ul>
 * </li>
 * </p>
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.1
 */
public class OptionDescription
{
    /** The name of the option */
    protected String name_ ;

    /** The arguments list */
    protected StringList arguments_;

    /** The properties list */
    protected Properties properties_;

    /** The class list */
    protected StringList classpath_ ;
    

    /**
     * Default Constructor
     */
    public OptionDescription() {
        this.arguments_ = new StringList();
        this.classpath_ = new StringList();
        this.properties_ = new Properties();
    }


    /**
     * Defines the identifer of the option
     *
     * @param name - identifier of the option
     */
    public void setName(String name) { name_ = name; }

    /**
     * Provides the identifier of the option
     *
     * @return identifier of the option
     */
    public String getName() { return name_; }


    /**
     * Adds an argument in the content of the option
     *
     * @param arg - argument to add
     */
    public void addArgument(String arg) { arguments_.add(arg); }
    
    /**
     * Provides the list of arguments for the option
     *
     * @return the list of arguments
     */
    public StringList getArguments() { return arguments_; }

    
    /**
     * Adds a property in the content of the option
     *
     * @param key - the key of the property
     * @param value - the value of the property
     */
    public void addProperty(String key, String value) { properties_.put(key,value); }
    
    /**
     * Provides the list of properties for the option
     *
     * @return the list of Java Properties
     */
    public Properties getProperties() { return properties_; }

    /**
     * Adds an element in the classpath of the option
     *
     * @param classname - the name of the class to add
     */
    public void addClass(String classname) { classpath_.add(classname); }

    /**
     * Provides the list of classes considered by the option
     *
     * @return the list of classes to load
     */
    public StringList getClasses() { return classpath_; }
}
