/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fr.lip6.src.modfact.generation.jmi;

import java.util.Enumeration;
import java.util.Vector;

import org.omg.CORBA.TCKind;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.TypeCodePackage.BadKind;
import org.omg.CORBA.TypeCodePackage.Bounds;
import org.omg.mof.Model.DataType;
import org.omg.mof.Model.DataTypeHelper;
import org.omg.mof.Model.ModelElement;
import org.omg.mof.Model.MofAttribute;
import org.omg.mof.Model.Operation;
import org.omg.mof.Model.ScopeKind;
import org.omg.mof.Reflective.MofError;

/**
 * 
 */
public class ClassProxyInterfaceTemplate extends CommonClassInterfaceTemplate {
	
	public void classProxyTemplate(org.omg.mof.Model.Class _class) throws MofError , BadKind , Bounds {
		//Tag todo
		
		out.println("public interface "+helper.jmiClassName(_class)+"Class extends javax.jmi.reflect.RefClass {");
		
		//If isAbstract is not set to true, generate factory methods.
		if (!_class.is_abstract()) {
			out.println("\tpublic "+helper.jmiClassName(_class)+" create"+helper.jmiClassName(_class)+"() throws javax.jmi.reflect.JmiException;\n");
			out.print("\tpublic "+helper.jmiClassName(_class)+" create"+helper.jmiClassName(_class)+"(");
			//for each non-derived direct or inherited attribute
			java.util.Vector attributs = helper.allAttributsForClasses(_class , ScopeKind.instance_level , false);
			for (Enumeration e = attributs.elements() ; e.hasMoreElements() ; ) {
				MofAttribute attribute = (MofAttribute) e.nextElement();
				out.print(helper.jmiClassifierName(attribute.type())+" "+helper.jmiAttributeName(attribute));
				if (e.hasMoreElements()) out.print(" , ");
				
			}
			out.println(") throws javax.jmi.reflect.JmiException;\n");
		}
		
		//for each StructType directly contained by the class (correction of the spec ?)
		//for each EnumType directly contained by the class (correction of the spec ?)
		ModelElement[] containedElements = _class.contents(); 
		for (int i=0 ; i < containedElements.length ; i++) {
			if (containedElements[i]._is_a(DataTypeHelper.id())) {
				DataType containedDataType = DataTypeHelper.narrow(containedElements[i]);
				TypeCode tc = containedDataType.type_code();
				if (tc.kind().equals(TCKind.tk_struct)) {
					out.print("\tpublic "+helper.jmiTypeCodeName(tc)+" create"+helper.jmiTypeCodeName(tc)+" (");
					for (int j=0 ; j < tc.member_count() ; j++) {
						out.print(helper.jmiTypeCodeName(tc.member_type(j)));//attribute type
						out.print(" ");
						out.print(tc.member_name(j));//attribute name
						if (j!=tc.member_count()) out.print(" , ");
					}
					out.println(") throws javax.jmi.reflect.JmiException;\n");
				}
				else if (tc.kind().equals(TCKind.tk_enum)) {
					out.println("\tpublic "+helper.jmiTypeCodeName(tc)+" create"+helper.jmiTypeCodeName(tc)+" (String value) throws javax.jmi.reflect.JmiException;\n");
				}
			}
		}
		
		//for each attribute and operation contained in this class or one of its
		//supertypes with "classifier-level" scope, generate the appropriate operations
		Vector attributes = helper.allAttributsForClasses(_class , ScopeKind.classifier_level, true);
		for (Enumeration e = attributes.elements() ; e.hasMoreElements() ; ) {
			MofAttribute attribute = (MofAttribute) e.nextElement();
			attributeTemplate(attribute);
			
		}
		
		Vector operations = helper.allOperationsForClasses( _class , ScopeKind.classifier_level);
		for (Enumeration e = operations.elements() ; e.hasMoreElements() ; ) {
			Operation operation = (Operation) e.nextElement();
			operationTemplate(operation);
		}
		
		out.println("}//end of interface");
		
	}

}
