/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fr.lip6.src.modfact.generation.jmi;

import java.util.Vector;

import org.omg.CORBA.TCKind;
import org.omg.mof.Model.Constant;
import org.omg.mof.Model.DataType;
import org.omg.mof.Model.DataTypeHelper;
import org.omg.mof.Model.DirectionKind;
import org.omg.mof.Model.ModelElement;
import org.omg.mof.Model.MofAttribute;
import org.omg.mof.Model.MofException;
import org.omg.mof.Model.MofExceptionHelper;
import org.omg.mof.Model.MultiplicityType;
import org.omg.mof.Model.Operation;
import org.omg.mof.Model.Parameter;
import org.omg.mof.Model.ParameterHelper;
import org.omg.mof.Model.Reference;
import org.omg.mof.Model.VisibilityKind;
import org.omg.mof.Reflective.MofError;

/**
 * 
 */
public class CommonClassInterfaceTemplate extends CommonInterfaceTemplate {

	public void attributeTemplate(MofAttribute attribute) throws MofError {
		if (attribute.visibility().equals(VisibilityKind.public_vis)) {
			//<<ANNOTATION TEMPLATE>>

			String attributeType = helper.jmiClassifierName(attribute.type());
			String attributeName = attribute.name();
			String accessorName = accessorName(attribute);
			//Accessor Operations
			//if optional-valued attribute or if single-valued attribute
			if (attribute.multiplicity().lower == 0
				|| attribute.multiplicity().upper == 1)
				out.println(
					"\tpublic "
						+ attributeType
						+ " "
						+ accessorName
						+ "() throws javax.jmi.reflect.JmiException;");
			//upper>1 and isOrdered = false
			if (attribute.multiplicity().upper > 1
				&& !attribute.multiplicity().isOrdered)
				out.println(
					"\tpublic Collection get"
						+ attributeName
						+ "() throws javax.jmi.reflect.JmiException;");
			//upper>1 and isOrdered = true
			if (attribute.multiplicity().upper > 1
				&& attribute.multiplicity().isOrdered)
				out.println(
					"\tpublic List get"
						+ attributeName
						+ "() throws javax.jmi.reflect.JmiException;");

			String mutatorName = mutatorName(attribute);
			//Mutator Operations
			//if optional-valued or if single-valued and isChangeable
			if ((attribute.multiplicity().lower == 0
				|| attribute.multiplicity().upper == 1)
				&& attribute.is_changeable()) {
				out.println(
					"\tpublic void "
						+ mutatorName
						+ "("
						+ attributeType
						+ " newValue) throws javax.jmi.reflect.JmiException;");
			}
		}
	}

	public void referenceTemplate(Reference reference) throws MofError {
		if (reference.visibility().equals(VisibilityKind.public_vis)) {
			//<<ANNOTATION TEMPLATE>>
			String referenceClass =
				helper.jmiClassifierName(reference.exposed_end().type());
			String referenceName = reference.name();
			//operation to return the Reference value
			//if upper = 1
			if (reference.multiplicity().upper == 1) {
				out.println(
					"\tpublic "
						+ referenceClass
						+ " get"
						+ referenceName
						+ "() throws javax.jmi.reflect.JmiException;");
				//if isChangeable
				if (reference.is_changeable()) {
					out.println(
						"\tpublic void set"
							+ referenceName
							+ "("
							+ referenceClass
							+ " newValue) throws javax.jmi.reflect.JmiException;");
				}
			}
			//if upper > 1 and isOrdered=false
			if (reference.multiplicity().upper > 1) {
				if (!reference.multiplicity().isOrdered) {
					out.println(
						"\tpublic Collection get"
							+ referenceName
							+ "() throws javax.jmi.reflect.JmiException;");
				} else
					out.println(
						"\tpublic List get"
							+ referenceName
							+ "() throws javax.jmi.reflect.JmiException;");
			}
		}
	}

	public void operationTemplate(Operation operation) throws MofError {
		if (operation.visibility().equals(VisibilityKind.public_vis)) {
			//<<JAVADOCS TEMPLATE>>
			//Get Parameters and exceptions
			Parameter[] parameters;
			MofException[] exceptions;
			Parameter returnParameter=null;
			
			Vector paramsTmp = new Vector();
			Vector exceptionTmp = new Vector();
			ModelElement[] contents = operation.contents();
			for (int i = 0; i < contents.length; i++) {
				if (contents[i]._is_a(ParameterHelper.id())) {
					Parameter parameter = ParameterHelper.narrow(contents[i]);
					paramsTmp.addElement(parameter);
				}
				else if (contents[i]._is_a(MofExceptionHelper.id())) {
					MofException ex = MofExceptionHelper.narrow(contents[i]);
					exceptionTmp.addElement(ex);
				}
			}
			parameters = new Parameter[paramsTmp.size()];
			for (int i = 0; i < parameters.length; i++) {
				parameters[i] = (Parameter) paramsTmp.elementAt(i);
				if (parameters[i].direction().equals(DirectionKind.return_dir)) returnParameter = parameters[i];
			}
			exceptions = new MofException[exceptionTmp.size()];
			for (int i = 0; i < exceptions.length; i++) {
				exceptions[i] = (MofException) exceptionTmp.elementAt(i);
			}

			if (returnParameter==null) out.print("\tpublic void ");
			else out.print("\tpublic "+helper.jmiClassifierName(returnParameter.type())+" ");
			out.print(operation.name()+"(");
			boolean virgule=false;
			for (int i=0 ; i <parameters.length ; i++) {
				if (virgule) out.print(",");
				if (parameters[i].direction().value()!=DirectionKind._return_dir) {
					out.print(helper.jmiClassifierName(parameters[i].type())+" "+parameters[i].name());
					virgule=true;
				}
			}
			out.print(") throws ");
			
			//for each exception
			for (int i=0 ; i < exceptions.length ; i++) {
				out.print(exceptions[i].name()+",");
			}
			out.println("javax.jmi.reflect.JmiException;");
		}
	}

	public void constantTemplate(Constant constant) throws MofError {
		//<<ANNOTATION TEMPLATE>>
		out.println(
			"\tpublic final "
				+ helper.jmiClassifierName(constant.type())
				+ " "
				+ constant.name()
				+ "="
				+ constant.value().extract_string()
				+ ";");
	}

	public String accessorName(MofAttribute attribute) throws MofError {
		String accessorName = attribute.name();
		MultiplicityType mult = attribute.multiplicity();
		//DataType
		boolean isBoolean = false;
		if (attribute.type()._is_a(DataTypeHelper.id())) {
			DataType type = DataTypeHelper.narrow(attribute.type());
			if (type.type_code().kind().equals(TCKind.tk_boolean))
				isBoolean = true;
		}
		if (mult.lower == 0 && mult.upper == 1 && isBoolean) {
			if (accessorName.matches("is*"))
				return accessorName;
			else
				return "is" + accessorName;
		} else
			return "get" + accessorName;
	}

	public String mutatorName(MofAttribute attribute) throws MofError {
		String mutatorName = "";
		MultiplicityType mult = attribute.multiplicity();
		//DataType
		boolean isBoolean = false;
		if (attribute.type()._is_a(DataTypeHelper.id())) {
			DataType type = DataTypeHelper.narrow(attribute.type());
			if (type.type_code().kind().equals(TCKind.tk_boolean))
				isBoolean = true;
		}
		if (isBoolean
			&& mult.lower == 0
			&& mult.upper == 1
			&& attribute.name().matches("is*")) {
			return "set" + attribute.name().substring(2);
		} else
			return "set" + attribute.name();

	}
}
