/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fr.lip6.src.modfact.generation.jmi;

import java.util.Vector;

import org.omg.CORBA.TCKind;
import org.omg.mof.Model.Association;
import org.omg.mof.Model.AssociationHelper;
import org.omg.mof.Model.ClassHelper;
import org.omg.mof.Model.DataType;
import org.omg.mof.Model.DataTypeHelper;
import org.omg.mof.Model.ModelElement;
import org.omg.mof.Model.Package;
import org.omg.mof.Model.PackageHelper;

import fr.lip6.src.modfact.util.Helper;

/**
 * 
 */
public class GenerationJMI1_0 {
	//Helper
	Helper helper;

	String directory = ".\\code\\";

	public void generateJMI1_0(Package _package)
		throws
			org.omg.mof.Reflective.MofError,
			org.omg.CORBA.TypeCodePackage.BadKind,
			org.omg.CORBA.TypeCodePackage.Bounds,
			org.omg.mof.Reflective.NotSet,
			java.io.FileNotFoundException {

		if (_package
			.visibility()
			.equals(org.omg.mof.Model.VisibilityKind.public_vis)) {

			if (helper == null) {
				helper = new Helper();
			}

			ModelElement[] contents = _package.contents();
			Vector packages = new Vector();
			Vector classes = new Vector();
			Vector associations = new Vector();
			Vector datatypes = new Vector();

			for (int i = 0; i < contents.length; i++) {
				if (contents[i]._is_a(ClassHelper.id())) {
					org.omg.mof.Model.Class _class = ClassHelper.narrow(contents[i]);
					classes.addElement(_class);

					//Claculer les contenus des classes pour les utilser aprs
					helper.allContentsOfClasses(_class);
				} else if (
					contents[i]._is_a(PackageHelper.id()))
					packages.addElement(
						PackageHelper.narrow(contents[i]));
				else if (
					contents[i]._is_a(
						AssociationHelper.id())) 
					associations.addElement(
						AssociationHelper.narrow(
							contents[i]));
				else if (
					contents[i]._is_a(DataTypeHelper.id()))
					datatypes.addElement(DataTypeHelper.narrow(contents[i]));
			}

			//Traitemement du Package
			String _file_name1 =
				directory
					+ helper.jmiPackageName(_package)+"Package"
					+ ".java";
			PackageInterfaceTemplate packageTemplate = new PackageInterfaceTemplate();
			packageTemplate.setOut(_file_name1);
			packageTemplate.setHelper(helper);
			packageTemplate.packageTemplate(_package);
			packageTemplate.closeFile();

			//Traitement des Classes
			for (int i = 0; i < classes.size(); i++) {
				org.omg.mof.Model.Class _class =
					(org.omg.mof.Model.Class) classes.elementAt(i);
				if (_class.visibility()
					== org.omg.mof.Model.VisibilityKind.public_vis) {

					// Cette Partie gnre toutes les ClasseProxy
					String _file_name2 =
						directory
							+ helper.jmiClassName(_class) + "Class"
							+ ".java";
					ClassProxyInterfaceTemplate classProxyTemplate = new ClassProxyInterfaceTemplate();
					classProxyTemplate.setOut(_file_name2);
					classProxyTemplate.setHelper(helper);
					classProxyTemplate.classProxyTemplate(_class);
					classProxyTemplate.closeFile();

					// Cette partie gnre toutes les Classes non abstraites
					if (!(_class.is_abstract())) {
						_file_name2 =
							directory
								+ helper.jmiClassName(_class)
								+ ".java";
						InstanceInterfaceTemplate classImpl =
							new InstanceInterfaceTemplate();
						classImpl.setOut(_file_name2);
						classImpl.setHelper(helper);
						classImpl.instanceTemplate(_class);
						classImpl.closeFile();
					}
				}
			}

			//Traitement des Associations
			for (int i = 0; i < associations.size(); i++) {
				Association _association =
					(Association) associations.elementAt(i);
				if (_association.visibility().equals(org.omg.mof.Model.VisibilityKind.public_vis)) {
					String _file_name =
						directory
							+ helper.jmiAssociationName(_association)
							+ ".java";
					AssociationInterfaceTemplate associationTemplate =
						new AssociationInterfaceTemplate();
					associationTemplate.setOut(_file_name);
					associationTemplate.setHelper(helper);
					associationTemplate.associationTemplate(_association);
					associationTemplate.closeFile();
				}
			}
			
			
			//Traitement des DataTypes Enum et Struct
			for (int i = 0; i < datatypes.size(); i++) {
				DataType datatype=
					(DataType) datatypes.elementAt(i);
				if (datatype.type_code().kind().equals(TCKind.tk_struct)) {
					String _file_name =
						directory
							+ datatype.name()
							+ ".java";
					StructureTypeInterfaceTemplate strutureTypeTemplate =
						new StructureTypeInterfaceTemplate();
					strutureTypeTemplate.setOut(_file_name);
					strutureTypeTemplate.setHelper(helper);
					strutureTypeTemplate.structureTypeTemplate(datatype);
					strutureTypeTemplate.closeFile();
				}
				else if (datatype.type_code().kind().equals(TCKind.tk_enum)) {
					String _file_name =
						directory
							+ datatype.name()
							+ "Enum.java";
					EnumerationTypeImplementationTemplate enumTypeTemplateImpl =
						new EnumerationTypeImplementationTemplate();
					enumTypeTemplateImpl.setOut(_file_name);
					enumTypeTemplateImpl.setHelper(helper);
					enumTypeTemplateImpl.enumerationTypeTemplate(datatype);
					enumTypeTemplateImpl.closeFile();
					
					_file_name =
						directory
							+ datatype.name()
							+ ".java";
					EnumerationTypeInterfaceTemplate enumTypeTemplate =
						new EnumerationTypeInterfaceTemplate();
					enumTypeTemplate.setOut(_file_name);
					enumTypeTemplate.setHelper(helper);
					enumTypeTemplate.enumerationTypeTemplate(datatype);
					enumTypeTemplate.closeFile();
				}
			}
			
			//RECURSIVITY
			//Go to others package
		}
	}

	/**
	 * Gets the directory
	 * @return Returns a String
	 */
	public String getDirectory() {
		return directory;
	}
	/**
	 * Sets the directory
	 * @param directory The directory to set
	 */
	public void setDirectory(String direct) {
		directory = direct;
	}

	/**
	 * Gets the helper
	 * @return Returns a Helper
	 */
	public Helper getHelper() {
		return helper;
	}
	/**
	 * Sets the helper
	 * @param helper The helper to set
	 */
	public void setHelper(Helper helper) {
		this.helper = helper;
	}
}
