/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fr.lip6.src.modfact.generation.jmi;

import java.util.Enumeration;
import java.util.Vector;

import org.omg.mof.Model.ClassHelper;
import org.omg.mof.Model.Constant;
import org.omg.mof.Model.GeneralizableElement;
import org.omg.mof.Model.ModelElementHelper;
import org.omg.mof.Model.MofAttribute;
import org.omg.mof.Model.Operation;
import org.omg.mof.Model.Reference;
import org.omg.mof.Model.ScopeKind;
import org.omg.mof.Model.VisibilityKind;
import org.omg.mof.Reflective.MofError;

/**
 * 
 */
public class InstanceInterfaceTemplate extends CommonClassInterfaceTemplate {
	
	public void instanceTemplate(org.omg.mof.Model.Class _class) throws MofError {
		javadocsTemplate(ModelElementHelper.narrow(_class));
		
		out.print("public interface "+helper.jmiClassName(_class));

		GeneralizableElement[] supertypes =  _class.supertypes();
		//if the class has no super-types
		if (supertypes.length==0) out.println("extends javax.jmi.reflect.RefObject {");
		//else, for each super-class
		else {
			out.print("extends ");
			for (int i=0 ; i<supertypes.length ; i++) {
				if (i!=0) out.print(" ,");
				out.print(helper.jmiClassName(ClassHelper.narrow(supertypes[i])));
			}
			out.print("{");
		}
		
		//for each constant, generate the appropriate code
		Vector constants = helper.constantsOfClass(_class);
		for (Enumeration e = constants.elements() ; e.hasMoreElements() ; ) {
			Constant constant = (Constant) e.nextElement();
			constantTemplate(constant);
		}
		
		//for each instance level attribute of this class, generate the appropriate code
		Vector attributes_d = helper.attributsForClass(_class , ScopeKind.instance_level , true );
		for (Enumeration e = attributes_d.elements() ; e.hasMoreElements() ; ) {
			MofAttribute att = (MofAttribute) e.nextElement();
			attributeTemplate(att);
		}
		
		Vector attributes_nd = helper.attributsForClass(_class , ScopeKind.instance_level , false );
		for (Enumeration e = attributes_nd.elements() ; e.hasMoreElements() ; ) {
			MofAttribute att = (MofAttribute) e.nextElement();
			attributeTemplate(att);
		}
		
		
		//for each instance level reference of this class, generate the appropriate code if public
		Vector references = helper.referencesForClass(_class);
		for (Enumeration e = references.elements() ; e.hasMoreElements() ; ) {
			Reference ref = (Reference) e.nextElement();
			if ((ref.scope().equals(ScopeKind.instance_level)) &&(ref.visibility().equals(VisibilityKind.public_vis))) {
				referenceTemplate(ref);
			}
		}

		//for each operation, generate the appropriate code if public_vis
		Vector operations = helper.operationsForClass(_class , ScopeKind.instance_level);
		for (Enumeration e = operations.elements() ; e.hasMoreElements() ; ) {
			Operation ope = (Operation) e.nextElement();
			if (ope.visibility().equals(VisibilityKind.public_vis)) operationTemplate(ope);
		}
		
		//end
		out.println("}");//end of interface
		
	}

}
