/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fr.lip6.src.modfact.generation.jmi;

import java.util.Enumeration;
import java.util.Vector;


import org.omg.CORBA.TCKind;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.TypeCodePackage.BadKind;
import org.omg.CORBA.TypeCodePackage.Bounds;
import org.omg.mof.Model.GeneralizableElement;
import org.omg.mof.Model.PackageHelper;
import org.omg.mof.Model.VisibilityKind;
import org.omg.mof.Reflective.MofError;



/**
 * 
 */
public class PackageInterfaceTemplate extends CommonInterfaceTemplate {
	
	public void packageTemplate(org.omg.mof.Model.Package _package) throws MofError , BadKind , Bounds {
		//Tag Import todo
		
		javadocsTemplate(_package);
		
		out.print("public interface "+helper.jmiPackageName(_package)+"Package extends ");
		GeneralizableElement[] generalizableElement = _package.supertypes();
		if (generalizableElement.length==0) out.println("javax.jmi.reflect.RefPackage");
		else {
			for (int i=0 ; i < generalizableElement.length ; i++) {
				out.print(helper.jmiQualifiedPackageName(PackageHelper.narrow(generalizableElement[i])));
				if (i!=generalizableElement.length) out.print(",");
			}
		}
		out.println(" { ");

		//for each imported package where:
		//	isClustered == ture and
		//	Import.visibility == public_vis
		//  importedNamespace.visibility == public_vis
		Vector imports = helper.importsOfPackage(_package);
		for (Enumeration e=imports.elements() ; e.hasMoreElements() ; ) {
			org.omg.mof.Model.Import imported = (org.omg.mof.Model.Import) e.nextElement();
			if (VisibilityKind.public_vis.equals(imported.visibility())  && imported.is_clustered()) {
				org.omg.mof.Model.Package importedPackage = org.omg.mof.Model.PackageHelper.narrow(imported.imported_namespace());
				if (importedPackage.visibility().equals(VisibilityKind.public_vis)) {
					out.println("\tpublic "+helper.jmiQualifiedPackageName(importedPackage)+"Package get"+helper.jmiImportName(imported)+"();");
				}
			}
		}

		//for each contained package where visibility = public_vis
		Vector containedPackages = helper.packagesOfPackage(_package);
		for (Enumeration e = containedPackages.elements() ; e.hasMoreElements() ; ) {
			org.omg.mof.Model.Package containedPackage = (org.omg.mof.Model.Package) e.nextElement();
			if (containedPackage.visibility().equals(VisibilityKind.public_vis)) {
				out.println("\tpublic "+helper.jmiPackageName(containedPackage)+"Package get"+helper.jmiPackageName(containedPackage)+"();");
			}
		}
		
		//for each contained class where visibility = public_vis
		Vector containedClasses = helper.classesOfPackage(_package);
		for (Enumeration e = containedClasses.elements() ; e.hasMoreElements() ; ) {
			org.omg.mof.Model.Class containedClass = (org.omg.mof.Model.Class) e.nextElement();
			if (containedClass.visibility().equals(VisibilityKind.public_vis)) {
				out.println("\tpublic "+helper.jmiClassName(containedClass)+"Class get"+helper.jmiClassName(containedClass)+"();");
			}
		}
		
		//for each contained association where visibility = public_vis
		Vector containedAssociations = helper.associationsOfPackage(_package);
		for (Enumeration e = containedAssociations.elements() ; e.hasMoreElements() ; ) {
			org.omg.mof.Model.Association containedAssociation = (org.omg.mof.Model.Association) e.nextElement();
			if (containedAssociation.visibility().equals(VisibilityKind.public_vis)) {
				out.println("\tpublic "+helper.jmiAssociationName(containedAssociation)+"Class get"+helper.jmiAssociationName(containedAssociation)+"();");
			}
		}
		
		//for each StructType directly contained by the package
		//for each EnumType directly contained by the package
		org.omg.mof.Model.ModelElement[] containedElements = _package.contents(); 
		for (int i=0 ; i < containedElements.length ; i++) {
			if (containedElements[i]._is_a(org.omg.mof.Model.DataTypeHelper.id())) {
				org.omg.mof.Model.DataType containedDataType = org.omg.mof.Model.DataTypeHelper.narrow(containedElements[i]);
				TypeCode tc = containedDataType.type_code();
				if (tc.kind().equals(TCKind.tk_struct)) {
					out.print("\tpublic "+helper.jmiTypeCodeName(tc)+" create"+helper.jmiTypeCodeName(tc)+" (");
					for (int j=0 ; j < tc.member_count() ; j++) {
						out.print(helper.jmiTypeCodeName(tc.member_type(j)));//attribute type
						out.print(" ");
						out.print(tc.member_name(j));//attribute name
						if (j!=tc.member_count()) out.print(" , ");
					}
					out.println(") throws javax.jmi.reflect.JmiException;");
				}
				else if (tc.kind().equals(TCKind.tk_enum)) {
					out.println("\tpublic "+helper.jmiTypeCodeName(tc)+" create"+helper.jmiTypeCodeName(tc)+" (String value) throws javax.jmi.reflect.JmiException;");
				}
			}
		}
		
		out.println("}");			
	}
}

