/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.services.pss.common.lib;


/**
 * CatalogBase is a base class for Catalog.
 * It implements the org.objectweb.openccm.services.pss.api.Catalog interface.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public abstract class CatalogBase
              extends org.omg.CORBA.LocalObject
           implements org.objectweb.openccm.services.pss.common.api.CatalogBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     * Catalog ID.
     */
    private int _id;

    /**
     * Connector reference.
     */
    private org.objectweb.openccm.services.pss.common.api.Connector _connector;

    /**
     * Catalog access mode, i.e. READ_ONLY or READ_WRTIE.
     */
    private short _access_mode;

    /**
     * Managed storage home incarnations.
     */
    protected java.util.Map _storage_home_incarnations;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     *
     * @param id - The catalog id.
     */
    public CatalogBase(int id)
    {
        _id = id;
        _storage_home_incarnations = new java.util.HashMap();
        _connector = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for org.omg.CosPersistentState.Catalog
    //
    // ==================================================================

    /**
     * Read accessor for access_mode attribute.
     *
     * @return the attribute value.
     */
    public short
    access_mode()
    {
        return _access_mode;
    }

    /**
     * Find a storage home incarnation from its id.
     */
    public org.omg.CosPersistentState.StorageHomeBase
    find_storage_home(String storage_home_id)
    throws org.omg.CosPersistentState.NotFound
    {
        org.objectweb.openccm.services.pss.common.api.StorageHomeBase sh = null;

        sh = (org.objectweb.openccm.services.pss.common.api.StorageHomeBase) _storage_home_incarnations.get(storage_home_id);

        if (sh == null)
        {
            java.lang.Class clazz = null;

            clazz = _connector.get_storage_home_factory(storage_home_id);
            if (clazz == null)
            {
                throw new org.omg.CosPersistentState.NotFound();
            }

            try
            {
                sh = (org.objectweb.openccm.services.pss.common.api.StorageHomeBase) clazz.newInstance();
                sh.initialize(this);
                register_storage_home_incarnation(storage_home_id, sh);
            }catch(Exception ex){
                ex.printStackTrace();
                throw new org.omg.CORBA.PERSIST_STORE("Problem in find_storage_home");
            }
        }

        return sh;
    }

    /**
     * Retrieve a storage object with its pid.
     */
    public abstract java.lang.Object
    find_by_pid(byte[] the_pid)
    throws org.omg.CosPersistentState.NotFound;

    /**
     * Operation flush
     */
    public abstract void
    flush();

    /**
     * Operation refresh
     */
    public abstract void
    refresh();

    /**
     * Operation free_all
     */
    public abstract void
    free_all();

    /**
     * Operation close
     */
    public abstract void
    close();

    // ==================================================================
    //
    // Methods for org.objectweb.openccm.services.pss.common.api.CatalogBase
    //
    // ==================================================================

    /**
     * Set the reference to the connector.
     *
     * @param connector - The connector to set.
     */
    public void
    set_connector(org.objectweb.openccm.services.pss.common.api.Connector connector)
    {
        _connector = connector;
    }

    /**
     * Get the reference to the connector.
     *
     * @return The associated connector.
     */
    public org.objectweb.openccm.services.pss.common.api.Connector
    get_connector()
    {
        return _connector;
    }

    /**
     * Get the catalog ID.
     *
     * @return The associated ID.
     */
    public int
    get_id()
    {
        return _id;
    }

   /**
     * Register a storage home incarnation.
     *
     * @param storage_home_id - The storage home id.
     * @param storage_home - The binding class.
     */
    public void
    register_storage_home_incarnation(String storage_home_id,
                                      org.omg.CosPersistentState.StorageHomeBase storage_home)
    {
        _storage_home_incarnations.put(storage_home_id, storage_home);
    }

    /**
     * Create a new storage_object incarnation.
     *
     * @param storage_type_id - The storage type id.
     *
     * @return A new storage object instance.
     */
    public org.objectweb.openccm.services.pss.common.api.StorageObject
    create_storage_object(String storage_type_id)
    throws org.omg.CosPersistentState.NotFound
    {
        java.lang.Class clazz = null;

        clazz = get_connector().get_storage_object_factory(storage_type_id);
        if (clazz == null)
        {
            throw new org.omg.CosPersistentState.NotFound();
        }

        try
        {
            return (org.objectweb.openccm.services.pss.common.api.StorageObject) clazz.newInstance();
        }catch(Exception ex){
            ex.printStackTrace();
            throw new org.omg.CORBA.PERSIST_STORE("Problem in create_storage_object");
        }
    }
}
